﻿using System.Collections;

namespace Mono.ILASM
{
	public class Assembly : IDeclSecurityTarget, ICustomAttrTarget
	{
		DeclSecurity decl_sec;
		ArrayList customattr_list;

		string name;
		byte[] public_key;
		int major_version;
		int minor_version;
		int build_version;
		int revision_version;
		string locale;
		int hash_algorithm;
		PEAPI.AssemAttr attr;

		public Assembly(string name)
		{
			this.name = name;
		}

		public string Name => name;

		public DeclSecurity DeclSecurity
		{
			get
			{
				if (decl_sec == null)
					decl_sec = new DeclSecurity();
				return decl_sec;
			}
		}

		public void SetVersion(int major, int minor, int build, int revision)
		{
			this.major_version = major;
			this.minor_version = minor;
			this.build_version = build;
			this.revision_version = revision;
		}

		public void SetLocale(string locale)
		{
			this.locale = locale;
		}

		public void SetHashAlgorithm(int algorithm)
		{
			hash_algorithm = algorithm;
		}

		public void SetPublicKey(byte[] public_key)
		{
			this.public_key = public_key;
		}

		public void SetAssemblyAttr(PEAPI.AssemAttr attr)
		{
			this.attr = attr;
		}

		public void AddCustomAttribute(CustomAttr customattr)
		{
			if (customattr_list == null)
				customattr_list = new ArrayList();

			customattr_list.Add(customattr);
		}

		public void Resolve(CodeGen code_gen, PEAPI.Assembly asm)
		{
			if (customattr_list != null)
				foreach (CustomAttr customattr in customattr_list)
				{
					customattr.AddTo(code_gen, asm);
				}

			if (decl_sec != null)
				decl_sec.AddTo(code_gen, asm);

			asm.AddAssemblyInfo(major_version,
					minor_version, build_version,
					revision_version, public_key,
					(uint)hash_algorithm, locale);

			asm.AddAssemblyAttr(attr);
		}
	}
}
