﻿using System;
using System.Collections;

namespace Mono.ILASM
{
	/// <summary>
	/// Definition of a parameter passed to a method
	/// </summary>
	public class ParamDef : ICustomAttrTarget
	{
		PEAPI.ParamAttr attr;
		string name;
		BaseTypeRef typeref;
		bool is_defined;
		PEAPI.Param peapi_param;
		PEAPI.Constant defval;
		ArrayList customattr_list;
		PEAPI.NativeType native_type;

		public static readonly ParamDef Ellipsis = new ParamDef(new PEAPI.ParamAttr(), "ELLIPSIS", null);

		public ParamDef(PEAPI.ParamAttr attr, string name,
						BaseTypeRef typeref)
		{
			this.attr = attr;
			this.name = name;
			this.typeref = typeref;
			is_defined = false;
			defval = null;
		}

		public void AddDefaultValue(PEAPI.Constant cVal)
		{
			defval = cVal;
		}

		public void AddCustomAttribute(CustomAttr customattr)
		{
			if (customattr_list == null)
				customattr_list = new ArrayList();

			customattr_list.Add(customattr);
		}

		public void AddMarshalInfo(PEAPI.NativeType native_type)
		{
			this.native_type = native_type;
		}

		public BaseTypeRef Type => typeref;

		public string TypeName => typeref.FullName;

		public string Name => name;

		public PEAPI.Param PeapiParam => peapi_param;

		public bool IsSentinel()
		{
			return (typeref is SentinelTypeRef && this != Ellipsis);
		}

		public void Define(CodeGen code_gen)
		{
			if (is_defined)
				return;

			typeref.Resolve(code_gen);

			peapi_param = new PEAPI.Param(attr,
							name, typeref.PeapiType);
			if (defval != null)
			{
				peapi_param.AddDefaultValue(defval);
			}

			if (customattr_list != null)
				foreach (CustomAttr customattr in customattr_list)
					customattr.AddTo(code_gen, peapi_param);

			if (native_type != null)
				peapi_param.AddMarshallInfo(native_type);

			is_defined = true;
		}
	}
}
