﻿using System;
using System.IO;
using System.Collections;
using System.Collections.Generic;
using System.Globalization;
using System.Reflection;
using System.Security;
using System.Security.Permissions;

using MIPermission = Mono.ILASM.Permission;
using MIPermissionSet = Mono.ILASM.PermissionSet;

using PEAPI;


namespace Mono.ILASM
{
	public partial class ILParser
	{
		const int yacc_verbose_flag = 0;

		class NameValuePair
		{
			public string Name;
			public object Value;

			public NameValuePair(string name, object value)
			{
				this.Name = name;
				this.Value = value;
			}
		};

		class PermPair
		{
			public PEAPI.SecurityAction sec_action;
			public object perm;

			public PermPair(PEAPI.SecurityAction sec_action, object perm)
			{
				this.sec_action = sec_action;
				this.perm = perm;
			}
		};

		public ILParser(CodeGen codegen, ILTokenizer tokenizer)
		{
			this.codegen = codegen;
			this.tokenizer = tokenizer;
		}

		CodeGen codegen;
		public CodeGen CodeGen => codegen;

		bool is_value_class;
		bool is_enum_class;
		bool pinvoke_info;
		string pinvoke_mod;
		string pinvoke_meth;
		PEAPI.PInvokeAttr pinvoke_attr;
		ILTokenizer tokenizer;
		KeyValuePair<string, TypeAttr> current_extern;

		public bool CheckSecurityActionValidity(System.Security.Permissions.SecurityAction action, bool for_assembly)
		{
#pragma warning disable 618
			if ((action == System.Security.Permissions.SecurityAction.RequestMinimum ||
					action == System.Security.Permissions.SecurityAction.RequestOptional ||
					action == System.Security.Permissions.SecurityAction.RequestRefuse) && !for_assembly)
			{
				Report.Warning(String.Format("System.Security.Permissions.SecurityAction '{0}' is not valid for this declaration", action));
				return false;
			}
#pragma warning restore 618
			return true;
		}

		public void AddSecDecl(object perm, bool for_assembly)
		{
			PermPair pp = perm as PermPair;

			if (pp == null)
			{
				MIPermissionSet ps_20 = (MIPermissionSet)perm;
				codegen.AddPermission(ps_20.SecurityAction, ps_20);
				return;
			}

			if (!CheckSecurityActionValidity((System.Security.Permissions.SecurityAction)pp.sec_action, for_assembly))
				Report.Error(String.Format("Invalid security action : {0}", pp.sec_action));

			codegen.AddPermission(pp.sec_action, pp.perm);
		}

		public object ClassRefToObject(object class_ref, object val)
		{
			ExternTypeRef etr = class_ref as ExternTypeRef;
			if (etr == null)
				/* FIXME: report error? can be PrimitiveTypeRef or TypeRef */
				return null;

			System.Type t = etr.GetReflectedType();
			return t.IsEnum ? Enum.Parse(t, String.Format("{0}", val)) : val;
		}

		/* Converts a type_spec to a corresponding PermPair */
		PermPair TypeSpecToPermPair(object action, object type_spec, ArrayList pairs)
		{
			ExternTypeRef etr = type_spec as ExternTypeRef;
			if (etr == null)
				/* FIXME: could be PrimitiveTypeRef or TypeRef
						  Report what error? */
				return null;

			System.Type t = etr.GetReflectedType();
			object obj = Activator.CreateInstance(t, new object[] { (System.Security.Permissions.SecurityAction)(short)action });

			if (pairs != null)
				foreach (NameValuePair pair in pairs)
				{
					PropertyInfo pi = t.GetProperty(pair.Name);
					pi.SetValue(obj, pair.Value, null);
				}

			IPermission iper = (IPermission)t.GetMethod("CreatePermission").Invoke(obj, null);
			return new PermPair((PEAPI.SecurityAction)action, iper);
		}

		BaseTypeRef GetTypeRef(BaseTypeRef b)
		{
			//FIXME: Caching required..
			return b.Clone();
		}
	};
}
