﻿using PEAPI;
using System;
using System.Collections.Generic;
using System.Collections;
using System.Reflection.Emit;

namespace Mono.ILASM
{
	public sealed class TokenLookup : Dictionary<String, ILToken>
	{
		public TokenLookup()
			: base(300, StringComparer.Ordinal)
		{
		}
	};

	public static class ILTables
	{
		public readonly static TokenLookup Keywords = new TokenLookup
		{
			{ "at",                                  	   /**/ new ILToken(Token.K_AT, "at") },
			{ "as",                                  	   /**/ new ILToken(Token.K_AS, "as") },
			{ "implicitcom",                         	   /**/ new ILToken(Token.K_IMPLICITCOM, "implicitcom") },
			{ "implicitres",                         	   /**/ new ILToken(Token.K_IMPLICITRES, "implicitres") },
			{ "noappdomain",                         	   /**/ new ILToken(Token.K_NOAPPDOMAIN, "noappdomain") },
			{ "noprocess",                           	   /**/ new ILToken(Token.K_NOPROCESS, "noprocess") },
			{ "nomachine",                           	   /**/ new ILToken(Token.K_NOMACHINE, "nomachine") },
			{ "extern",                              	   /**/ new ILToken(Token.K_EXTERN, "extern") },
			{ "instance",                            	   /**/ new ILToken(Token.K_INSTANCE, "instance") },
			{ "explicit",                            	   /**/ new ILToken(Token.K_EXPLICIT, "explicit") },
			{ "default",                             	   /**/ new ILToken(Token.K_DEFAULT, "default") },
			{ "vararg",                              	   /**/ new ILToken(Token.K_VARARG, "vararg") },
			{ "unmanaged",                           	   /**/ new ILToken(Token.K_UNMANAGED, "unmanaged") },
			{ "cdecl",                               	   /**/ new ILToken(Token.K_CDECL, "cdecl") },
			{ "stdcall",                             	   /**/ new ILToken(Token.K_STDCALL, "stdcall") },
			{ "thiscall",                            	   /**/ new ILToken(Token.K_THISCALL, "thiscall") },
			{ "fastcall",                            	   /**/ new ILToken(Token.K_FASTCALL, "fastcall") },
			{ "marshal",                             	   /**/ new ILToken(Token.K_MARSHAL, "marshal") },
			{ "in",                                  	   /**/ new ILToken(Token.K_IN, "in") },
			{ "out",                                 	   /**/ new ILToken(Token.K_OUT, "out") },
			{ "opt",                                 	   /**/ new ILToken(Token.K_OPT, "opt") },
			// Not a keyword according to ilasm 1.1
			//{ "lcid",                              	   /**/ new ILToken(Token.K_LCID, "lcid") },
			//{ "retval",                            	   /**/ new ILToken(Token.K_RETVAL, "retval") },
			{ "static",                              	   /**/ new ILToken(Token.K_STATIC, "static") },
			{ "public",                              	   /**/ new ILToken(Token.K_PUBLIC, "public") },
			{ "private",                             	   /**/ new ILToken(Token.K_PRIVATE, "private") },
			{ "family",                              	   /**/ new ILToken(Token.K_FAMILY, "family") },
			{ "initonly",                            	   /**/ new ILToken(Token.K_INITONLY, "initonly") },
			{ "rtspecialname",                       	   /**/ new ILToken(Token.K_RTSPECIALNAME, "rtspecialname") },
			{ "specialname",                         	   /**/ new ILToken(Token.K_SPECIALNAME, "specialname") },
			{ "assembly",                            	   /**/ new ILToken(Token.K_ASSEMBLY, "assembly") },
			{ "famandassem",                         	   /**/ new ILToken(Token.K_FAMANDASSEM, "famandassem") },
			{ "famorassem",                          	   /**/ new ILToken(Token.K_FAMORASSEM, "famorassem") },
			{ "privatescope",                        	   /**/ new ILToken(Token.K_PRIVATESCOPE, "privatescope") },
			{ "literal",                             	   /**/ new ILToken(Token.K_LITERAL, "literal") },
			{ "notserialized",                       	   /**/ new ILToken(Token.K_NOTSERIALIZED, "notserialized") },
			{ "value",                               	   /**/ new ILToken(Token.K_VALUE, "value") },
			{ "not_in_gc_heap",                      	   /**/ new ILToken(Token.K_NOT_IN_GC_HEAP, "not_in_gc_heap") },
			{ "interface",                           	   /**/ new ILToken(Token.K_INTERFACE, "interface") },
			{ "sealed",                              	   /**/ new ILToken(Token.K_SEALED, "sealed") },
			{ "abstract",                            	   /**/ new ILToken(Token.K_ABSTRACT, "abstract") },
			{ "auto",                                	   /**/ new ILToken(Token.K_AUTO, "auto") },
			{ "sequential",                          	   /**/ new ILToken(Token.K_SEQUENTIAL, "sequential") },
			{ "ansi",                                	   /**/ new ILToken(Token.K_ANSI, "ansi") },
			{ "unicode",                             	   /**/ new ILToken(Token.K_UNICODE, "unicode") },
			{ "autochar",                            	   /**/ new ILToken(Token.K_AUTOCHAR, "autochar") },
			{ "bestfit",                             	   /**/ new ILToken(Token.K_BESTFIT, "bestfit") },
			{ "charmaperror",                        	   /**/ new ILToken(Token.K_CHARMAPERROR, "charmaperror") },
			{ "import",                              	   /**/ new ILToken(Token.K_IMPORT, "import") },
			{ "serializable",                        	   /**/ new ILToken(Token.K_SERIALIZABLE, "serializable") },
			{ "nested",                              	   /**/ new ILToken(Token.K_NESTED, "nested") },
			{ "lateinit",                            	   /**/ new ILToken(Token.K_LATEINIT, "lateinit") },
			{ "extends",                             	   /**/ new ILToken(Token.K_EXTENDS, "extends") },
			{ "implements",                          	   /**/ new ILToken(Token.K_IMPLEMENTS, "implements") },
			{ "final",                               	   /**/ new ILToken(Token.K_FINAL, "final") },
			{ "virtual",                             	   /**/ new ILToken(Token.K_VIRTUAL, "virtual") },
			{ "hidebysig",                           	   /**/ new ILToken(Token.K_HIDEBYSIG, "hidebysig") },
			{ "newslot",                             	   /**/ new ILToken(Token.K_NEWSLOT, "newslot") },
			{ "unmanagedexp",                        	   /**/ new ILToken(Token.K_UNMANAGEDEXP, "unmanagedexp") },
			{ "pinvokeimpl",                         	   /**/ new ILToken(Token.K_PINVOKEIMPL, "pinvokeimpl") },
			{ "nomangle",                            	   /**/ new ILToken(Token.K_NOMANGLE, "nomangle") },
			{ "ole",                                 	   /**/ new ILToken(Token.K_OLE, "ole") },
			{ "lasterr",                             	   /**/ new ILToken(Token.K_LASTERR, "lasterr") },
			{ "winapi",                              	   /**/ new ILToken(Token.K_WINAPI, "winapi") },
			{ "native",                              	   /**/ new ILToken(Token.K_NATIVE, "native") },
			{ "il",                                  	   /**/ new ILToken(Token.K_IL, "il") },
			{ "cil",                                 	   /**/ new ILToken(Token.K_CIL, "cil") },
			{ "optil",                               	   /**/ new ILToken(Token.K_OPTIL, "optil") },
			{ "managed",                             	   /**/ new ILToken(Token.K_MANAGED, "managed") },
			{ "forwardref",                          	   /**/ new ILToken(Token.K_FORWARDREF, "forwardref") },
			{ "runtime",                             	   /**/ new ILToken(Token.K_RUNTIME, "runtime") },
			{ "internalcall",                        	   /**/ new ILToken(Token.K_INTERNALCALL, "internalcall") },
			{ "synchronized",                        	   /**/ new ILToken(Token.K_SYNCHRONIZED, "synchronized") },
			{ "noinlining",                          	   /**/ new ILToken(Token.K_NOINLINING, "noinlining") },
			{ "nooptimization",                      	   /**/ new ILToken(Token.K_NOOPTIMIZATION, "nooptimization") },
			{ "custom",                              	   /**/ new ILToken(Token.K_CUSTOM, "custom") },
			{ "fixed",                               	   /**/ new ILToken(Token.K_FIXED, "fixed") },
			{ "sysstring",                           	   /**/ new ILToken(Token.K_SYSSTRING, "sysstring") },
			{ "array",                               	   /**/ new ILToken(Token.K_ARRAY, "array") },
			{ "variant",                             	   /**/ new ILToken(Token.K_VARIANT, "variant") },
			{ "currency",                            	   /**/ new ILToken(Token.K_CURRENCY, "currency") },
			{ "syschar",                             	   /**/ new ILToken(Token.K_SYSCHAR, "syschar") },
			{ "void",                                	   /**/ new ILToken(Token.K_VOID, "void") },
			{ "bool",                                	   /**/ new ILToken(Token.K_BOOL, "bool") },
			{ "int8",                                	   /**/ new ILToken(Token.K_INT8, "int8") },
			{ "int16",                               	   /**/ new ILToken(Token.K_INT16, "int16") },
			{ "int32",                               	   /**/ new ILToken(Token.K_INT32, "int32") },
			{ "int64",                               	   /**/ new ILToken(Token.K_INT64, "int64") },
			{ "float32",                             	   /**/ new ILToken(Token.K_FLOAT32, "float32") },
			{ "float64",                             	   /**/ new ILToken(Token.K_FLOAT64, "float64") },
			{ "error",                               	   /**/ new ILToken(Token.K_ERROR, "error") },
			{ "unsigned",                            	   /**/ new ILToken(Token.K_UNSIGNED, "unsigned") },
			{ "uint",                                	   /**/ new ILToken(Token.K_UINT, "uint") },
			{ "uint8",                               	   /**/ new ILToken(Token.K_UINT8, "uint8") },
			{ "uint16",                              	   /**/ new ILToken(Token.K_UINT16, "uint16") },
			{ "uint32",                              	   /**/ new ILToken(Token.K_UINT32, "uint32") },
			{ "uint64",                              	   /**/ new ILToken(Token.K_UINT64, "uint64") },
			{ "decimal",                             	   /**/ new ILToken(Token.K_DECIMAL, "decimal") },
			{ "date",                                	   /**/ new ILToken(Token.K_DATE, "date") },
			{ "bstr",                                	   /**/ new ILToken(Token.K_BSTR, "bstr") },
			{ "lpstr",                               	   /**/ new ILToken(Token.K_LPSTR, "lpstr") },
			{ "lpwstr",                              	   /**/ new ILToken(Token.K_LPWSTR, "lpwstr") },
			{ "lptstr",                              	   /**/ new ILToken(Token.K_LPTSTR, "lptstr") },
			{ "vbbyrefstr",                          	   /**/ new ILToken(Token.K_VBBYREFSTR, "vbbyrefstr") },
			{ "objectref",                           	   /**/ new ILToken(Token.K_OBJECTREF, "objectref") },
			{ "iunknown",                            	   /**/ new ILToken(Token.K_IUNKNOWN, "iunknown") },
			{ "idispatch",                           	   /**/ new ILToken(Token.K_IDISPATCH, "idispatch") },
			{ "struct",                              	   /**/ new ILToken(Token.K_STRUCT, "struct") },
			{ "safearray",                           	   /**/ new ILToken(Token.K_SAFEARRAY, "safearray") },
			{ "int",                                 	   /**/ new ILToken(Token.K_INT, "int") },
			{ "byvalstr",                            	   /**/ new ILToken(Token.K_BYVALSTR, "byvalstr") },
			{ "tbstr",                               	   /**/ new ILToken(Token.K_TBSTR, "tbstr") },
			{ "lpvoid",                              	   /**/ new ILToken(Token.K_LPVOID, "lpvoid") },
			{ "any",                                 	   /**/ new ILToken(Token.K_ANY, "any") },
			{ "float",                               	   /**/ new ILToken(Token.K_FLOAT, "float") },
			{ "lpstruct",                            	   /**/ new ILToken(Token.K_LPSTRUCT, "lpstruct") },
			{ "null",                                	   /**/ new ILToken(Token.K_NULL, "null") },
			//{ "ptr",                               	   /**/ new ILToken(Token.K_PTR, "ptr") },
			{ "vector",                              	   /**/ new ILToken(Token.K_VECTOR, "vector") },
			{ "hresult",                             	   /**/ new ILToken(Token.K_HRESULT, "hresult") },
			{ "carray",                              	   /**/ new ILToken(Token.K_CARRAY, "carray") },
			{ "userdefined",                         	   /**/ new ILToken(Token.K_USERDEFINED, "userdefined") },
			{ "record",                              	   /**/ new ILToken(Token.K_RECORD, "record") },
			{ "filetime",                            	   /**/ new ILToken(Token.K_FILETIME, "filetime") },
			{ "blob",                                	   /**/ new ILToken(Token.K_BLOB, "blob") },
			{ "stream",                              	   /**/ new ILToken(Token.K_STREAM, "stream") },
			{ "storage",                             	   /**/ new ILToken(Token.K_STORAGE, "storage") },
			{ "streamed_object",                     	   /**/ new ILToken(Token.K_STREAMED_OBJECT, "streamed_object") },
			{ "stored_object",                       	   /**/ new ILToken(Token.K_STORED_OBJECT, "stored_object") },
			{ "blob_object",                         	   /**/ new ILToken(Token.K_BLOB_OBJECT, "blob_object") },
			{ "cf",                                  	   /**/ new ILToken(Token.K_CF, "cf") },
			{ "clsid",                               	   /**/ new ILToken(Token.K_CLSID, "clsid") },
			{ "method",                              	   /**/ new ILToken(Token.K_METHOD, "method") },
			{ "class",                               	   /**/ new ILToken(Token.K_CLASS, "class") },
			{ "pinned",                              	   /**/ new ILToken(Token.K_PINNED, "pinned") },
			{ "modreq",                              	   /**/ new ILToken(Token.K_MODREQ, "modreq") },
			{ "modopt",                              	   /**/ new ILToken(Token.K_MODOPT, "modopt") },
			{ "typedref",                            	   /**/ new ILToken(Token.K_TYPEDREF, "typedref") },
			{ "property",                            	   /**/ new ILToken(Token.K_PROPERTY, "property") },
			{ "type",                                	   /**/ new ILToken(Token.K_TYPE, "type") },
			{ "refany",                              	   /**/ new ILToken(Token.K_TYPEDREF, "typedref") },
			{ "wchar",                               	   /**/ new ILToken(Token.K_WCHAR, "wchar") },
			{ "char",                                	   /**/ new ILToken(Token.K_CHAR, "char") },
			{ "fromunmanaged",                       	   /**/ new ILToken(Token.K_FROMUNMANAGED, "fromunmanaged") },
			{ "callmostderived",                     	   /**/ new ILToken(Token.K_CALLMOSTDERIVED, "callmostderived") },
			{ "bytearray",                           	   /**/ new ILToken(Token.K_BYTEARRAY, "bytearray") },
			{ "with",                                	   /**/ new ILToken(Token.K_WITH, "with") },
			{ "init",                                	   /**/ new ILToken(Token.K_INIT, "init") },
			{ "to",                                  	   /**/ new ILToken(Token.K_TO, "to") },
			{ "catch",                               	   /**/ new ILToken(Token.K_CATCH, "catch") },
			{ "filter",                              	   /**/ new ILToken(Token.K_FILTER, "filter") },
			{ "finally",                             	   /**/ new ILToken(Token.K_FINALLY, "finally") },
			{ "fault",                               	   /**/ new ILToken(Token.K_FAULT, "fault") },
			{ "handler",                             	   /**/ new ILToken(Token.K_HANDLER, "handler") },
			{ "tls",                                 	   /**/ new ILToken(Token.K_TLS, "tls") },
			{ "field",                               	   /**/ new ILToken(Token.K_FIELD, "field") },
			{ "request",                             	   /**/ new ILToken(Token.K_REQUEST, "request") },
			{ "demand",                              	   /**/ new ILToken(Token.K_DEMAND, "demand") },
			{ "assert",                              	   /**/ new ILToken(Token.K_ASSERT, "assert") },
			{ "deny",                                	   /**/ new ILToken(Token.K_DENY, "deny") },
			{ "permitonly",                          	   /**/ new ILToken(Token.K_PERMITONLY, "permitonly") },
			{ "linkcheck",                           	   /**/ new ILToken(Token.K_LINKCHECK, "linkcheck") },
			{ "inheritcheck",                        	   /**/ new ILToken(Token.K_INHERITCHECK, "inheritcheck") },
			{ "reqmin",                              	   /**/ new ILToken(Token.K_REQMIN, "reqmin") },
			{ "reqopt",                              	   /**/ new ILToken(Token.K_REQOPT, "reqopt") },
			{ "reqrefuse",                           	   /**/ new ILToken(Token.K_REQREFUSE, "reqrefuse") },
			{ "prejitgrant",                         	   /**/ new ILToken(Token.K_PREJITGRANT, "prejitgrant") },
			{ "prejitdeny",                          	   /**/ new ILToken(Token.K_PREJITDENY, "prejitdeny") },
			{ "noncasdemand",                        	   /**/ new ILToken(Token.K_NONCASDEMAND, "noncasdemand") },
			{ "noncaslinkdemand",                    	   /**/ new ILToken(Token.K_NONCASLINKDEMAND, "noncaslinkdemand") },
			{ "noncasinheritance",                   	   /**/ new ILToken(Token.K_NONCASINHERITANCE, "noncasinheritance") },
			{ "readonly",                            	   /**/ new ILToken(Token.K_READONLY, "readonly") },
			{ "nometadata",                          	   /**/ new ILToken(Token.K_NOMETADATA, "nometadata") },
			{ "algorithm",                           	   /**/ new ILToken(Token.K_ALGORITHM, "algorithm") },
			{ "fullorigin",                          	   /**/ new ILToken(Token.K_FULLORIGIN, "fullorigin") },
			//{ "nan",                               	   /**/ new ILToken(Token.K_NAN, "nan") },
			//{ "inf",                               	   /**/ new ILToken(Token.K_INF, "inf") },
			//{ "publickey",                         	   /**/ new ILToken(Token.K_PUBLICKEY, "publickey") },
			{ "enablejittracking",                   	   /**/ new ILToken(Token.K_ENABLEJITTRACKING, "enablejittracking") },
			{ "disablejitoptimizer",                 	   /**/ new ILToken(Token.K_DISABLEJITOPTIMIZER, "disablejitoptimizer") },
			{ "retargetable",                        	   /**/ new ILToken(Token.K_RETARGETABLE, "retargetable") },
			{ "preservesig",                         	   /**/ new ILToken(Token.K_PRESERVESIG, "preservesig") },
			{ "beforefieldinit",                     	   /**/ new ILToken(Token.K_BEFOREFIELDINIT, "beforefieldinit") },
			{ "alignment",                           	   /**/ new ILToken(Token.K_ALIGNMENT, "alignment") },
			{ "nullref",                             	   /**/ new ILToken(Token.K_NULLREF, "nullref") },
			{ "valuetype",                           	   /**/ new ILToken(Token.K_VALUETYPE, "valuetype") },
			{ "compilercontrolled",                  	   /**/ new ILToken(Token.K_COMPILERCONTROLLED, "compilercontrolled") },
			{ "reqsecobj",                           	   /**/ new ILToken(Token.K_REQSECOBJ, "reqsecobj") },
			{ "enum",                                	   /**/ new ILToken(Token.K_ENUM, "enum") },
			{ "object",                              	   /**/ new ILToken(Token.K_OBJECT, "object") },
			{ "string",                              	   /**/ new ILToken(Token.K_STRING, "string") },
			{ "true",                                	   /**/ new ILToken(Token.K_TRUE, "true") },
			{ "false",                               	   /**/ new ILToken(Token.K_FALSE, "false") },
			{ "is",                                  	   /**/ new ILToken(Token.K_IS, "is") },
			{ "on",                                  	   /**/ new ILToken(Token.K_ON, "on") },
			{ "off",                                 	   /**/ new ILToken(Token.K_OFF, "off") },
			{ "strict",                              	   /**/ new ILToken(Token.K_STRICT, "strict") },
			{ "forwarder",                           	   /**/ new ILToken(Token.K_FORWARDER, "forwarder") },
			{ "legacy",                              	   /**/ new ILToken(Token.K_LEGACY, "legacy") },
			{ "library",                             	   /**/ new ILToken(Token.K_LIBRARY, "library") },
			{ "aggressiveinlining",                  	   /**/ new ILToken(Token.K_AGGRESSIVEINLINING, "aggressiveinlining") },
		};


		public readonly static TokenLookup Directives = new TokenLookup
		{
			{ ".addon",                        			   /**/ new ILToken(Token.D_ADDON, ".addon") },
			{ ".algorithm",                    			   /**/ new ILToken(Token.D_ALGORITHM, ".algorithm") },
			{ ".assembly",                     			   /**/ new ILToken(Token.D_ASSEMBLY, ".assembly") },
			{ ".backing",                      			   /**/ new ILToken(Token.D_BACKING, ".backing") },
			{ ".blob",                         			   /**/ new ILToken(Token.D_BLOB, ".blob") },
			{ ".capability",                   			   /**/ new ILToken(Token.D_CAPABILITY, ".capability") },
			{ ".cctor",                        			   /**/ new ILToken(Token.D_CCTOR, ".cctor") },
			{ ".class",                        			   /**/ new ILToken(Token.D_CLASS, ".class") },
			{ ".comtype",                      			   /**/ new ILToken(Token.D_COMTYPE, ".comtype") },
			{ ".config",                       			   /**/ new ILToken(Token.D_CONFIG, ".config") },
			{ ".imagebase",                    			   /**/ new ILToken(Token.D_IMAGEBASE, ".imagebase") },
			{ ".corflags",                     			   /**/ new ILToken(Token.D_CORFLAGS, ".corflags") },
			{ ".ctor",                         			   /**/ new ILToken(Token.D_CTOR, ".ctor") },
			{ ".custom",                       			   /**/ new ILToken(Token.D_CUSTOM, ".custom") },
			{ ".data",                         			   /**/ new ILToken(Token.D_DATA, ".data") },
			{ ".emitbyte",                     			   /**/ new ILToken(Token.D_EMITBYTE, ".emitbyte") },
			{ ".entrypoint",                   			   /**/ new ILToken(Token.D_ENTRYPOINT, ".entrypoint") },
			{ ".event",                        			   /**/ new ILToken(Token.D_EVENT, ".event") },
			{ ".exeloc",                       			   /**/ new ILToken(Token.D_EXELOC, ".exeloc") },
			{ ".export",                       			   /**/ new ILToken(Token.D_EXPORT, ".export") },
			{ ".field",                        			   /**/ new ILToken(Token.D_FIELD, ".field") },
			{ ".file",                         			   /**/ new ILToken(Token.D_FILE, ".file") },
			{ ".fire",                         			   /**/ new ILToken(Token.D_FIRE, ".fire") },
			{ ".get",                          			   /**/ new ILToken(Token.D_GET, ".get") },
			{ ".hash",                         			   /**/ new ILToken(Token.D_HASH, ".hash") },
			{ ".implicitcom",                  			   /**/ new ILToken(Token.D_IMPLICITCOM, ".implicitcom") },
			{ ".language",                     			   /**/ new ILToken(Token.D_LANGUAGE, ".language") },
			{ ".line",                         			   /**/ new ILToken(Token.D_LINE, ".line") },
			{ "#line",                         			   /**/ new ILToken(Token.D_XLINE, "#line") },
			{ ".locale",                       			   /**/ new ILToken(Token.D_LOCALE, ".locale") },
			{ ".locals",                       			   /**/ new ILToken(Token.D_LOCALS, ".locals") },
			{ ".manifestres",                  			   /**/ new ILToken(Token.D_MANIFESTRES, ".manifestres") },
			{ ".maxstack",                     			   /**/ new ILToken(Token.D_MAXSTACK, ".maxstack") },
			{ ".method",                       			   /**/ new ILToken(Token.D_METHOD, ".method") },
			{ ".mime",                         			   /**/ new ILToken(Token.D_MIME, ".mime") },
			{ ".module",                       			   /**/ new ILToken(Token.D_MODULE, ".module") },
			{ ".mresource",                    			   /**/ new ILToken(Token.D_MRESOURCE, ".mresource") },
			{ ".namespace",                    			   /**/ new ILToken(Token.D_NAMESPACE, ".namespace") },
			{ ".originator",                   			   /**/ new ILToken(Token.D_ORIGINATOR, ".originator") },
			{ ".os",                           			   /**/ new ILToken(Token.D_OS, ".os") },
			{ ".other",                        			   /**/ new ILToken(Token.D_OTHER, ".other") },
			{ ".override",                     			   /**/ new ILToken(Token.D_OVERRIDE, ".override") },
			{ ".pack",                         			   /**/ new ILToken(Token.D_PACK, ".pack") },
			{ ".param",                        			   /**/ new ILToken(Token.D_PARAM, ".param") },
			{ ".permission",                   			   /**/ new ILToken(Token.D_PERMISSION, ".permission") },
			{ ".permissionset",                			   /**/ new ILToken(Token.D_PERMISSIONSET, ".permissionset") },
			{ ".processor",                    			   /**/ new ILToken(Token.D_PROCESSOR, ".processor") },
			{ ".property",                     			   /**/ new ILToken(Token.D_PROPERTY, ".property") },
			{ ".publickey",                    			   /**/ new ILToken(Token.D_PUBLICKEY, ".publickey") },
			{ ".publickeytoken",               			   /**/ new ILToken(Token.D_PUBLICKEYTOKEN, ".publickeytoken") },
			{ ".removeon",                     			   /**/ new ILToken(Token.D_REMOVEON, ".removeon") },
			{ ".set",                          			   /**/ new ILToken(Token.D_SET, ".set") },
			{ ".size",                         			   /**/ new ILToken(Token.D_SIZE, ".size") },
			{ ".stackreserve",                 			   /**/ new ILToken(Token.D_STACKRESERVE, ".stackreserve") },
			{ ".subsystem",                    			   /**/ new ILToken(Token.D_SUBSYSTEM, ".subsystem") },
			{ ".title",                        			   /**/ new ILToken(Token.D_TITLE, ".title") },
			{ ".try",                          			   /**/ new ILToken(Token.D_TRY, ".try") },
			{ ".ver",                          			   /**/ new ILToken(Token.D_VER, ".ver") },
			{ ".vtable",                       			   /**/ new ILToken(Token.D_VTABLE, ".vtable") },
			{ ".vtentry",                      			   /**/ new ILToken(Token.D_VTENTRY, ".vtentry") },
			{ ".vtfixup",                      			   /**/ new ILToken(Token.D_VTFIXUP, ".vtfixup") },
			{ ".zeroinit",                     			   /**/ new ILToken(Token.D_ZEROINIT, ".zeroinit") },
		};

		public readonly static TokenLookup Instr = new TokenLookup
		{
			{ "nop",                                       /**/ new ILToken(Token.INSTR_NONE, Op.nop) },
			{ "break",                                     /**/ new ILToken(Token.INSTR_NONE, Op.breakOp) },
			{ "ldarg.0",                                   /**/ new ILToken(Token.INSTR_NONE, Op.ldarg_0) },
			{ "ldarg.1",                                   /**/ new ILToken(Token.INSTR_NONE, Op.ldarg_1) },
			{ "ldarg.2",                                   /**/ new ILToken(Token.INSTR_NONE, Op.ldarg_2) },
			{ "ldarg.3",                                   /**/ new ILToken(Token.INSTR_NONE, Op.ldarg_3) },
			{ "ldloc.0",                                   /**/ new ILToken(Token.INSTR_NONE, Op.ldloc_0) },
			{ "ldloc.1",                                   /**/ new ILToken(Token.INSTR_NONE, Op.ldloc_1) },
			{ "ldloc.2",                                   /**/ new ILToken(Token.INSTR_NONE, Op.ldloc_2) },
			{ "ldloc.3",                                   /**/ new ILToken(Token.INSTR_NONE, Op.ldloc_3) },
			{ "stloc.0",                                   /**/ new ILToken(Token.INSTR_NONE, Op.stloc_0) },
			{ "stloc.1",                                   /**/ new ILToken(Token.INSTR_NONE, Op.stloc_1) },
			{ "stloc.2",                                   /**/ new ILToken(Token.INSTR_NONE, Op.stloc_2) },
			{ "stloc.3",                                   /**/ new ILToken(Token.INSTR_NONE, Op.stloc_3) },
			{ "ldnull",                                    /**/ new ILToken(Token.INSTR_NONE, Op.ldnull) },
			{ "ldc.i4.m1",                                 /**/ new ILToken(Token.INSTR_NONE, Op.ldc_i4_m1) },
			{ "ldc.i4.M1",                                 /**/ new ILToken(Token.INSTR_NONE, Op.ldc_i4_m1) },
			{ "ldc.i4.0",                                  /**/ new ILToken(Token.INSTR_NONE, Op.ldc_i4_0) },
			{ "ldc.i4.1",                                  /**/ new ILToken(Token.INSTR_NONE, Op.ldc_i4_1) },
			{ "ldc.i4.2",                                  /**/ new ILToken(Token.INSTR_NONE, Op.ldc_i4_2) },
			{ "ldc.i4.3",                                  /**/ new ILToken(Token.INSTR_NONE, Op.ldc_i4_3) },
			{ "ldc.i4.4",                                  /**/ new ILToken(Token.INSTR_NONE, Op.ldc_i4_4) },
			{ "ldc.i4.5",                                  /**/ new ILToken(Token.INSTR_NONE, Op.ldc_i4_5) },
			{ "ldc.i4.6",                                  /**/ new ILToken(Token.INSTR_NONE, Op.ldc_i4_6) },
			{ "ldc.i4.7",                                  /**/ new ILToken(Token.INSTR_NONE, Op.ldc_i4_7) },
			{ "ldc.i4.8",                                  /**/ new ILToken(Token.INSTR_NONE, Op.ldc_i4_8) },
			{ "dup",                                       /**/ new ILToken(Token.INSTR_NONE, Op.dup) },
			{ "pop",                                       /**/ new ILToken(Token.INSTR_NONE, Op.pop) },
			{ "ret",                                       /**/ new ILToken(Token.INSTR_NONE, Op.ret) },
			{ "ldind.i1",                                  /**/ new ILToken(Token.INSTR_NONE, Op.ldind_i1) },
			{ "ldind.u1",                                  /**/ new ILToken(Token.INSTR_NONE, Op.ldind_u1) },
			{ "ldind.i2",                                  /**/ new ILToken(Token.INSTR_NONE, Op.ldind_i2) },
			{ "ldind.u2",                                  /**/ new ILToken(Token.INSTR_NONE, Op.ldind_u2) },
			{ "ldind.i4",                                  /**/ new ILToken(Token.INSTR_NONE, Op.ldind_i4) },
			{ "ldind.u4",                                  /**/ new ILToken(Token.INSTR_NONE, Op.ldind_u4) },
			{ "ldind.i8",                                  /**/ new ILToken(Token.INSTR_NONE, Op.ldind_i8) },
			{ "ldind.u8",                                  /**/ new ILToken(Token.INSTR_NONE, Op.ldind_i8) },
			{ "ldind.i",                                   /**/ new ILToken(Token.INSTR_NONE, Op.ldind_i) },
			{ "ldind.r4",                                  /**/ new ILToken(Token.INSTR_NONE, Op.ldind_r4) },
			{ "ldind.r8",                                  /**/ new ILToken(Token.INSTR_NONE, Op.ldind_r8) },
			{ "ldind.ref",                                 /**/ new ILToken(Token.INSTR_NONE, Op.ldind_ref) },
			{ "stind.ref",                                 /**/ new ILToken(Token.INSTR_NONE, Op.stind_ref) },
			{ "stind.i1",                                  /**/ new ILToken(Token.INSTR_NONE, Op.stind_i1) },
			{ "stind.i2",                                  /**/ new ILToken(Token.INSTR_NONE, Op.stind_i2) },
			{ "stind.i4",                                  /**/ new ILToken(Token.INSTR_NONE, Op.stind_i4) },
			{ "stind.i8",                                  /**/ new ILToken(Token.INSTR_NONE, Op.stind_i8) },
			{ "stind.r4",                                  /**/ new ILToken(Token.INSTR_NONE, Op.stind_r4) },
			{ "stind.r8",                                  /**/ new ILToken(Token.INSTR_NONE, Op.stind_r8) },
			{ "add",                                       /**/ new ILToken(Token.INSTR_NONE, Op.add) },
			{ "sub",                                       /**/ new ILToken(Token.INSTR_NONE, Op.sub) },
			{ "mul",                                       /**/ new ILToken(Token.INSTR_NONE, Op.mul) },
			{ "div",                                       /**/ new ILToken(Token.INSTR_NONE, Op.div) },
			{ "div.un",                                    /**/ new ILToken(Token.INSTR_NONE, Op.div_un) },
			{ "rem",                                       /**/ new ILToken(Token.INSTR_NONE, Op.rem) },
			{ "rem.un",                                    /**/ new ILToken(Token.INSTR_NONE, Op.rem_un) },
			{ "and",                                       /**/ new ILToken(Token.INSTR_NONE, Op.and) },
			{ "or",                                        /**/ new ILToken(Token.INSTR_NONE, Op.or) },
			{ "xor",                                       /**/ new ILToken(Token.INSTR_NONE, Op.xor) },
			{ "shl",                                       /**/ new ILToken(Token.INSTR_NONE, Op.shl) },
			{ "shr",                                       /**/ new ILToken(Token.INSTR_NONE, Op.shr) },
			{ "shr.un",                                    /**/ new ILToken(Token.INSTR_NONE, Op.shr_un) },
			{ "neg",                                       /**/ new ILToken(Token.INSTR_NONE, Op.neg) },
			{ "not",                                       /**/ new ILToken(Token.INSTR_NONE, Op.not) },
			{ "conv.i1",                                   /**/ new ILToken(Token.INSTR_NONE, Op.conv_i1) },
			{ "conv.i2",                                   /**/ new ILToken(Token.INSTR_NONE, Op.conv_i2) },
			{ "conv.i4",                                   /**/ new ILToken(Token.INSTR_NONE, Op.conv_i4) },
			{ "conv.i8",                                   /**/ new ILToken(Token.INSTR_NONE, Op.conv_i8) },
			{ "conv.r4",                                   /**/ new ILToken(Token.INSTR_NONE, Op.conv_r4) },
			{ "conv.r8",                                   /**/ new ILToken(Token.INSTR_NONE, Op.conv_r8) },
			{ "conv.u4",                                   /**/ new ILToken(Token.INSTR_NONE, Op.conv_u4) },
			{ "conv.u8",                                   /**/ new ILToken(Token.INSTR_NONE, Op.conv_u8) },
			{ "conv.r.un",                                 /**/ new ILToken(Token.INSTR_NONE, Op.conv_r_un) },
			{ "throw",                                     /**/ new ILToken(Token.INSTR_NONE, Op.throwOp) },
			{ "conv.ovf.i1.un",                            /**/ new ILToken(Token.INSTR_NONE, Op.conv_ovf_i1_un) },
			{ "conv.ovf.i2.un",                            /**/ new ILToken(Token.INSTR_NONE, Op.conv_ovf_i2_un) },
			{ "conv.ovf.i4.un",                            /**/ new ILToken(Token.INSTR_NONE, Op.conv_ovf_i4_un) },
			{ "conv.ovf.i8.un",                            /**/ new ILToken(Token.INSTR_NONE, Op.conv_ovf_i8_un) },
			{ "conv.ovf.u1.un",                            /**/ new ILToken(Token.INSTR_NONE, Op.conv_ovf_u1_un) },
			{ "conv.ovf.u2.un",                            /**/ new ILToken(Token.INSTR_NONE, Op.conv_ovf_u2_un) },
			{ "conv.ovf.u4.un",                            /**/ new ILToken(Token.INSTR_NONE, Op.conv_ovf_u4_un) },
			{ "conv.ovf.u8.un",                            /**/ new ILToken(Token.INSTR_NONE, Op.conv_ovf_u8_un) },
			{ "conv.ovf.i.un",                             /**/ new ILToken(Token.INSTR_NONE, Op.conv_ovf_i_un) },
			{ "conv.ovf.u.un",                             /**/ new ILToken(Token.INSTR_NONE, Op.conv_ovf_u_un) },
			{ "conv.ovf.i1",                               /**/ new ILToken(Token.INSTR_NONE, Op.conv_ovf_i1) },
			{ "conv.ovf.u1",                               /**/ new ILToken(Token.INSTR_NONE, Op.conv_ovf_u1) },
			{ "conv.ovf.i2",                               /**/ new ILToken(Token.INSTR_NONE, Op.conv_ovf_i2) },
			{ "conv.ovf.u2",                               /**/ new ILToken(Token.INSTR_NONE, Op.conv_ovf_u2) },
			{ "conv.ovf.i4",                               /**/ new ILToken(Token.INSTR_NONE, Op.conv_ovf_i4) },
			{ "conv.ovf.u4",                               /**/ new ILToken(Token.INSTR_NONE, Op.conv_ovf_u4) },
			{ "conv.ovf.i8",                               /**/ new ILToken(Token.INSTR_NONE, Op.conv_ovf_i8) },
			{ "conv.ovf.u8",                               /**/ new ILToken(Token.INSTR_NONE, Op.conv_ovf_u8) },
			{ "ldlen",                                     /**/ new ILToken(Token.INSTR_NONE, Op.ldlen) },
			{ "ldelem.i1",                                 /**/ new ILToken(Token.INSTR_NONE, Op.ldelem_i1) },
			{ "ldelem.u1",                                 /**/ new ILToken(Token.INSTR_NONE, Op.ldelem_u1) },
			{ "ldelem.i2",                                 /**/ new ILToken(Token.INSTR_NONE, Op.ldelem_i2) },
			{ "ldelem.u2",                                 /**/ new ILToken(Token.INSTR_NONE, Op.ldelem_u2) },
			{ "ldelem.i4",                                 /**/ new ILToken(Token.INSTR_NONE, Op.ldelem_i4) },
			{ "ldelem.u4",                                 /**/ new ILToken(Token.INSTR_NONE, Op.ldelem_u4) },
			{ "ldelem.i8",                                 /**/ new ILToken(Token.INSTR_NONE, Op.ldelem_i8) },
			{ "ldelem.u8",                                 /**/ new ILToken(Token.INSTR_NONE, Op.ldelem_i8) },
			{ "ldelem.i",                                  /**/ new ILToken(Token.INSTR_NONE, Op.ldelem_i) },
			{ "ldelem.r4",                                 /**/ new ILToken(Token.INSTR_NONE, Op.ldelem_r4) },
			{ "ldelem.r8",                                 /**/ new ILToken(Token.INSTR_NONE, Op.ldelem_r8) },
			{ "ldelem.ref",                                /**/ new ILToken(Token.INSTR_NONE, Op.ldelem_ref) },
			{ "stelem.i",                                  /**/ new ILToken(Token.INSTR_NONE, Op.stelem_i) },
			{ "stelem.i1",                                 /**/ new ILToken(Token.INSTR_NONE, Op.stelem_i1) },
			{ "stelem.i2",                                 /**/ new ILToken(Token.INSTR_NONE, Op.stelem_i2) },
			{ "stelem.i4",                                 /**/ new ILToken(Token.INSTR_NONE, Op.stelem_i4) },
			{ "stelem.i8",                                 /**/ new ILToken(Token.INSTR_NONE, Op.stelem_i8) },
			{ "stelem.r4",                                 /**/ new ILToken(Token.INSTR_NONE, Op.stelem_r4) },
			{ "stelem.r8",                                 /**/ new ILToken(Token.INSTR_NONE, Op.stelem_r8) },
			{ "stelem.ref",                                /**/ new ILToken(Token.INSTR_NONE, Op.stelem_ref) },
			{ "ckfinite",                                  /**/ new ILToken(Token.INSTR_NONE, Op.ckfinite) },
			{ "conv.u2",                                   /**/ new ILToken(Token.INSTR_NONE, Op.conv_u2) },
			{ "conv.u1",                                   /**/ new ILToken(Token.INSTR_NONE, Op.conv_u1) },
			{ "conv.i",                                    /**/ new ILToken(Token.INSTR_NONE, Op.conv_i) },
			{ "conv.ovf.i",                                /**/ new ILToken(Token.INSTR_NONE, Op.conv_ovf_i) },
			{ "conv.ovf.u",                                /**/ new ILToken(Token.INSTR_NONE, Op.conv_ovf_u) },
			{ "add.ovf",                                   /**/ new ILToken(Token.INSTR_NONE, Op.add_ovf) },
			{ "add.ovf.un",                                /**/ new ILToken(Token.INSTR_NONE, Op.add_ovf_un) },
			{ "mul.ovf",                                   /**/ new ILToken(Token.INSTR_NONE, Op.mul_ovf) },
			{ "mul.ovf.un",                                /**/ new ILToken(Token.INSTR_NONE, Op.mul_ovf_un) },
			{ "sub.ovf",                                   /**/ new ILToken(Token.INSTR_NONE, Op.sub_ovf) },
			{ "sub.ovf.un",                                /**/ new ILToken(Token.INSTR_NONE, Op.sub_ovf_un) },
			{ "endfinally",                                /**/ new ILToken(Token.INSTR_NONE, Op.endfinally) },
			// endfault is really just an alias for endfinally
			{ "endfault",                                  /**/ new ILToken(Token.INSTR_NONE, Op.endfinally) },
			{ "stind.i",                                   /**/ new ILToken(Token.INSTR_NONE, Op.stind_i) },
			{ "conv.u",                                    /**/ new ILToken(Token.INSTR_NONE, Op.conv_u) },
			{ "arglist",                                   /**/ new ILToken(Token.INSTR_NONE, Op.arglist) },
			{ "ceq",                                       /**/ new ILToken(Token.INSTR_NONE, Op.ceq) },
			{ "cgt",                                       /**/ new ILToken(Token.INSTR_NONE, Op.cgt) },
			{ "cgt.un",                                    /**/ new ILToken(Token.INSTR_NONE, Op.cgt_un) },
			{ "clt",                                       /**/ new ILToken(Token.INSTR_NONE, Op.clt) },
			{ "clt.un",                                    /**/ new ILToken(Token.INSTR_NONE, Op.clt_un) },
			{ "localloc",                                  /**/ new ILToken(Token.INSTR_NONE, Op.localloc) },
			{ "endfilter",                                 /**/ new ILToken(Token.INSTR_NONE, Op.endfilter) },
			{ "volatile.",                                 /**/ new ILToken(Token.INSTR_NONE, Op.volatile_) },
			{ "tail.",                                     /**/ new ILToken(Token.INSTR_NONE, Op.tail_) },
			{ "cpblk",                                     /**/ new ILToken(Token.INSTR_NONE, Op.cpblk) },
			{ "initblk",                                   /**/ new ILToken(Token.INSTR_NONE, Op.initblk) },
			{ "rethrow",                                   /**/ new ILToken(Token.INSTR_NONE, Op.rethrow) },
			{ "refanytype",                                /**/ new ILToken(Token.INSTR_NONE, Op.refanytype) },
			{ "readonly.",                                 /**/ new ILToken(Token.INSTR_NONE, Op.readonly_) },

			//
			// Int operations
			//

			// param
			{ "ldarg",                                     /**/ new ILToken(Token.INSTR_PARAM, IntOp.ldarg) },
			{ "ldarga",                                    /**/ new ILToken(Token.INSTR_PARAM, IntOp.ldarga) },
			{ "starg",                                     /**/ new ILToken(Token.INSTR_PARAM, IntOp.starg) },
			{ "ldarg.s",                                   /**/ new ILToken(Token.INSTR_PARAM, IntOp.ldarg_s) },
			{ "ldarga.s",                                  /**/ new ILToken(Token.INSTR_PARAM, IntOp.ldarga_s) },
			{ "starg.s",                                   /**/ new ILToken(Token.INSTR_PARAM, IntOp.starg_s) },

			// local
			{ "ldloc",                                     /**/ new ILToken(Token.INSTR_LOCAL, IntOp.ldloc) },
			{ "ldloca",                                    /**/ new ILToken(Token.INSTR_LOCAL, IntOp.ldloca) },
			{ "stloc",                                     /**/ new ILToken(Token.INSTR_LOCAL, IntOp.stloc) },
			{ "ldloc.s",                                   /**/ new ILToken(Token.INSTR_LOCAL, IntOp.ldloc_s) },
			{ "ldloca.s",                                  /**/ new ILToken(Token.INSTR_LOCAL, IntOp.ldloca_s) },
			{ "stloc.s",                                   /**/ new ILToken(Token.INSTR_LOCAL, IntOp.stloc_s) },

			{ "ldc.i4.s",                                  /**/ new ILToken(Token.INSTR_I, IntOp.ldc_i4_s) },
			{ "ldc.i4",                                    /**/ new ILToken(Token.INSTR_I, IntOp.ldc_i4) },
			{ "unaligned.",                                /**/ new ILToken(Token.INSTR_I, IntOp.unaligned) },

			//
			// Type operations
			//

			{ "cpobj",                                     /**/ new ILToken(Token.INSTR_TYPE, TypeOp.cpobj) },
			{ "ldobj",                                     /**/ new ILToken(Token.INSTR_TYPE, TypeOp.ldobj) },
			{ "castclass",                                 /**/ new ILToken(Token.INSTR_TYPE, TypeOp.castclass) },
			{ "isinst",                                    /**/ new ILToken(Token.INSTR_TYPE, TypeOp.isinst) },
			{ "unbox",                                     /**/ new ILToken(Token.INSTR_TYPE, TypeOp.unbox) },
			{ "unbox.any",                                 /**/ new ILToken(Token.INSTR_TYPE, TypeOp.unbox_any) },
			{ "stobj",                                     /**/ new ILToken(Token.INSTR_TYPE, TypeOp.stobj) },
			{ "box",                                       /**/ new ILToken(Token.INSTR_TYPE, TypeOp.box) },
			{ "newarr",                                    /**/ new ILToken(Token.INSTR_TYPE, TypeOp.newarr) },
			{ "ldelema",                                   /**/ new ILToken(Token.INSTR_TYPE, TypeOp.ldelema) },
			{ "refanyval",                                 /**/ new ILToken(Token.INSTR_TYPE, TypeOp.refanyval) },
			{ "mkrefany",                                  /**/ new ILToken(Token.INSTR_TYPE, TypeOp.mkrefany) },
			{ "initobj",                                   /**/ new ILToken(Token.INSTR_TYPE, TypeOp.initobj) },
			{ "sizeof",                                    /**/ new ILToken(Token.INSTR_TYPE, TypeOp.sizeOf) },
			{ "stelem",                                    /**/ new ILToken(Token.INSTR_TYPE, TypeOp.stelem) },
			{ "ldelem",                                    /**/ new ILToken(Token.INSTR_TYPE, TypeOp.ldelem) },
			{ "stelem.any",                                /**/ new ILToken(Token.INSTR_TYPE, TypeOp.stelem) },
			{ "ldelem.any",                                /**/ new ILToken(Token.INSTR_TYPE, TypeOp.ldelem) },
			{ "constrained.",                              /**/ new ILToken(Token.INSTR_TYPE, TypeOp.constrained) },

			//
			// MethodRef operations
			//

			{ "jmp",                                       /**/ new ILToken(Token.INSTR_METHOD, MethodOp.jmp) },
			{ "call",                                      /**/ new ILToken(Token.INSTR_METHOD, MethodOp.call) },
			{ "callvirt",                                  /**/ new ILToken(Token.INSTR_METHOD, MethodOp.callvirt) },
			{ "newobj",                                    /**/ new ILToken(Token.INSTR_METHOD, MethodOp.newobj) },
			{ "ldftn",                                     /**/ new ILToken(Token.INSTR_METHOD, MethodOp.ldftn) },
			{ "ldvirtftn",                                 /**/ new ILToken(Token.INSTR_METHOD, MethodOp.ldvirtfn) },

			//
			// FieldRef instructions
			//

			{ "ldfld",                                     /**/ new ILToken(Token.INSTR_FIELD, FieldOp.ldfld) },
			{ "ldflda",                                    /**/ new ILToken(Token.INSTR_FIELD, FieldOp.ldflda) },
			{ "stfld",                                     /**/ new ILToken(Token.INSTR_FIELD, FieldOp.stfld) },
			{ "ldsfld",                                    /**/ new ILToken(Token.INSTR_FIELD, FieldOp.ldsfld) },
			{ "ldsflda",                                   /**/ new ILToken(Token.INSTR_FIELD, FieldOp.ldsflda) },
			{ "stsfld",                                    /**/ new ILToken(Token.INSTR_FIELD, FieldOp.stsfld) },

			//
			// Branch Instructions
			//

			{ "br",                                        /**/ new ILToken(Token.INSTR_BRTARGET, BranchOp.br) },
			{ "brfalse",                                   /**/ new ILToken(Token.INSTR_BRTARGET, BranchOp.brfalse) },
			{ "brzero",                                    /**/ new ILToken(Token.INSTR_BRTARGET, BranchOp.brfalse) },
			{ "brnull",                                    /**/ new ILToken(Token.INSTR_BRTARGET, BranchOp.brfalse) },
			{ "brtrue",                                    /**/ new ILToken(Token.INSTR_BRTARGET, BranchOp.brtrue) },
			{ "beq",                                       /**/ new ILToken(Token.INSTR_BRTARGET, BranchOp.beq) },
			{ "bge",                                       /**/ new ILToken(Token.INSTR_BRTARGET, BranchOp.bge) },
			{ "bgt",                                       /**/ new ILToken(Token.INSTR_BRTARGET, BranchOp.bgt) },
			{ "ble",                                       /**/ new ILToken(Token.INSTR_BRTARGET, BranchOp.ble) },
			{ "blt",                                       /**/ new ILToken(Token.INSTR_BRTARGET, BranchOp.blt) },
			{ "bne.un",                                    /**/ new ILToken(Token.INSTR_BRTARGET, BranchOp.bne_un) },
			{ "bge.un",                                    /**/ new ILToken(Token.INSTR_BRTARGET, BranchOp.bge_un) },
			{ "bgt.un",                                    /**/ new ILToken(Token.INSTR_BRTARGET, BranchOp.bgt_un) },
			{ "ble.un",                                    /**/ new ILToken(Token.INSTR_BRTARGET, BranchOp.ble_un) },
			{ "blt.un",                                    /**/ new ILToken(Token.INSTR_BRTARGET, BranchOp.blt_un) },
			{ "leave",                                     /**/ new ILToken(Token.INSTR_BRTARGET, BranchOp.leave) },

			{ "br.s",                                      /**/ new ILToken(Token.INSTR_BRTARGET, BranchOp.br_s) },
			{ "brfalse.s",                                 /**/ new ILToken(Token.INSTR_BRTARGET, BranchOp.brfalse_s) },
			{ "brtrue.s",                                  /**/ new ILToken(Token.INSTR_BRTARGET, BranchOp.brtrue_s) },
			{ "beq.s",                                     /**/ new ILToken(Token.INSTR_BRTARGET, BranchOp.beq_s) },
			{ "bge.s",                                     /**/ new ILToken(Token.INSTR_BRTARGET, BranchOp.bge_s) },
			{ "bgt.s",                                     /**/ new ILToken(Token.INSTR_BRTARGET, BranchOp.bgt_s) },
			{ "ble.s",                                     /**/ new ILToken(Token.INSTR_BRTARGET, BranchOp.ble_s) },
			{ "blt.s",                                     /**/ new ILToken(Token.INSTR_BRTARGET, BranchOp.blt_s) },
			{ "bne.un.s",                                  /**/ new ILToken(Token.INSTR_BRTARGET, BranchOp.bne_un_s) },
			{ "bge.un.s",                                  /**/ new ILToken(Token.INSTR_BRTARGET, BranchOp.bge_un_s) },
			{ "bgt.un.s",                                  /**/ new ILToken(Token.INSTR_BRTARGET, BranchOp.bgt_un_s) },
			{ "ble.un.s",                                  /**/ new ILToken(Token.INSTR_BRTARGET, BranchOp.ble_un_s) },
			{ "blt.un.s",                                  /**/ new ILToken(Token.INSTR_BRTARGET, BranchOp.blt_un_s) },
			{ "leave.s",                                   /**/ new ILToken(Token.INSTR_BRTARGET, BranchOp.leave_s) },

			//
			// Misc other instructions
			//

			{ "ldstr",                                     /**/ new ILToken(Token.INSTR_STRING, MiscInstr.ldstr) },
			{ "ldc.r4",                                    /**/ new ILToken(Token.INSTR_R, MiscInstr.ldc_r4) },
			{ "ldc.r8",                                    /**/ new ILToken(Token.INSTR_R, MiscInstr.ldc_r8) },
			{ "ldc.i8",                                    /**/ new ILToken(Token.INSTR_I8, MiscInstr.ldc_i8) },
			{ "switch",                                    /**/ new ILToken(Token.INSTR_SWITCH, MiscInstr._switch) },
			{ "calli",                                     /**/ new ILToken(Token.INSTR_SIG, MiscInstr.calli) },
			{ "ldtoken",                                   /**/ new ILToken(Token.INSTR_TOK, MiscInstr.ldtoken) },
		};
	};
}
