﻿using System;

namespace Mono.ILASM
{
	public class ILToken
	{
		public static readonly ILToken Invalid;
		public static readonly ILToken EOF;

		public static readonly ILToken Dot;

		public static readonly ILToken
			OpenBrace,
			CloseBrace,
			OpenBracket,
			CloseBracket,
			OpenParens,
			CloseParens,
			Comma,
			Colon,
			DoubleColon,
			Semicolon,
			Assign,
			Star,
			Ampersand,
			Plus,
			Slash,
			Bang,
			Ellipsis,
			Dash,
			OpenAngleBracket,
			CloseAngleBracket;

		static readonly ILToken[] punctuations;

		public static ILToken GetPunctuation(int ch)
		{
			int id = "{}[](),:;=*&+/!<>".IndexOf((char)ch);
			ILToken res = null;

			if (id != -1)
				res = punctuations[id];

			return res;
		}

		static ILToken()
		{
			Invalid = new ILToken(-1, "invalid");
			EOF = new ILToken(Token.EOF, "eof");

			Dot = new ILToken(Token.DOT, ".");
			OpenBrace = new ILToken(Token.OPEN_BRACE, "{");
			CloseBrace = new ILToken(Token.CLOSE_BRACE, "}");
			OpenBracket = new ILToken(Token.OPEN_BRACKET, "[");
			CloseBracket = new ILToken(Token.CLOSE_BRACKET, "]");
			OpenParens = new ILToken(Token.OPEN_PARENS, "(");
			CloseParens = new ILToken(Token.CLOSE_PARENS, ")");
			Comma = new ILToken(Token.COMMA, ",");
			Colon = new ILToken(Token.COLON, ":");
			DoubleColon = new ILToken(Token.DOUBLE_COLON, "::");
			Semicolon = new ILToken(Token.SEMICOLON, ";");
			Assign = new ILToken(Token.ASSIGN, "=");
			Star = new ILToken(Token.STAR, "*");
			Ampersand = new ILToken(Token.AMPERSAND, "&");
			Plus = new ILToken(Token.PLUS, "+");
			Slash = new ILToken(Token.SLASH, "/");
			Bang = new ILToken(Token.BANG, "!");
			Ellipsis = new ILToken(Token.ELLIPSIS, "...");
			Dash = new ILToken(Token.DASH, "-");
			OpenAngleBracket = new ILToken(Token.OPEN_ANGLE_BRACKET, "<");
			CloseAngleBracket = new ILToken(Token.CLOSE_ANGLE_BRACKET, ">");

			punctuations = new ILToken[]
			{
				OpenBrace, CloseBrace,
				OpenBracket, CloseBracket,
				OpenParens, CloseParens,
				Comma, Colon, Semicolon,
				Assign, Star, Ampersand,
				Plus, Slash, Bang,
				OpenAngleBracket, CloseAngleBracket
			};
		}

		readonly int token;
		readonly object val;

		public ILToken(int token, object val)
		{
			this.token = token;
			this.val = val;
		}

		public int TokenId => token;

		public object Value => val;

		static bool EqImpl(ILToken t1, ILToken t2)
		{
			bool res = false;
			if ((t1 as object) != null)
				res = t1.Equals(t2);
			else
				res = ((t2 as object) == null);

			return res;
		}

		public static bool operator ==(ILToken t1, ILToken t2)
		{
			return EqImpl(t1, t2);
		}

		public static bool operator !=(ILToken t1, ILToken t2)
		{
			return !EqImpl(t1, t2);
		}

		public override int GetHashCode()
		{
			int h = token;
			if (val != null) h ^= val.GetHashCode();
			return h;
		}

		public override String ToString()
		{
			return (token.ToString() + " : " + (val != null ? val.ToString() : "<null>"));
		}

		public override bool Equals(object o)
		{
			bool res = (o != null);

			if (res)
			{
				res = Object.ReferenceEquals(this, o);
				if (!res)
				{
					res = o is ILToken;
					if (res)
					{
						ILToken that = o as ILToken;
						res = (this.token == that.token) && (this.val.Equals(that.val));
					}
				}
			}

			return res;
		}
	};
}
