// created by jay 0.7 (c) 1998 Axel.Schreiner@informatik.uni-osnabrueck.de

#line 2 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
//
// Mono::ILASM::ILParser
//
// (C) Sergey Chaban (serge@wildwestsoftware.com)
// (C) 2003 Jackson Harper, All rights reserved
//

using PEAPI;
using System;
using System.IO;
using System.Collections;
using System.Collections.Generic;
using System.Globalization;
using System.Reflection;
using System.Security;
using System.Security.Permissions;

using MIPermission = Mono.ILASM.Permission;
using MIPermissionSet = Mono.ILASM.PermissionSet;

namespace Mono.ILASM
{
		public partial class ILParser
		{
#line default

		/** error output stream. It should be changeable. */
		public System.IO.TextWriter ErrorOutput = System.Console.Out;

		static int[] global_yyStates;

		static object[] global_yyVals;

		protected bool use_global_stacks;

		/// <summary>
		/// initial size and increment of the state/value stack [default 256]. Can be overwritten outside 
		/// of invocations of yyparse().
		/// </summary>
		protected int yyMax;

		object[] yyVals;                    // value stack

		object yyVal;                       // value stack ptr

		/* An EOF token */
		public int eof_token;

		int yyToken;                        // current input

		int yyTop;

		/// <summary>
		/// executed at the beginning of a reduce action. Used as $$ = yyDefault($1), prior to the 
		/// user-specified action, if any. Can be overwritten to provide deep copy, etc.
		/// </summary>
		/// <param name="first"> first value for $1, or null </param>
		/// <returns> first </returns>
		protected Object yyDefault(Object first) => first;

		/// (syntax) error message. Can be overwritten to control message format.
		/// @param message text to be displayed.
		/// @param expected vector of acceptable tokens, if available.
		public void yyerror(string message, string[] expected)
		{
			if ((yacc_verbose_flag > 0) && (expected != null) && (expected.Length > 0))
			{
				ErrorOutput.Write(message + ", expecting");
				for (int n = 0; n < expected.Length; n++)
					ErrorOutput.Write(" " + expected[n]);
				ErrorOutput.WriteLine();
			}
			else
				ErrorOutput.WriteLine(message);
		}

		/* simplified error message */
		public void yyerror(string message) => yyerror(message, null);

		/* debugging support, requires the package jay.yydebug. Set to null to suppress debugging messages. */
		internal yydebug.yyDebug debug;
protected const int yyFinal = 1;

// Put this array into a separate class so it is only initialized if debugging is actually used
// Use MarshalByRefObject to disable inlining
class YYRules : MarshalByRefObject
{
	static readonly string[] yyRule =
	{
		"$accept : il_file",
		"il_file : decls",
		"decls :",
		"decls : decls decl",
		"decl : class_all",
		"decl : namespace_all",
		"decl : method_all",
		"decl : field_decl",
		"decl : data_decl",
		"decl : vtfixup_decl",
		"decl : file_decl",
		"decl : assembly_all",
		"decl : assemblyref_all",
		"decl : exptype_all",
		"decl : manifestres_all",
		"decl : module_head",
		"decl : sec_decl",
		"decl : customattr_decl",
		"decl : D_SUBSYSTEM int32",
		"decl : D_CORFLAGS int32",
		"decl : D_FILE K_ALIGNMENT int32",
		"decl : D_IMAGEBASE int64",
		"decl : D_STACKRESERVE int64",
		"decl : extsource_spec",
		"decl : language_decl",
		"extsource_spec : D_LINE int32 SQSTRING",
		"extsource_spec : D_LINE int32",
		"extsource_spec : D_LINE int32 COLON int32 SQSTRING",
		"extsource_spec : D_LINE int32 COLON int32",
		"language_decl : D_LANGUAGE SQSTRING",
		"language_decl : D_LANGUAGE SQSTRING COMMA SQSTRING",
		"language_decl : D_LANGUAGE SQSTRING COMMA SQSTRING COMMA SQSTRING",
		"vtfixup_decl : D_VTFIXUP OPEN_BRACKET int32 CLOSE_BRACKET vtfixup_attr K_AT id",
		"vtfixup_attr :",
		"vtfixup_attr : vtfixup_attr K_INT32",
		"vtfixup_attr : vtfixup_attr K_INT64",
		"vtfixup_attr : vtfixup_attr K_FROMUNMANAGED",
		"vtfixup_attr : vtfixup_attr K_CALLMOSTDERIVED",
		"namespace_all : namespace_head OPEN_BRACE decls CLOSE_BRACE",
		"namespace_head : D_NAMESPACE comp_name",
		"class_all : class_head OPEN_BRACE class_decls CLOSE_BRACE",
		"class_head : D_CLASS class_attr comp_name formal_typars_clause extends_clause impl_clause",
		"class_attr :",
		"class_attr : class_attr K_PUBLIC",
		"class_attr : class_attr K_PRIVATE",
		"class_attr : class_attr K_NESTED K_PRIVATE",
		"class_attr : class_attr K_NESTED K_PUBLIC",
		"class_attr : class_attr K_NESTED K_FAMILY",
		"class_attr : class_attr K_NESTED K_ASSEMBLY",
		"class_attr : class_attr K_NESTED K_FAMANDASSEM",
		"class_attr : class_attr K_NESTED K_FAMORASSEM",
		"class_attr : class_attr K_VALUE",
		"class_attr : class_attr K_ENUM",
		"class_attr : class_attr K_INTERFACE",
		"class_attr : class_attr K_SEALED",
		"class_attr : class_attr K_ABSTRACT",
		"class_attr : class_attr K_AUTO",
		"class_attr : class_attr K_SEQUENTIAL",
		"class_attr : class_attr K_EXPLICIT",
		"class_attr : class_attr K_ANSI",
		"class_attr : class_attr K_UNICODE",
		"class_attr : class_attr K_AUTOCHAR",
		"class_attr : class_attr K_IMPORT",
		"class_attr : class_attr K_SERIALIZABLE",
		"class_attr : class_attr K_BEFOREFIELDINIT",
		"class_attr : class_attr K_SPECIALNAME",
		"class_attr : class_attr K_RTSPECIALNAME",
		"extends_clause :",
		"extends_clause : K_EXTENDS generic_class_ref",
		"impl_clause :",
		"impl_clause : impl_class_refs",
		"impl_class_refs : K_IMPLEMENTS generic_class_ref",
		"impl_class_refs : impl_class_refs COMMA generic_class_ref",
		"formal_typars_clause :",
		"formal_typars_clause : OPEN_ANGLE_BRACKET formal_typars CLOSE_ANGLE_BRACKET",
		"typars_clause :",
		"typars_clause : OPEN_ANGLE_BRACKET typars CLOSE_ANGLE_BRACKET",
		"typars : type",
		"typars : typars COMMA type",
		"constraints_clause :",
		"constraints_clause : OPEN_PARENS constraints CLOSE_PARENS",
		"constraints : type",
		"constraints : constraints COMMA type",
		"generic_class_ref : class_ref",
		"generic_class_ref : K_OBJECT",
		"generic_class_ref : K_CLASS class_ref typars_clause",
		"generic_class_ref : BANG int32",
		"generic_class_ref : BANG BANG int32",
		"generic_class_ref : BANG id",
		"generic_class_ref : BANG BANG id",
		"formal_typars : formal_typar_attr constraints_clause formal_typar",
		"formal_typars : formal_typars COMMA formal_typar_attr constraints_clause formal_typar",
		"formal_typar_attr :",
		"formal_typar_attr : formal_typar_attr PLUS",
		"formal_typar_attr : formal_typar_attr DASH",
		"formal_typar_attr : formal_typar_attr D_CTOR",
		"formal_typar_attr : formal_typar_attr K_VALUETYPE",
		"formal_typar_attr : formal_typar_attr K_CLASS",
		"formal_typar : id",
		"param_type_decl : D_PARAM K_TYPE id",
		"param_type_decl : D_PARAM K_TYPE OPEN_BRACKET int32 CLOSE_BRACKET",
		"class_refs : class_ref",
		"class_refs : class_refs COMMA class_ref",
		"slashed_name : comp_name",
		"slashed_name : slashed_name SLASH comp_name",
		"class_ref : OPEN_BRACKET slashed_name CLOSE_BRACKET slashed_name",
		"class_ref : OPEN_BRACKET D_MODULE slashed_name CLOSE_BRACKET slashed_name",
		"class_ref : slashed_name",
		"class_decls :",
		"class_decls : class_decls class_decl",
		"class_decl : method_all",
		"class_decl : class_all",
		"class_decl : event_all",
		"class_decl : prop_all",
		"class_decl : field_decl",
		"class_decl : data_decl",
		"class_decl : sec_decl",
		"class_decl : extsource_spec",
		"class_decl : customattr_decl",
		"class_decl : param_type_decl",
		"class_decl : D_SIZE int32",
		"class_decl : D_PACK int32",
		"class_decl : D_OVERRIDE type_spec DOUBLE_COLON method_name K_WITH call_conv type type_spec DOUBLE_COLON method_name OPEN_PARENS sig_args CLOSE_PARENS",
		"class_decl : language_decl",
		"type : generic_class_ref",
		"type : K_VALUE K_CLASS class_ref",
		"type : K_VALUETYPE OPEN_BRACKET slashed_name CLOSE_BRACKET slashed_name typars_clause",
		"type : K_VALUETYPE slashed_name typars_clause",
		"type : type OPEN_BRACKET CLOSE_BRACKET",
		"type : type OPEN_BRACKET bounds CLOSE_BRACKET",
		"type : type AMPERSAND",
		"type : type STAR",
		"type : type K_PINNED",
		"type : type K_MODREQ OPEN_PARENS custom_modifier_type CLOSE_PARENS",
		"type : type K_MODOPT OPEN_PARENS custom_modifier_type CLOSE_PARENS",
		"type : K_METHOD call_conv type STAR OPEN_PARENS sig_args CLOSE_PARENS",
		"type : primitive_type",
		"primitive_type : K_INT8",
		"primitive_type : K_INT16",
		"primitive_type : K_INT32",
		"primitive_type : K_INT64",
		"primitive_type : K_FLOAT32",
		"primitive_type : K_FLOAT64",
		"primitive_type : K_UNSIGNED K_INT8",
		"primitive_type : K_UINT8",
		"primitive_type : K_UNSIGNED K_INT16",
		"primitive_type : K_UINT16",
		"primitive_type : K_UNSIGNED K_INT32",
		"primitive_type : K_UINT32",
		"primitive_type : K_UNSIGNED K_INT64",
		"primitive_type : K_UINT64",
		"primitive_type : K_NATIVE K_INT",
		"primitive_type : K_NATIVE K_UNSIGNED K_INT",
		"primitive_type : K_NATIVE K_UINT",
		"primitive_type : K_TYPEDREF",
		"primitive_type : K_CHAR",
		"primitive_type : K_WCHAR",
		"primitive_type : K_VOID",
		"primitive_type : K_BOOL",
		"primitive_type : K_STRING",
		"bounds : bound",
		"bounds : bounds COMMA bound",
		"bound :",
		"bound : ELLIPSIS",
		"bound : int32",
		"bound : int32 ELLIPSIS int32",
		"bound : int32 ELLIPSIS",
		"call_conv : K_INSTANCE call_conv",
		"call_conv : K_EXPLICIT call_conv",
		"call_conv : call_kind",
		"call_kind :",
		"call_kind : K_DEFAULT",
		"call_kind : K_VARARG",
		"call_kind : K_UNMANAGED K_CDECL",
		"call_kind : K_UNMANAGED K_STDCALL",
		"call_kind : K_UNMANAGED K_THISCALL",
		"call_kind : K_UNMANAGED K_FASTCALL",
		"native_type :",
		"native_type : K_CUSTOM OPEN_PARENS comp_qstring COMMA comp_qstring CLOSE_PARENS",
		"native_type : K_FIXED K_SYSSTRING OPEN_BRACKET int32 CLOSE_BRACKET",
		"native_type : K_FIXED K_ARRAY OPEN_BRACKET int32 CLOSE_BRACKET",
		"native_type : K_VARIANT",
		"native_type : K_CURRENCY",
		"native_type : K_SYSCHAR",
		"native_type : K_VOID",
		"native_type : K_BOOL",
		"native_type : K_INT8",
		"native_type : K_INT16",
		"native_type : K_INT32",
		"native_type : K_INT64",
		"native_type : K_FLOAT32",
		"native_type : K_FLOAT64",
		"native_type : K_ERROR",
		"native_type : K_UNSIGNED K_INT8",
		"native_type : K_UINT8",
		"native_type : K_UNSIGNED K_INT16",
		"native_type : K_UINT16",
		"native_type : K_UNSIGNED K_INT32",
		"native_type : K_UINT32",
		"native_type : K_UNSIGNED K_INT64",
		"native_type : K_UINT64",
		"native_type : native_type STAR",
		"native_type : native_type OPEN_BRACKET CLOSE_BRACKET",
		"native_type : native_type OPEN_BRACKET int32 CLOSE_BRACKET",
		"native_type : native_type OPEN_BRACKET int32 PLUS int32 CLOSE_BRACKET",
		"native_type : native_type OPEN_BRACKET PLUS int32 CLOSE_BRACKET",
		"native_type : K_DECIMAL",
		"native_type : K_DATE",
		"native_type : K_BSTR",
		"native_type : K_LPSTR",
		"native_type : K_LPWSTR",
		"native_type : K_LPTSTR",
		"native_type : K_VBBYREFSTR",
		"native_type : K_OBJECTREF",
		"native_type : K_IUNKNOWN",
		"native_type : K_IDISPATCH",
		"native_type : K_STRUCT",
		"native_type : K_INTERFACE",
		"native_type : K_SAFEARRAY variant_type",
		"native_type : K_SAFEARRAY variant_type COMMA comp_qstring",
		"native_type : K_INT",
		"native_type : K_UNSIGNED K_INT",
		"native_type : K_NESTED K_STRUCT",
		"native_type : K_BYVALSTR",
		"native_type : K_ANSI K_BSTR",
		"native_type : K_TBSTR",
		"native_type : K_VARIANT K_BOOL",
		"native_type : K_METHOD",
		"native_type : K_AS K_ANY",
		"native_type : K_LPSTRUCT",
		"variant_type :",
		"variant_type : K_NULL",
		"variant_type : K_VARIANT",
		"variant_type : K_CURRENCY",
		"variant_type : K_VOID",
		"variant_type : K_BOOL",
		"variant_type : K_INT8",
		"variant_type : K_INT16",
		"variant_type : K_INT32",
		"variant_type : K_INT64",
		"variant_type : K_FLOAT32",
		"variant_type : K_FLOAT64",
		"variant_type : K_UNSIGNED K_INT8",
		"variant_type : K_UNSIGNED K_INT16",
		"variant_type : K_UNSIGNED K_INT32",
		"variant_type : K_UNSIGNED K_INT64",
		"variant_type : STAR",
		"variant_type : variant_type OPEN_BRACKET CLOSE_BRACKET",
		"variant_type : variant_type K_VECTOR",
		"variant_type : variant_type AMPERSAND",
		"variant_type : K_DECIMAL",
		"variant_type : K_DATE",
		"variant_type : K_BSTR",
		"variant_type : K_LPSTR",
		"variant_type : K_LPWSTR",
		"variant_type : K_IUNKNOWN",
		"variant_type : K_IDISPATCH",
		"variant_type : K_SAFEARRAY",
		"variant_type : K_INT",
		"variant_type : K_UNSIGNED K_INT",
		"variant_type : K_ERROR",
		"variant_type : K_HRESULT",
		"variant_type : K_CARRAY",
		"variant_type : K_USERDEFINED",
		"variant_type : K_RECORD",
		"variant_type : K_FILETIME",
		"variant_type : K_BLOB",
		"variant_type : K_STREAM",
		"variant_type : K_STORAGE",
		"variant_type : K_STREAMED_OBJECT",
		"variant_type : K_STORED_OBJECT",
		"variant_type : K_BLOB_OBJECT",
		"variant_type : K_CF",
		"variant_type : K_CLSID",
		"custom_modifier_type : primitive_type",
		"custom_modifier_type : class_ref",
		"field_decl : D_FIELD repeat_opt field_attr type id at_opt init_opt semicolon_opt",
		"repeat_opt :",
		"repeat_opt : OPEN_BRACKET int32 CLOSE_BRACKET",
		"field_attr :",
		"field_attr : field_attr K_PUBLIC",
		"field_attr : field_attr K_PRIVATE",
		"field_attr : field_attr K_FAMILY",
		"field_attr : field_attr K_ASSEMBLY",
		"field_attr : field_attr K_FAMANDASSEM",
		"field_attr : field_attr K_FAMORASSEM",
		"field_attr : field_attr K_PRIVATESCOPE",
		"field_attr : field_attr K_STATIC",
		"field_attr : field_attr K_INITONLY",
		"field_attr : field_attr K_RTSPECIALNAME",
		"field_attr : field_attr K_SPECIALNAME",
		"field_attr : field_attr K_MARSHAL OPEN_PARENS native_type CLOSE_PARENS",
		"field_attr : field_attr K_LITERAL",
		"field_attr : field_attr K_NOTSERIALIZED",
		"at_opt :",
		"at_opt : K_AT id",
		"init_opt :",
		"init_opt : ASSIGN field_init",
		"field_init_primitive : K_FLOAT32 OPEN_PARENS float64 CLOSE_PARENS",
		"field_init_primitive : K_FLOAT64 OPEN_PARENS float64 CLOSE_PARENS",
		"field_init_primitive : K_FLOAT32 OPEN_PARENS int64 CLOSE_PARENS",
		"field_init_primitive : K_FLOAT64 OPEN_PARENS int64 CLOSE_PARENS",
		"field_init_primitive : K_INT64 OPEN_PARENS int64 CLOSE_PARENS",
		"field_init_primitive : K_UINT64 OPEN_PARENS int64 CLOSE_PARENS",
		"field_init_primitive : K_INT32 OPEN_PARENS int64 CLOSE_PARENS",
		"field_init_primitive : K_UINT32 OPEN_PARENS int64 CLOSE_PARENS",
		"field_init_primitive : K_INT16 OPEN_PARENS int64 CLOSE_PARENS",
		"field_init_primitive : K_UINT16 OPEN_PARENS int64 CLOSE_PARENS",
		"field_init_primitive : K_CHAR OPEN_PARENS int64 CLOSE_PARENS",
		"field_init_primitive : K_WCHAR OPEN_PARENS int64 CLOSE_PARENS",
		"field_init_primitive : K_INT8 OPEN_PARENS int64 CLOSE_PARENS",
		"field_init_primitive : K_UINT8 OPEN_PARENS int64 CLOSE_PARENS",
		"field_init_primitive : K_BOOL OPEN_PARENS truefalse CLOSE_PARENS",
		"field_init_full : field_init_primitive",
		"field_init_full : K_BYTEARRAY bytes_list",
		"field_init : field_init_full",
		"field_init : comp_qstring",
		"field_init : K_NULLREF",
		"member_init : field_init_full",
		"member_init : K_STRING OPEN_PARENS SQSTRING CLOSE_PARENS",
		"opt_truefalse_list : truefalse_list",
		"truefalse_list : truefalse",
		"truefalse_list : truefalse_list truefalse",
		"data_decl : data_head data_body",
		"data_head : D_DATA data_attr id ASSIGN",
		"data_head : D_DATA data_attr",
		"data_attr :",
		"data_attr : K_TLS",
		"data_attr : K_CIL",
		"data_body : OPEN_BRACE dataitem_list CLOSE_BRACE",
		"data_body : dataitem",
		"dataitem_list : dataitem",
		"dataitem_list : dataitem_list COMMA dataitem",
		"dataitem : K_CHAR STAR OPEN_PARENS comp_qstring CLOSE_PARENS",
		"dataitem : K_WCHAR STAR OPEN_PARENS comp_qstring CLOSE_PARENS",
		"dataitem : AMPERSAND OPEN_PARENS id CLOSE_PARENS",
		"dataitem : K_BYTEARRAY ASSIGN bytes_list",
		"dataitem : K_BYTEARRAY bytes_list",
		"dataitem : K_FLOAT32 OPEN_PARENS float64 CLOSE_PARENS repeat_opt",
		"dataitem : K_FLOAT64 OPEN_PARENS float64 CLOSE_PARENS repeat_opt",
		"dataitem : K_INT64 OPEN_PARENS int64 CLOSE_PARENS repeat_opt",
		"dataitem : K_INT32 OPEN_PARENS int32 CLOSE_PARENS repeat_opt",
		"dataitem : K_INT16 OPEN_PARENS int32 CLOSE_PARENS repeat_opt",
		"dataitem : K_INT8 OPEN_PARENS int32 CLOSE_PARENS repeat_opt",
		"dataitem : K_FLOAT32 repeat_opt",
		"dataitem : K_FLOAT64 repeat_opt",
		"dataitem : K_INT64 repeat_opt",
		"dataitem : K_INT32 repeat_opt",
		"dataitem : K_INT16 repeat_opt",
		"dataitem : K_INT8 repeat_opt",
		"method_all : method_head OPEN_BRACE method_decls CLOSE_BRACE",
		"method_head : D_METHOD meth_attr call_conv param_attr type method_name formal_typars_clause OPEN_PARENS sig_args CLOSE_PARENS impl_attr",
		"method_head : D_METHOD meth_attr call_conv param_attr type K_MARSHAL OPEN_PARENS native_type CLOSE_PARENS method_name OPEN_PARENS sig_args CLOSE_PARENS impl_attr",
		"meth_attr :",
		"meth_attr : meth_attr K_STATIC",
		"meth_attr : meth_attr K_PUBLIC",
		"meth_attr : meth_attr K_PRIVATE",
		"meth_attr : meth_attr K_FAMILY",
		"meth_attr : meth_attr K_ASSEMBLY",
		"meth_attr : meth_attr K_FAMANDASSEM",
		"meth_attr : meth_attr K_FAMORASSEM",
		"meth_attr : meth_attr K_PRIVATESCOPE",
		"meth_attr : meth_attr K_FINAL",
		"meth_attr : meth_attr K_VIRTUAL",
		"meth_attr : meth_attr K_ABSTRACT",
		"meth_attr : meth_attr K_HIDEBYSIG",
		"meth_attr : meth_attr K_NEWSLOT",
		"meth_attr : meth_attr K_REQSECOBJ",
		"meth_attr : meth_attr K_SPECIALNAME",
		"meth_attr : meth_attr K_RTSPECIALNAME",
		"meth_attr : meth_attr K_STRICT",
		"meth_attr : meth_attr K_COMPILERCONTROLLED",
		"meth_attr : meth_attr K_UNMANAGEDEXP",
		"meth_attr : meth_attr K_PINVOKEIMPL OPEN_PARENS comp_qstring K_AS comp_qstring pinv_attr CLOSE_PARENS",
		"meth_attr : meth_attr K_PINVOKEIMPL OPEN_PARENS comp_qstring pinv_attr CLOSE_PARENS",
		"meth_attr : meth_attr K_PINVOKEIMPL OPEN_PARENS pinv_attr CLOSE_PARENS",
		"pinv_attr :",
		"pinv_attr : pinv_attr K_NOMANGLE",
		"pinv_attr : pinv_attr K_ANSI",
		"pinv_attr : pinv_attr K_UNICODE",
		"pinv_attr : pinv_attr K_AUTOCHAR",
		"pinv_attr : pinv_attr K_LASTERR",
		"pinv_attr : pinv_attr K_WINAPI",
		"pinv_attr : pinv_attr K_CDECL",
		"pinv_attr : pinv_attr K_STDCALL",
		"pinv_attr : pinv_attr K_THISCALL",
		"pinv_attr : pinv_attr K_FASTCALL",
		"pinv_attr : pinv_attr K_BESTFIT COLON K_ON",
		"pinv_attr : pinv_attr K_BESTFIT COLON K_OFF",
		"pinv_attr : pinv_attr K_CHARMAPERROR COLON K_ON",
		"pinv_attr : pinv_attr K_CHARMAPERROR COLON K_OFF",
		"method_name : D_CTOR",
		"method_name : D_CCTOR",
		"method_name : comp_name",
		"param_attr :",
		"param_attr : param_attr OPEN_BRACKET K_IN CLOSE_BRACKET",
		"param_attr : param_attr OPEN_BRACKET K_OUT CLOSE_BRACKET",
		"param_attr : param_attr OPEN_BRACKET K_OPT CLOSE_BRACKET",
		"impl_attr :",
		"impl_attr : impl_attr K_NATIVE",
		"impl_attr : impl_attr K_CIL",
		"impl_attr : impl_attr K_IL",
		"impl_attr : impl_attr K_OPTIL",
		"impl_attr : impl_attr K_MANAGED",
		"impl_attr : impl_attr K_UNMANAGED",
		"impl_attr : impl_attr K_FORWARDREF",
		"impl_attr : impl_attr K_PRESERVESIG",
		"impl_attr : impl_attr K_RUNTIME",
		"impl_attr : impl_attr K_INTERNALCALL",
		"impl_attr : impl_attr K_SYNCHRONIZED",
		"impl_attr : impl_attr K_NOINLINING",
		"impl_attr : impl_attr K_NOOPTIMIZATION",
		"impl_attr : impl_attr K_AGGRESSIVEINLINING",
		"sig_args :",
		"sig_args : sig_arg_list",
		"sig_arg_list : sig_arg",
		"sig_arg_list : sig_arg_list COMMA sig_arg",
		"sig_arg : param_attr type",
		"sig_arg : param_attr type id",
		"sig_arg : ELLIPSIS",
		"sig_arg : param_attr type K_MARSHAL OPEN_PARENS native_type CLOSE_PARENS",
		"sig_arg : param_attr type K_MARSHAL OPEN_PARENS native_type CLOSE_PARENS id",
		"type_list :",
		"type_list : ELLIPSIS",
		"type_list : type_list COMMA ELLIPSIS",
		"type_list : param_attr type opt_id",
		"type_list : type_list COMMA param_attr type opt_id",
		"opt_id :",
		"opt_id : id",
		"method_decls :",
		"method_decls : method_decls method_decl",
		"method_decl : D_EMITBYTE int32",
		"method_decl : D_MAXSTACK int32",
		"method_decl : D_LOCALS OPEN_PARENS local_list CLOSE_PARENS",
		"method_decl : D_LOCALS K_INIT OPEN_PARENS local_list CLOSE_PARENS",
		"method_decl : D_ENTRYPOINT",
		"method_decl : D_ZEROINIT",
		"method_decl : D_EXPORT OPEN_BRACKET int32 CLOSE_BRACKET",
		"method_decl : D_EXPORT OPEN_BRACKET int32 CLOSE_BRACKET K_AS id",
		"method_decl : D_VTENTRY int32 COLON int32",
		"method_decl : D_OVERRIDE type_spec DOUBLE_COLON method_name",
		"method_decl : D_OVERRIDE K_METHOD method_ref",
		"method_decl : D_OVERRIDE K_METHOD call_conv type type_spec DOUBLE_COLON method_name OPEN_ANGLE_BRACKET OPEN_BRACKET int32 CLOSE_BRACKET CLOSE_ANGLE_BRACKET OPEN_PARENS type_list CLOSE_PARENS",
		"method_decl : scope_block",
		"method_decl : D_PARAM OPEN_BRACKET int32 CLOSE_BRACKET init_opt",
		"method_decl : param_type_decl",
		"method_decl : id COLON",
		"method_decl : seh_block",
		"method_decl : instr",
		"method_decl : sec_decl",
		"method_decl : extsource_spec",
		"method_decl : language_decl",
		"method_decl : customattr_decl",
		"method_decl : data_decl",
		"local_list :",
		"local_list : local",
		"local_list : local_list COMMA local",
		"local : type",
		"local : type id",
		"local : slot_num type",
		"local : slot_num type id",
		"slot_num : OPEN_BRACKET int32 CLOSE_BRACKET",
		"type_spec : OPEN_BRACKET slashed_name CLOSE_BRACKET",
		"type_spec : OPEN_BRACKET D_MODULE slashed_name CLOSE_BRACKET",
		"type_spec : type",
		"scope_block : scope_block_begin method_decls CLOSE_BRACE",
		"scope_block_begin : OPEN_BRACE",
		"seh_block : try_block seh_clauses",
		"try_block : D_TRY scope_block",
		"try_block : D_TRY id K_TO id",
		"try_block : D_TRY int32 K_TO int32",
		"seh_clauses : seh_clause",
		"seh_clauses : seh_clauses seh_clause",
		"seh_clause : K_CATCH type handler_block",
		"seh_clause : K_FINALLY handler_block",
		"seh_clause : K_FAULT handler_block",
		"seh_clause : filter_clause handler_block",
		"filter_clause : K_FILTER scope_block",
		"filter_clause : K_FILTER id",
		"filter_clause : K_FILTER int32",
		"handler_block : scope_block",
		"handler_block : K_HANDLER id K_TO id",
		"handler_block : K_HANDLER int32 K_TO int32",
		"instr : INSTR_NONE",
		"instr : INSTR_LOCAL int32",
		"instr : INSTR_LOCAL id",
		"instr : INSTR_PARAM int32",
		"instr : INSTR_PARAM id",
		"instr : INSTR_I int32",
		"instr : INSTR_I id",
		"instr : INSTR_I8 int64",
		"instr : INSTR_R float64",
		"instr : INSTR_R int64",
		"instr : INSTR_R bytes_list",
		"instr : INSTR_BRTARGET int32",
		"instr : INSTR_BRTARGET id",
		"instr : INSTR_METHOD method_ref",
		"instr : INSTR_FIELD type type_spec DOUBLE_COLON id",
		"instr : INSTR_FIELD type id",
		"instr : INSTR_TYPE type_spec",
		"instr : INSTR_STRING comp_qstring",
		"instr : INSTR_STRING K_BYTEARRAY ASSIGN bytes_list",
		"instr : INSTR_STRING K_BYTEARRAY bytes_list",
		"instr : INSTR_SIG call_conv type OPEN_PARENS type_list CLOSE_PARENS",
		"instr : INSTR_TOK owner_type",
		"instr : INSTR_SWITCH OPEN_PARENS labels CLOSE_PARENS",
		"method_ref : call_conv type method_name typars_clause OPEN_PARENS type_list CLOSE_PARENS",
		"method_ref : call_conv type type_spec DOUBLE_COLON method_name typars_clause OPEN_PARENS type_list CLOSE_PARENS",
		"labels :",
		"labels : id",
		"labels : int32",
		"labels : labels COMMA id",
		"labels : labels COMMA int32",
		"owner_type : type_spec",
		"owner_type : member_ref",
		"member_ref : K_METHOD method_ref",
		"member_ref : K_FIELD type type_spec DOUBLE_COLON id",
		"member_ref : K_FIELD type id",
		"event_all : event_head OPEN_BRACE event_decls CLOSE_BRACE",
		"event_head : D_EVENT event_attr type_spec comp_name",
		"event_head : D_EVENT event_attr id",
		"event_attr :",
		"event_attr : event_attr K_RTSPECIALNAME",
		"event_attr : event_attr K_SPECIALNAME",
		"event_decls :",
		"event_decls : event_decls event_decl",
		"event_decl : D_ADDON method_ref semicolon_opt",
		"event_decl : D_REMOVEON method_ref semicolon_opt",
		"event_decl : D_FIRE method_ref semicolon_opt",
		"event_decl : D_OTHER method_ref semicolon_opt",
		"event_decl : customattr_decl",
		"event_decl : extsource_spec",
		"event_decl : language_decl",
		"prop_all : prop_head OPEN_BRACE prop_decls CLOSE_BRACE",
		"prop_head : D_PROPERTY prop_attr type comp_name OPEN_PARENS type_list CLOSE_PARENS init_opt",
		"prop_attr :",
		"prop_attr : prop_attr K_RTSPECIALNAME",
		"prop_attr : prop_attr K_SPECIALNAME",
		"prop_attr : prop_attr K_INSTANCE",
		"prop_decls :",
		"prop_decls : prop_decls prop_decl",
		"prop_decl : D_SET method_ref",
		"prop_decl : D_GET method_ref",
		"prop_decl : D_OTHER method_ref",
		"prop_decl : customattr_decl",
		"prop_decl : extsource_spec",
		"prop_decl : language_decl",
		"customattr_decl : D_CUSTOM customattr_owner_type_opt custom_type",
		"customattr_decl : D_CUSTOM customattr_owner_type_opt custom_type ASSIGN comp_qstring",
		"customattr_decl : D_CUSTOM customattr_owner_type_opt custom_type ASSIGN bytes_list",
		"customattr_decl : D_CUSTOM customattr_owner_type_opt custom_type ASSIGN OPEN_BRACE customattr_values CLOSE_BRACE",
		"customattr_owner_type_opt :",
		"customattr_owner_type_opt : OPEN_PARENS type CLOSE_PARENS",
		"customattr_values :",
		"customattr_values : K_BOOL OPEN_BRACKET int32 CLOSE_BRACKET OPEN_PARENS opt_truefalse_list CLOSE_PARENS",
		"customattr_values : K_BYTEARRAY bytes_list",
		"customattr_values : K_STRING OPEN_PARENS SQSTRING CLOSE_PARENS",
		"customattr_values : customattr_ctor_args",
		"customattr_ctor_args : customattr_ctor_arg",
		"customattr_ctor_args : customattr_ctor_args customattr_ctor_arg",
		"customattr_ctor_arg : field_init_primitive",
		"customattr_ctor_arg : K_TYPE OPEN_PARENS type CLOSE_PARENS",
		"custom_type : call_conv type type_spec DOUBLE_COLON method_name OPEN_PARENS type_list CLOSE_PARENS",
		"custom_type : call_conv type method_name OPEN_PARENS type_list CLOSE_PARENS",
		"sec_decl : D_PERMISSION sec_action type_spec OPEN_PARENS nameval_pairs CLOSE_PARENS",
		"sec_decl : D_PERMISSION sec_action type_spec",
		"sec_decl : D_PERMISSIONSET sec_action ASSIGN bytes_list",
		"sec_decl : D_PERMISSIONSET sec_action comp_qstring",
		"sec_decl : D_PERMISSIONSET sec_action ASSIGN OPEN_BRACE permissions CLOSE_BRACE",
		"permissions : permission",
		"permissions : permissions COMMA permission",
		"permission : class_ref ASSIGN OPEN_BRACE permission_members CLOSE_BRACE",
		"permission_members : permission_member",
		"permission_members : permission_members permission_member",
		"permission_member : prop_or_field primitive_type perm_mbr_nameval_pair",
		"permission_member : prop_or_field K_ENUM class_ref perm_mbr_nameval_pair",
		"perm_mbr_nameval_pair : SQSTRING ASSIGN member_init",
		"prop_or_field : K_PROPERTY",
		"prop_or_field : K_FIELD",
		"nameval_pairs : nameval_pair",
		"nameval_pairs : nameval_pairs COMMA nameval_pair",
		"nameval_pair : comp_qstring ASSIGN cavalue",
		"cavalue : truefalse",
		"cavalue : int32",
		"cavalue : int32 OPEN_PARENS int32 CLOSE_PARENS",
		"cavalue : comp_qstring",
		"cavalue : class_ref OPEN_PARENS K_INT8 COLON int32 CLOSE_PARENS",
		"cavalue : class_ref OPEN_PARENS K_INT16 COLON int32 CLOSE_PARENS",
		"cavalue : class_ref OPEN_PARENS K_INT32 COLON int32 CLOSE_PARENS",
		"cavalue : class_ref OPEN_PARENS int32 CLOSE_PARENS",
		"sec_action : K_REQUEST",
		"sec_action : K_DEMAND",
		"sec_action : K_ASSERT",
		"sec_action : K_DENY",
		"sec_action : K_PERMITONLY",
		"sec_action : K_LINKCHECK",
		"sec_action : K_INHERITCHECK",
		"sec_action : K_REQMIN",
		"sec_action : K_REQOPT",
		"sec_action : K_REQREFUSE",
		"sec_action : K_PREJITGRANT",
		"sec_action : K_PREJITDENY",
		"sec_action : K_NONCASDEMAND",
		"sec_action : K_NONCASLINKDEMAND",
		"sec_action : K_NONCASINHERITANCE",
		"module_head : D_MODULE",
		"module_head : D_MODULE comp_name",
		"module_head : D_MODULE K_EXTERN comp_name",
		"file_decl : D_FILE file_attr comp_name file_entry D_HASH ASSIGN bytes_list file_entry",
		"file_decl : D_FILE file_attr comp_name file_entry",
		"file_attr :",
		"file_attr : file_attr K_NOMETADATA",
		"file_entry :",
		"file_entry : D_ENTRYPOINT",
		"assembly_all : assembly_head OPEN_BRACE assembly_decls CLOSE_BRACE",
		"assembly_head : D_ASSEMBLY legacylibrary_opt asm_attr slashed_name",
		"asm_attr :",
		"asm_attr : asm_attr K_RETARGETABLE",
		"assembly_decls :",
		"assembly_decls : assembly_decls assembly_decl",
		"assembly_decl : D_PUBLICKEY ASSIGN bytes_list",
		"assembly_decl : D_VER int32 COLON int32 COLON int32 COLON int32",
		"assembly_decl : D_LOCALE comp_qstring",
		"assembly_decl : D_LOCALE ASSIGN bytes_list",
		"assembly_decl : D_HASH K_ALGORITHM int32",
		"assembly_decl : customattr_decl",
		"assembly_decl : sec_decl",
		"asm_or_ref_decl : D_PUBLICKEY ASSIGN bytes_list",
		"asm_or_ref_decl : D_VER int32 COLON int32 COLON int32 COLON int32",
		"asm_or_ref_decl : D_LOCALE comp_qstring",
		"asm_or_ref_decl : D_LOCALE ASSIGN bytes_list",
		"asm_or_ref_decl : customattr_decl",
		"assemblyref_all : assemblyref_head OPEN_BRACE assemblyref_decls CLOSE_BRACE",
		"assemblyref_head : D_ASSEMBLY K_EXTERN legacylibrary_opt asm_attr slashed_name",
		"assemblyref_head : D_ASSEMBLY K_EXTERN legacylibrary_opt asm_attr slashed_name K_AS slashed_name",
		"assemblyref_decls :",
		"assemblyref_decls : assemblyref_decls assemblyref_decl",
		"assemblyref_decl : D_VER int32 COLON int32 COLON int32 COLON int32",
		"assemblyref_decl : D_PUBLICKEY ASSIGN bytes_list",
		"assemblyref_decl : D_PUBLICKEYTOKEN ASSIGN bytes_list",
		"assemblyref_decl : D_LOCALE comp_qstring",
		"assemblyref_decl : D_LOCALE ASSIGN bytes_list",
		"assemblyref_decl : D_HASH ASSIGN bytes_list",
		"assemblyref_decl : customattr_decl",
		"assemblyref_decl : K_AUTO",
		"exptype_all : exptype_head OPEN_BRACE exptype_decls CLOSE_BRACE",
		"exptype_head : D_CLASS K_EXTERN expt_attr comp_name",
		"expt_attr :",
		"expt_attr : expt_attr K_PRIVATE",
		"expt_attr : expt_attr K_PUBLIC",
		"expt_attr : expt_attr K_FORWARDER",
		"expt_attr : expt_attr K_NESTED K_PUBLIC",
		"expt_attr : expt_attr K_NESTED K_PRIVATE",
		"expt_attr : expt_attr K_NESTED K_FAMILY",
		"expt_attr : expt_attr K_NESTED K_ASSEMBLY",
		"expt_attr : expt_attr K_NESTED K_FAMANDASSEM",
		"expt_attr : expt_attr K_NESTED K_FAMORASSEM",
		"exptype_decls :",
		"exptype_decls : exptype_decls exptype_decl",
		"exptype_decl : D_FILE comp_name",
		"exptype_decl : D_CLASS K_EXTERN comp_name",
		"exptype_decl : customattr_decl",
		"exptype_decl : D_ASSEMBLY K_EXTERN comp_name",
		"manifestres_all : manifestres_head OPEN_BRACE manifestres_decls CLOSE_BRACE",
		"manifestres_head : D_MRESOURCE manres_attr comp_name",
		"manres_attr :",
		"manres_attr : manres_attr K_PUBLIC",
		"manres_attr : manres_attr K_PRIVATE",
		"manifestres_decls :",
		"manifestres_decls : manifestres_decls manifestres_decl",
		"manifestres_decl : D_FILE comp_name K_AT int32",
		"manifestres_decl : D_ASSEMBLY K_EXTERN slashed_name",
		"manifestres_decl : customattr_decl",
		"comp_qstring : QSTRING",
		"comp_qstring : comp_qstring PLUS QSTRING",
		"int32 : INT64",
		"int64 : INT64",
		"float64 : FLOAT64",
		"float64 : K_FLOAT32 OPEN_PARENS INT32 CLOSE_PARENS",
		"float64 : K_FLOAT32 OPEN_PARENS INT64 CLOSE_PARENS",
		"float64 : K_FLOAT64 OPEN_PARENS INT64 CLOSE_PARENS",
		"float64 : K_FLOAT64 OPEN_PARENS INT32 CLOSE_PARENS",
		"hexbyte : HEXBYTE",
		"$$1 :",
		"bytes_list : OPEN_PARENS $$1 bytes CLOSE_PARENS",
		"bytes :",
		"bytes : hexbytes",
		"hexbytes : hexbyte",
		"hexbytes : hexbytes hexbyte",
		"truefalse : K_TRUE",
		"truefalse : K_FALSE",
		"id : ID",
		"id : SQSTRING",
		"comp_name : id",
		"comp_name : comp_name DOT comp_name",
		"comp_name : COMP_NAME",
		"semicolon_opt :",
		"semicolon_opt : SEMICOLON",
		"legacylibrary_opt :",
		"legacylibrary_opt : K_LEGACY K_LIBRARY",
	};
	public static string getRule(int ix) => yyRule[ix];
}

static readonly string[] yyNames =
{
	"end-of-file",
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	"'!'",
	null,
	null,
	null,
	null,
	"'&'",
	null,
	"'('",
	"')'",
	"'*'",
	"'+'",
	"','",
	"'-'",
	"'.'",
	"'/'",
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	"':'",
	"';'",
	"'<'",
	"'='",
	"'>'",
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	"'['",
	null,
	"']'",
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	"'{'",
	null,
	"'}'",
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	null,
	"EOF",
	"ID",
	"QSTRING",
	"SQSTRING",
	"COMP_NAME",
	"INT32",
	"INT64",
	"FLOAT64",
	"HEXBYTE",
	"DOT",
	"OPEN_BRACE",
	"CLOSE_BRACE",
	"OPEN_BRACKET",
	"CLOSE_BRACKET",
	"OPEN_PARENS",
	"CLOSE_PARENS",
	"COMMA",
	"COLON",
	"DOUBLE_COLON",
	"\"::\"",
	"SEMICOLON",
	"ASSIGN",
	"STAR",
	"AMPERSAND",
	"PLUS",
	"SLASH",
	"BANG",
	"ELLIPSIS",
	"\"...\"",
	"DASH",
	"OPEN_ANGLE_BRACKET",
	"CLOSE_ANGLE_BRACKET",
	"UNKNOWN",
	"INSTR_NONE",
	"INSTR_VAR",
	"INSTR_I",
	"INSTR_I8",
	"INSTR_R",
	"INSTR_BRTARGET",
	"INSTR_METHOD",
	"INSTR_NEWOBJ",
	"INSTR_FIELD",
	"INSTR_TYPE",
	"INSTR_STRING",
	"INSTR_SIG",
	"INSTR_RVA",
	"INSTR_TOK",
	"INSTR_SWITCH",
	"INSTR_PHI",
	"INSTR_LOCAL",
	"INSTR_PARAM",
	"D_ADDON",
	"D_ALGORITHM",
	"D_ASSEMBLY",
	"D_BACKING",
	"D_BLOB",
	"D_CAPABILITY",
	"D_CCTOR",
	"D_CLASS",
	"D_COMTYPE",
	"D_CONFIG",
	"D_IMAGEBASE",
	"D_CORFLAGS",
	"D_CTOR",
	"D_CUSTOM",
	"D_DATA",
	"D_EMITBYTE",
	"D_ENTRYPOINT",
	"D_EVENT",
	"D_EXELOC",
	"D_EXPORT",
	"D_FIELD",
	"D_FILE",
	"D_FIRE",
	"D_GET",
	"D_HASH",
	"D_IMPLICITCOM",
	"D_LANGUAGE",
	"D_LINE",
	"D_XLINE",
	"D_LOCALE",
	"D_LOCALS",
	"D_MANIFESTRES",
	"D_MAXSTACK",
	"D_METHOD",
	"D_MIME",
	"D_MODULE",
	"D_MRESOURCE",
	"D_NAMESPACE",
	"D_ORIGINATOR",
	"D_OS",
	"D_OTHER",
	"D_OVERRIDE",
	"D_PACK",
	"D_PARAM",
	"D_PERMISSION",
	"D_PERMISSIONSET",
	"D_PROCESSOR",
	"D_PROPERTY",
	"D_PUBLICKEY",
	"D_PUBLICKEYTOKEN",
	"D_REMOVEON",
	"D_SET",
	"D_SIZE",
	"D_STACKRESERVE",
	"D_SUBSYSTEM",
	"D_TITLE",
	"D_TRY",
	"D_VER",
	"D_VTABLE",
	"D_VTENTRY",
	"D_VTFIXUP",
	"D_ZEROINIT",
	"K_AT",
	"K_AS",
	"K_AGGRESSIVEINLINING",
	"K_IMPLICITCOM",
	"K_IMPLICITRES",
	"K_NOAPPDOMAIN",
	"K_NOPROCESS",
	"K_NOMACHINE",
	"K_EXTERN",
	"K_INSTANCE",
	"K_EXPLICIT",
	"K_DEFAULT",
	"K_VARARG",
	"K_UNMANAGED",
	"K_CDECL",
	"K_STDCALL",
	"K_THISCALL",
	"K_FASTCALL",
	"K_MARSHAL",
	"K_IN",
	"K_OUT",
	"K_OPT",
	"K_STATIC",
	"K_PUBLIC",
	"K_PRIVATE",
	"K_FAMILY",
	"K_INITONLY",
	"K_RTSPECIALNAME",
	"K_STRICT",
	"K_SPECIALNAME",
	"K_ASSEMBLY",
	"K_FAMANDASSEM",
	"K_FAMORASSEM",
	"K_PRIVATESCOPE",
	"K_LITERAL",
	"K_NOTSERIALIZED",
	"K_VALUE",
	"K_NOT_IN_GC_HEAP",
	"K_INTERFACE",
	"K_SEALED",
	"K_ABSTRACT",
	"K_AUTO",
	"K_SEQUENTIAL",
	"K_ANSI",
	"K_UNICODE",
	"K_AUTOCHAR",
	"K_BESTFIT",
	"K_IMPORT",
	"K_SERIALIZABLE",
	"K_NESTED",
	"K_LATEINIT",
	"K_EXTENDS",
	"K_IMPLEMENTS",
	"K_FINAL",
	"K_VIRTUAL",
	"K_HIDEBYSIG",
	"K_NEWSLOT",
	"K_UNMANAGEDEXP",
	"K_PINVOKEIMPL",
	"K_NOMANGLE",
	"K_OLE",
	"K_LASTERR",
	"K_WINAPI",
	"K_NATIVE",
	"K_IL",
	"K_CIL",
	"K_OPTIL",
	"K_MANAGED",
	"K_FORWARDREF",
	"K_RUNTIME",
	"K_INTERNALCALL",
	"K_SYNCHRONIZED",
	"K_NOINLINING",
	"K_NOOPTIMIZATION",
	"K_CUSTOM",
	"K_FIXED",
	"K_SYSSTRING",
	"K_ARRAY",
	"K_VARIANT",
	"K_CURRENCY",
	"K_SYSCHAR",
	"K_VOID",
	"K_BOOL",
	"K_INT8",
	"K_INT16",
	"K_INT32",
	"K_INT64",
	"K_FLOAT32",
	"K_FLOAT64",
	"K_ERROR",
	"K_UNSIGNED",
	"K_UINT",
	"K_UINT8",
	"K_UINT16",
	"K_UINT32",
	"K_UINT64",
	"K_DECIMAL",
	"K_DATE",
	"K_BSTR",
	"K_LPSTR",
	"K_LPWSTR",
	"K_LPTSTR",
	"K_VBBYREFSTR",
	"K_OBJECTREF",
	"K_IUNKNOWN",
	"K_IDISPATCH",
	"K_STRUCT",
	"K_SAFEARRAY",
	"K_INT",
	"K_BYVALSTR",
	"K_TBSTR",
	"K_LPVOID",
	"K_ANY",
	"K_FLOAT",
	"K_LPSTRUCT",
	"K_NULL",
	"K_PTR",
	"K_VECTOR",
	"K_HRESULT",
	"K_CARRAY",
	"K_USERDEFINED",
	"K_RECORD",
	"K_FILETIME",
	"K_BLOB",
	"K_STREAM",
	"K_STORAGE",
	"K_STREAMED_OBJECT",
	"K_STORED_OBJECT",
	"K_BLOB_OBJECT",
	"K_CF",
	"K_CLSID",
	"K_METHOD",
	"K_CLASS",
	"K_PINNED",
	"K_MODREQ",
	"K_MODOPT",
	"K_TYPEDREF",
	"K_TYPE",
	"K_WCHAR",
	"K_CHAR",
	"K_FROMUNMANAGED",
	"K_CALLMOSTDERIVED",
	"K_BYTEARRAY",
	"K_WITH",
	"K_INIT",
	"K_TO",
	"K_CATCH",
	"K_FILTER",
	"K_FINALLY",
	"K_FAULT",
	"K_HANDLER",
	"K_TLS",
	"K_FIELD",
	"K_PROPERTY",
	"K_REQUEST",
	"K_DEMAND",
	"K_ASSERT",
	"K_DENY",
	"K_PERMITONLY",
	"K_LINKCHECK",
	"K_INHERITCHECK",
	"K_REQMIN",
	"K_REQOPT",
	"K_REQREFUSE",
	"K_PREJITGRANT",
	"K_PREJITDENY",
	"K_NONCASDEMAND",
	"K_NONCASLINKDEMAND",
	"K_NONCASINHERITANCE",
	"K_READONLY",
	"K_NOMETADATA",
	"K_ALGORITHM",
	"K_FULLORIGIN",
	"K_ENABLEJITTRACKING",
	"K_DISABLEJITOPTIMIZER",
	"K_RETARGETABLE",
	"K_PRESERVESIG",
	"K_BEFOREFIELDINIT",
	"K_ALIGNMENT",
	"K_NULLREF",
	"K_VALUETYPE",
	"K_COMPILERCONTROLLED",
	"K_REQSECOBJ",
	"K_ENUM",
	"K_OBJECT",
	"K_STRING",
	"K_TRUE",
	"K_FALSE",
	"K_IS",
	"K_ON",
	"K_OFF",
	"K_FORWARDER",
	"K_CHARMAPERROR",
	"K_LEGACY",
	"K_LIBRARY"
};

		/// <summary> index-checked interface to yyNames[] </summary>
		/// <param name="token">single character or %token value </param>
		/// <returns>token name or [illegal] or [unknown] </returns>
		public static string yyname(int token)
		{
			if (token < 0 || token > yyNames.Length)
				return "[illegal]";
			return yyNames[token] is string name ? name : "[unknown]";
		}


		int yyExpectingState;

		/// <summary> computes list of expected tokens on error by tracing the tables </summary>
		/// <param name="state">state for which to compute the list</param>
		/// <returns>list of token names.</returns>
		protected int[] yyExpectingTokens(int state)
		{
			int token, n, len = 0;
			bool[] ok = new bool[yyNames.Length];
			if ((n = yySindex[state]) != 0)
				for (token = n < 0 ? -n : 0; token < yyNames.Length && n + token < yyTable.Length; token++)
					if (yyCheck[n + token] == token && !ok[token] && yyNames[token] != null)
					{
						len++;
						ok[token] = true;
					}
			if ((n = yyRindex[state]) != 0)
				for (token = n < 0 ? -n : 0;
					 (token < yyNames.Length) && (n + token < yyTable.Length); token++)
					if (yyCheck[n + token] == token && !ok[token] && yyNames[token] != null)
					{
						len++;
						ok[token] = true;
					}
			int[] result = new int[len];
			for (n = token = 0; n < len; token++)
				if (ok[token]) result[n++] = token;
			return result;
		}

		protected string[] yyExpecting(int state)
		{
			int[] tokens = yyExpectingTokens(state);
			string[] result = new string[tokens.Length];
			for (int n = 0; n < tokens.Length; n++)
				result[n++] = yyNames[tokens[n]];
			return result;
		}

		/// <summary> The generated parser, with debugging messages. Maintains a state and a value 
		/// stack, currently with fixed maximum size </summary>
		/// <param name="yyLex"> scanner </param>
		/// <param name="yyd"> debug message writer implementing yyDebug, or null </param>
		/// <returns> result of the last reduction, if any. </returns>
		/// <exception cref="yyException"> irrecoverable parse error </exception>
		internal Object yyparse(yyParser.yyInput yyLex, Object yyd)
		{
			this.debug = (yydebug.yyDebug)yyd;
			return yyparse(yyLex);
		}

		/// <summary>
		/// the generated parser. Maintains a state and a value stack, currently with fixed maximum size.
		/// </summary>
		/// <param name="yyLex">scanner</param>
		/// <returns>result of the last reduction, if any.</returns>
		/// <exception cref="yyException"> on irrecoverable parse error.</exception>
		internal Object yyparse(yyParser.yyInput yyLex)
		{
			if (yyMax <= 0) yyMax = 256;        // initial size
			int yyState = 0;                    // state stack ptr
			int[] yyStates;						// state stack 
			yyVal = null;
			yyToken = -1;
			int yyErrorFlag = 0;                // #tks to shift

			if (use_global_stacks && global_yyStates != null)
			{
				yyVals = global_yyVals;
				yyStates = global_yyStates;
			}
			else
			{
				yyVals = new object[yyMax];
				yyStates = new int[yyMax];
				if (use_global_stacks)
				{
					global_yyVals = yyVals;
					global_yyStates = yyStates;
				}
			}
			/*yyLoop:*/
			for (yyTop = 0; ; ++yyTop)
			{
				if (yyTop >= yyStates.Length)
				{           // dynamically increase
					global::System.Array.Resize(ref yyStates, yyStates.Length + yyMax);
					global::System.Array.Resize(ref yyVals, yyVals.Length + yyMax);
				}
				yyStates[yyTop] = yyState;
				yyVals[yyTop] = yyVal;
				if (debug != null) debug.push(yyState, yyVal);

				/*yyDiscarded:*/
				while (true)
				{   // discarding a token does not change stack
					int yyN;
					if ((yyN = yyDefRed[yyState]) == 0)
					{   // else [default] reduce(yyN)
						if (yyToken < 0)
						{
							yyToken = yyLex.advance() ? yyLex.token() : 0;
							if (debug != null) debug.lex(yyState, yyToken, yyname(yyToken), yyLex.value());
						}

						if ((yyN = yySindex[yyState]) != 0 && (yyN += yyToken) >= 0 && yyN < yyTable.Length && yyCheck[yyN] == yyToken)
						{
							if (debug != null) debug.shift(yyState, yyTable[yyN], yyErrorFlag - 1);
							yyState = yyTable[yyN];     // shift to yyN
							yyVal = yyLex.value();
							yyToken = -1;
							if (yyErrorFlag > 0)
								yyErrorFlag--;
							goto continue_yyLoop;
						}

						if ((yyN = yyRindex[yyState]) != 0 && (yyN += yyToken) >= 0 && yyN < yyTable.Length && yyCheck[yyN] == yyToken)
							yyN = yyTable[yyN];         // reduce(yyN)
						else
							switch (yyErrorFlag)
							{
								case 0:
									yyExpectingState = yyState;
									// yyerror(String.Format("syntax error, got token `{0}'", yyname(yyToken)), yyExpecting(yyState));
									if (debug != null) debug.error("syntax error");
									if (yyToken == 0 /*eof*/ || yyToken == eof_token) throw new yyParser.yyUnexpectedEof();
									goto case 1;

								case 1:
								case 2:
									yyErrorFlag = 3;
									do
									{
										if ((yyN = yySindex[yyStates[yyTop]]) != 0 && (yyN += Token.yyErrorCode) >= 0 && yyN < yyTable.Length && yyCheck[yyN] == Token.yyErrorCode)
										{
											if (debug != null) debug.shift(yyStates[yyTop], yyTable[yyN], 3);
											yyState = yyTable[yyN];
											yyVal = yyLex.value();
											goto continue_yyLoop;
										}
										if (debug != null) debug.pop(yyStates[yyTop]);
									} while (--yyTop >= 0);
									if (debug != null) debug.reject();
									throw new yyParser.yyException("irrecoverable syntax error");

								case 3:
									if (yyToken == 0)
									{
										if (debug != null) debug.reject();
										throw new yyParser.yyException("irrecoverable syntax error at end-of-file");
									}
									if (debug != null) debug.discard(yyState, yyToken, yyname(yyToken), yyLex.value());
									yyToken = -1;
									goto continue_yyDiscarded;      // leave stack alone
							}
					}
					int yyV = yyTop + 1 - yyLen[yyN];
					if (debug != null) debug.reduce(yyState, yyStates[yyV - 1], yyN, YYRules.getRule(yyN), yyLen[yyN]);

					yyVal = yyV > yyTop ? null : yyVals[yyV]; // yyVal = yyDefault(yyV > yyTop ? null : yyVals[yyV]);
					switch (yyN)
					{
case 17:
  case_17();
  break;
case 18:
#line 402 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.SetSubSystem((int)yyVals[0+yyTop]); }
  break;
case 19:
#line 403 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.SetCorFlags((int)yyVals[0+yyTop]); }
  break;
case 21:
#line 405 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.SetImageBase((long)yyVals[0+yyTop]); }
  break;
case 22:
#line 406 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.SetStackReserve((long)yyVals[0+yyTop]); }
  break;
case 38:
#line 434 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.CurrentNameSpace = null; }
  break;
case 39:
#line 437 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.CurrentNameSpace = (string)yyVals[0+yyTop]; }
  break;
case 40:
#line 440 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.EndTypeDef(); }
  break;
case 41:
  case_41();
  break;
case 42:
  case_42();
  break;
case 43:
#line 461 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (TypeAttr)yyVals[-1+yyTop] | TypeAttr.Public; }
  break;
case 44:
#line 462 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (TypeAttr)yyVals[-1+yyTop] | TypeAttr.Private; }
  break;
case 45:
#line 463 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (TypeAttr)yyVals[-2+yyTop] | TypeAttr.NestedPrivate; }
  break;
case 46:
#line 464 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (TypeAttr)yyVals[-2+yyTop] | TypeAttr.NestedPublic; }
  break;
case 47:
#line 465 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (TypeAttr)yyVals[-2+yyTop] | TypeAttr.NestedFamily; }
  break;
case 48:
#line 466 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (TypeAttr)yyVals[-2+yyTop] | TypeAttr.NestedAssembly;}
  break;
case 49:
#line 467 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (TypeAttr)yyVals[-2+yyTop] | TypeAttr.NestedFamAndAssem; }
  break;
case 50:
#line 468 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (TypeAttr)yyVals[-2+yyTop] | TypeAttr.NestedFamOrAssem; }
  break;
case 51:
#line 469 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { is_value_class = true; }
  break;
case 52:
#line 470 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { is_enum_class = true; }
  break;
case 53:
#line 471 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (TypeAttr)yyVals[-1+yyTop] | TypeAttr.Interface; }
  break;
case 54:
#line 472 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (TypeAttr)yyVals[-1+yyTop] | TypeAttr.Sealed; }
  break;
case 55:
#line 473 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (TypeAttr)yyVals[-1+yyTop] | TypeAttr.Abstract; }
  break;
case 56:
#line 474 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  {  }
  break;
case 57:
#line 475 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (TypeAttr)yyVals[-1+yyTop] | TypeAttr.SequentialLayout; }
  break;
case 58:
#line 476 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (TypeAttr)yyVals[-1+yyTop] | TypeAttr.ExplicitLayout; }
  break;
case 59:
#line 477 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  {  }
  break;
case 60:
#line 478 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (TypeAttr)yyVals[-1+yyTop] | TypeAttr.UnicodeClass; }
  break;
case 61:
#line 479 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (TypeAttr)yyVals[-1+yyTop] | TypeAttr.AutoClass; }
  break;
case 62:
#line 480 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (TypeAttr)yyVals[-1+yyTop] | TypeAttr.Import; }
  break;
case 63:
#line 481 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (TypeAttr)yyVals[-1+yyTop] | TypeAttr.Serializable; }
  break;
case 64:
#line 482 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (TypeAttr)yyVals[-1+yyTop] | TypeAttr.BeforeFieldInit; }
  break;
case 65:
#line 483 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (TypeAttr)yyVals[-1+yyTop] | TypeAttr.SpecialName; }
  break;
case 66:
#line 484 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (TypeAttr)yyVals[-1+yyTop] | TypeAttr.RTSpecialName; }
  break;
case 68:
#line 488 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = yyVals[0+yyTop]; }
  break;
case 71:
  case_71();
  break;
case 72:
  case_72();
  break;
case 74:
#line 511 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = yyVals[-1+yyTop]; }
  break;
case 76:
#line 515 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = yyVals[-1+yyTop]; }
  break;
case 77:
  case_77();
  break;
case 78:
  case_78();
  break;
case 80:
#line 532 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = yyVals[-1+yyTop]; }
  break;
case 81:
  case_81();
  break;
case 82:
  case_82();
  break;
case 83:
#line 550 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = yyVals[0+yyTop]; }
  break;
case 84:
#line 551 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new PrimitiveTypeRef(PrimitiveType.Object, "System.Object"); }
  break;
case 85:
  case_85();
  break;
case 86:
  case_86();
  break;
case 87:
  case_87();
  break;
case 88:
  case_88();
  break;
case 89:
  case_89();
  break;
case 90:
  case_90();
  break;
case 91:
  case_91();
  break;
case 92:
#line 604 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new PEAPI.GenericParamAttributes(); }
  break;
case 93:
#line 605 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (PEAPI.GenericParamAttributes)yyVals[-1+yyTop] | PEAPI.GenericParamAttributes.Covariant; }
  break;
case 94:
#line 606 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (PEAPI.GenericParamAttributes)yyVals[-1+yyTop] | PEAPI.GenericParamAttributes.Contravariant; }
  break;
case 95:
#line 607 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (PEAPI.GenericParamAttributes)yyVals[-1+yyTop] | PEAPI.GenericParamAttributes.DefaultConstructorConstrait; }
  break;
case 96:
#line 608 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (PEAPI.GenericParamAttributes)yyVals[-1+yyTop] | PEAPI.GenericParamAttributes.NotNullableValueTypeConstraint; }
  break;
case 97:
#line 609 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (PEAPI.GenericParamAttributes)yyVals[-1+yyTop] | PEAPI.GenericParamAttributes.ReferenceTypeConstraint; }
  break;
case 98:
#line 612 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = yyVals[0+yyTop]; }
  break;
case 99:
  case_99();
  break;
case 100:
  case_100();
  break;
case 101:
  case_101();
  break;
case 102:
  case_102();
  break;
case 104:
#line 650 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = String.Format("{0}/{1}", yyVals[-2+yyTop], yyVals[0+yyTop]); }
  break;
case 105:
  case_105();
  break;
case 106:
  case_106();
  break;
case 107:
  case_107();
  break;
case 116:
#line 688 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { AddSecDecl(yyVals[0+yyTop], false); }
  break;
case 118:
  case_118();
  break;
case 120:
#line 696 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.CurrentTypeDef.SetSize((int)yyVals[0+yyTop]); }
  break;
case 121:
#line 697 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.CurrentTypeDef.SetPack((int)yyVals[0+yyTop]); }
  break;
case 122:
  case_122();
  break;
case 124:
#line 727 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = GetTypeRef((BaseTypeRef)yyVals[0+yyTop]); }
  break;
case 125:
  case_125();
  break;
case 126:
  case_126();
  break;
case 127:
  case_127();
  break;
case 128:
  case_128();
  break;
case 129:
  case_129();
  break;
case 130:
  case_130();
  break;
case 131:
  case_131();
  break;
case 132:
  case_132();
  break;
case 133:
  case_133();
  break;
case 134:
  case_134();
  break;
case 135:
#line 796 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new MethodPointerTypeRef((CallConv)yyVals[-5+yyTop], (BaseTypeRef)yyVals[-4+yyTop], (ArrayList)yyVals[-1+yyTop]); }
  break;
case 137:
#line 800 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new PrimitiveTypeRef(PrimitiveType.Int8, "System.SByte"); }
  break;
case 138:
#line 801 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new PrimitiveTypeRef(PrimitiveType.Int16, "System.Int16"); }
  break;
case 139:
#line 802 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new PrimitiveTypeRef(PrimitiveType.Int32, "System.Int32"); }
  break;
case 140:
#line 803 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new PrimitiveTypeRef(PrimitiveType.Int64, "System.Int64"); }
  break;
case 141:
#line 804 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new PrimitiveTypeRef(PrimitiveType.Float32, "System.Single"); }
  break;
case 142:
#line 805 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new PrimitiveTypeRef(PrimitiveType.Float64, "System.Double"); }
  break;
case 143:
#line 806 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new PrimitiveTypeRef(PrimitiveType.UInt8, "System.Byte"); }
  break;
case 144:
#line 807 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new PrimitiveTypeRef(PrimitiveType.UInt8, "System.Byte"); }
  break;
case 145:
#line 808 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new PrimitiveTypeRef(PrimitiveType.UInt16, "System.UInt16"); }
  break;
case 146:
#line 809 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new PrimitiveTypeRef(PrimitiveType.UInt16, "System.UInt16"); }
  break;
case 147:
#line 810 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new PrimitiveTypeRef(PrimitiveType.UInt32, "System.UInt32"); }
  break;
case 148:
#line 811 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new PrimitiveTypeRef(PrimitiveType.UInt32, "System.UInt32"); }
  break;
case 149:
#line 812 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new PrimitiveTypeRef(PrimitiveType.UInt64, "System.UInt64"); }
  break;
case 150:
#line 813 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new PrimitiveTypeRef(PrimitiveType.UInt64, "System.UInt64"); }
  break;
case 151:
  case_151();
  break;
case 152:
#line 819 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new PrimitiveTypeRef(PrimitiveType.NativeUInt, "System.UIntPtr"); }
  break;
case 153:
#line 820 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new PrimitiveTypeRef(PrimitiveType.NativeUInt, "System.UIntPtr"); }
  break;
case 154:
#line 821 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new PrimitiveTypeRef(PrimitiveType.TypedRef, "System.TypedReference"); }
  break;
case 155:
#line 822 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new PrimitiveTypeRef(PrimitiveType.Char, "System.Char"); }
  break;
case 156:
#line 823 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new PrimitiveTypeRef(PrimitiveType.Char, "System.Char"); }
  break;
case 157:
#line 824 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new PrimitiveTypeRef(PrimitiveType.Void, "System.Void"); }
  break;
case 158:
#line 825 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new PrimitiveTypeRef(PrimitiveType.Boolean, "System.Boolean"); }
  break;
case 159:
#line 826 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new PrimitiveTypeRef(PrimitiveType.String, "System.String"); }
  break;
case 160:
  case_160();
  break;
case 161:
  case_161();
  break;
case 162:
  case_162();
  break;
case 163:
  case_163();
  break;
case 164:
  case_164();
  break;
case 165:
  case_165();
  break;
case 166:
  case_166();
  break;
case 167:
#line 878 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (CallConv)yyVals[0+yyTop] | CallConv.Instance; }
  break;
case 168:
#line 879 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (CallConv)yyVals[0+yyTop] | CallConv.InstanceExplicit; }
  break;
case 170:
#line 883 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new CallConv(); }
  break;
case 171:
#line 884 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = CallConv.Default; }
  break;
case 172:
#line 885 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = CallConv.Vararg; }
  break;
case 173:
#line 886 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = CallConv.Cdecl; }
  break;
case 174:
#line 887 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = CallConv.Stdcall; }
  break;
case 175:
#line 888 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = CallConv.Thiscall; }
  break;
case 176:
#line 889 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = CallConv.Fastcall; }
  break;
case 178:
#line 893 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new CustomMarshaller((string)yyVals[-3+yyTop], (string)yyVals[-1+yyTop]); }
  break;
case 179:
#line 894 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new FixedSysString((uint)(int)yyVals[-1+yyTop]); }
  break;
case 180:
#line 895 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new FixedArray((int)yyVals[-1+yyTop]); }
  break;
case 182:
#line 897 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.Currency; }
  break;
case 184:
#line 899 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.Void; }
  break;
case 185:
#line 900 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.Boolean; }
  break;
case 186:
#line 901 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.Int8; }
  break;
case 187:
#line 902 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.Int16; }
  break;
case 188:
#line 903 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.Int32; }
  break;
case 189:
#line 904 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.Int64; }
  break;
case 190:
#line 905 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.Float32; }
  break;
case 191:
#line 906 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.Float64; }
  break;
case 192:
#line 907 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.Error; }
  break;
case 193:
#line 908 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.UInt8; }
  break;
case 194:
#line 909 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.UInt8; }
  break;
case 195:
#line 910 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.UInt16; }
  break;
case 196:
#line 911 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.UInt16; }
  break;
case 197:
#line 912 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.UInt32; }
  break;
case 198:
#line 913 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.UInt32; }
  break;
case 199:
#line 914 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.UInt64; }
  break;
case 200:
#line 915 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.UInt64; }
  break;
case 202:
#line 917 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new NativeArray((NativeType)yyVals[-2+yyTop]); }
  break;
case 203:
#line 918 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new NativeArray((NativeType)yyVals[-3+yyTop], (int)yyVals[-1+yyTop], 0, 0); }
  break;
case 204:
  case_204();
  break;
case 205:
  case_205();
  break;
case 208:
#line 931 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.BStr; }
  break;
case 209:
#line 932 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.LPStr; }
  break;
case 210:
#line 933 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.LPWStr; }
  break;
case 211:
#line 934 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.LPTStr; }
  break;
case 212:
#line 935 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.ByValStr; }
  break;
case 214:
#line 937 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.IUnknown; }
  break;
case 215:
#line 938 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.IDispatch; }
  break;
case 216:
#line 939 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.Struct; }
  break;
case 217:
#line 940 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.Interface; }
  break;
case 218:
  case_218();
  break;
case 220:
#line 949 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.Int; }
  break;
case 221:
#line 950 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.UInt; }
  break;
case 223:
#line 952 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.ByValStr; }
  break;
case 224:
#line 953 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.AnsiBStr; }
  break;
case 225:
#line 954 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.TBstr; }
  break;
case 226:
#line 955 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.VariantBool; }
  break;
case 227:
#line 956 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.FuncPtr; }
  break;
case 228:
#line 957 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.AsAny; }
  break;
case 229:
#line 958 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = NativeType.LPStruct; }
  break;
case 232:
#line 963 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = SafeArrayType.variant; }
  break;
case 233:
#line 964 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = SafeArrayType.currency; }
  break;
case 235:
#line 966 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = SafeArrayType.boolean; }
  break;
case 236:
#line 967 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = SafeArrayType.int8; }
  break;
case 237:
#line 968 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = SafeArrayType.int16; }
  break;
case 238:
#line 969 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = SafeArrayType.int32; }
  break;
case 240:
#line 971 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = SafeArrayType.float32; }
  break;
case 241:
#line 972 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = SafeArrayType.float64; }
  break;
case 242:
#line 973 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = SafeArrayType.uint8; }
  break;
case 243:
#line 974 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = SafeArrayType.uint16; }
  break;
case 244:
#line 975 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = SafeArrayType.uint32; }
  break;
case 250:
#line 981 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = SafeArrayType.Decimal; }
  break;
case 251:
#line 982 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = SafeArrayType.date; }
  break;
case 252:
#line 983 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = SafeArrayType.bstr; }
  break;
case 255:
#line 986 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = SafeArrayType.unknown; }
  break;
case 256:
#line 987 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = SafeArrayType.unknown; }
  break;
case 258:
#line 989 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = SafeArrayType.Int; }
  break;
case 259:
#line 990 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = SafeArrayType.UInt; }
  break;
case 260:
#line 991 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = SafeArrayType.error; }
  break;
case 276:
  case_276();
  break;
case 278:
#line 1026 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = yyVals[-1+yyTop]; }
  break;
case 279:
#line 1029 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new FieldAttr(); }
  break;
case 280:
#line 1030 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (FieldAttr)yyVals[-1+yyTop] | FieldAttr.Public; }
  break;
case 281:
#line 1031 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (FieldAttr)yyVals[-1+yyTop] | FieldAttr.Private; }
  break;
case 282:
#line 1032 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (FieldAttr)yyVals[-1+yyTop] | FieldAttr.Family; }
  break;
case 283:
#line 1033 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (FieldAttr)yyVals[-1+yyTop] | FieldAttr.Assembly; }
  break;
case 284:
#line 1034 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (FieldAttr)yyVals[-1+yyTop] | FieldAttr.FamAndAssem; }
  break;
case 285:
#line 1035 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (FieldAttr)yyVals[-1+yyTop] | FieldAttr.FamOrAssem; }
  break;
case 286:
#line 1039 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  {
								/* This is just 0x0000*/
						  }
  break;
case 287:
#line 1040 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (FieldAttr)yyVals[-1+yyTop] | FieldAttr.Static; }
  break;
case 288:
#line 1041 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (FieldAttr)yyVals[-1+yyTop] | FieldAttr.Initonly; }
  break;
case 289:
#line 1042 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (FieldAttr)yyVals[-1+yyTop] | FieldAttr.RTSpecialName; }
  break;
case 290:
#line 1043 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (FieldAttr)yyVals[-1+yyTop] | FieldAttr.SpecialName; }
  break;
case 291:
  case_291();
  break;
case 292:
#line 1049 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (FieldAttr)yyVals[-1+yyTop] | FieldAttr.Literal; }
  break;
case 293:
#line 1050 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (FieldAttr)yyVals[-1+yyTop] | FieldAttr.Notserialized; }
  break;
case 295:
#line 1054 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = yyVals[0+yyTop]; }
  break;
case 297:
#line 1058 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = yyVals[0+yyTop]; }
  break;
case 298:
#line 1061 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new FloatConst(Convert.ToSingle(yyVals[-1+yyTop])); }
  break;
case 299:
#line 1062 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new DoubleConst(Convert.ToDouble(yyVals[-1+yyTop])); }
  break;
case 300:
#line 1063 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new FloatConst(BitConverter.ToSingle(BitConverter.GetBytes((long)yyVals[-1+yyTop]), BitConverter.IsLittleEndian ? 0 : 4)); }
  break;
case 301:
#line 1064 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new DoubleConst(BitConverter.Int64BitsToDouble((long)yyVals[-1+yyTop])); }
  break;
case 302:
#line 1065 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new IntConst(Convert.ToInt64(yyVals[-1+yyTop])); }
  break;
case 303:
#line 1066 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new UIntConst(Convert.ToUInt64((ulong)(long)yyVals[-1+yyTop])); }
  break;
case 304:
#line 1067 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new IntConst((int)((long)yyVals[-1+yyTop])); }
  break;
case 305:
#line 1068 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new UIntConst((uint)((long)yyVals[-1+yyTop])); }
  break;
case 306:
#line 1069 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new IntConst((short)((long)yyVals[-1+yyTop])); }
  break;
case 307:
#line 1070 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new UIntConst((ushort)((long)yyVals[-1+yyTop])); }
  break;
case 308:
#line 1071 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new CharConst(Convert.ToChar(yyVals[-1+yyTop])); }
  break;
case 309:
#line 1072 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new CharConst(Convert.ToChar(yyVals[-1+yyTop])); }
  break;
case 310:
#line 1073 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new IntConst((sbyte)((long)(yyVals[-1+yyTop]))); }
  break;
case 311:
#line 1074 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new UIntConst((byte)((long)(yyVals[-1+yyTop]))); }
  break;
case 312:
#line 1075 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new BoolConst((bool)yyVals[-1+yyTop]); }
  break;
case 314:
#line 1079 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new ByteArrConst((byte[])yyVals[0+yyTop]); }
  break;
case 316:
  case_316();
  break;
case 317:
#line 1088 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new NullConst(); }
  break;
case 319:
#line 1092 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new StringConst((string)yyVals[-1+yyTop]); }
  break;
case 321:
#line 1098 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new List<BoolConst>() { new BoolConst((bool)yyVals[0+yyTop]) }; }
  break;
case 322:
  case_322();
  break;
case 323:
  case_323();
  break;
case 324:
#line 1128 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new DataDef((string)yyVals[-1+yyTop], (DataSegment)yyVals[-2+yyTop]); }
  break;
case 325:
#line 1129 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new DataDef(String.Empty, (DataSegment)yyVals[0+yyTop]); }
  break;
case 326:
#line 1132 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = DataSegment.Data; }
  break;
case 327:
#line 1133 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = DataSegment.TLS; }
  break;
case 328:
#line 1134 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = DataSegment.CIL; }
  break;
case 329:
#line 1137 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = yyVals[-1+yyTop]; }
  break;
case 331:
  case_331();
  break;
case 332:
  case_332();
  break;
case 333:
#line 1154 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new StringConst((string)yyVals[-1+yyTop]); }
  break;
case 334:
#line 1155 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new StringConst((string)yyVals[-1+yyTop]); }
  break;
case 335:
  case_335();
  break;
case 336:
#line 1161 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new ByteArrConst((byte[])yyVals[0+yyTop]); }
  break;
case 337:
  case_337();
  break;
case 338:
  case_338();
  break;
case 339:
  case_339();
  break;
case 340:
  case_340();
  break;
case 341:
  case_341();
  break;
case 342:
  case_342();
  break;
case 343:
  case_343();
  break;
case 344:
  case_344();
  break;
case 345:
  case_345();
  break;
case 346:
  case_346();
  break;
case 347:
  case_347();
  break;
case 348:
  case_348();
  break;
case 349:
  case_349();
  break;
case 350:
#line 1268 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.EndMethodDef(tokenizer.Location); }
  break;
case 351:
  case_351();
  break;
case 352:
  case_352();
  break;
case 353:
#line 1299 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new MethAttr(); }
  break;
case 354:
#line 1300 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (MethAttr)yyVals[-1+yyTop] | MethAttr.Static; }
  break;
case 355:
#line 1301 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (MethAttr)yyVals[-1+yyTop] | MethAttr.Public; }
  break;
case 356:
#line 1302 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (MethAttr)yyVals[-1+yyTop] | MethAttr.Private; }
  break;
case 357:
#line 1303 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (MethAttr)yyVals[-1+yyTop] | MethAttr.Family; }
  break;
case 358:
#line 1304 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (MethAttr)yyVals[-1+yyTop] | MethAttr.Assembly; }
  break;
case 359:
#line 1305 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (MethAttr)yyVals[-1+yyTop] | MethAttr.FamAndAssem; }
  break;
case 360:
#line 1306 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (MethAttr)yyVals[-1+yyTop] | MethAttr.FamOrAssem; }
  break;
case 361:
#line 1307 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { /* CHECK HEADERS */ }
  break;
case 362:
#line 1308 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (MethAttr)yyVals[-1+yyTop] | MethAttr.Final; }
  break;
case 363:
#line 1309 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (MethAttr)yyVals[-1+yyTop] | MethAttr.Virtual; }
  break;
case 364:
#line 1310 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (MethAttr)yyVals[-1+yyTop] | MethAttr.Abstract; }
  break;
case 365:
#line 1311 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (MethAttr)yyVals[-1+yyTop] | MethAttr.HideBySig; }
  break;
case 366:
#line 1312 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (MethAttr)yyVals[-1+yyTop] | MethAttr.NewSlot; }
  break;
case 367:
#line 1313 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (MethAttr)yyVals[-1+yyTop] | MethAttr.RequireSecObject; }
  break;
case 368:
#line 1314 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (MethAttr)yyVals[-1+yyTop] | MethAttr.SpecialName; }
  break;
case 369:
#line 1315 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (MethAttr)yyVals[-1+yyTop] | MethAttr.RTSpecialName; }
  break;
case 370:
#line 1316 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (MethAttr)yyVals[-1+yyTop] | MethAttr.Strict; }
  break;
case 371:
#line 1317 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { /* Do nothing */ }
  break;
case 373:
  case_373();
  break;
case 374:
  case_374();
  break;
case 375:
  case_375();
  break;
case 376:
#line 1342 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new PInvokeAttr(); }
  break;
case 377:
#line 1343 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (PInvokeAttr)yyVals[-1+yyTop] | PInvokeAttr.nomangle; }
  break;
case 378:
#line 1344 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (PInvokeAttr)yyVals[-1+yyTop] | PInvokeAttr.ansi; }
  break;
case 379:
#line 1345 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (PInvokeAttr)yyVals[-1+yyTop] | PInvokeAttr.unicode; }
  break;
case 380:
#line 1346 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (PInvokeAttr)yyVals[-1+yyTop] | PInvokeAttr.autochar; }
  break;
case 381:
#line 1347 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (PInvokeAttr)yyVals[-1+yyTop] | PInvokeAttr.lasterr; }
  break;
case 382:
#line 1348 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (PInvokeAttr)yyVals[-1+yyTop] | PInvokeAttr.winapi; }
  break;
case 383:
#line 1349 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (PInvokeAttr)yyVals[-1+yyTop] | PInvokeAttr.cdecl; }
  break;
case 384:
#line 1350 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (PInvokeAttr)yyVals[-1+yyTop] | PInvokeAttr.stdcall; }
  break;
case 385:
#line 1351 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (PInvokeAttr)yyVals[-1+yyTop] | PInvokeAttr.thiscall; }
  break;
case 386:
#line 1352 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (PInvokeAttr)yyVals[-1+yyTop] | PInvokeAttr.fastcall; }
  break;
case 387:
#line 1353 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (PInvokeAttr)yyVals[-3+yyTop] | PInvokeAttr.bestfit_on; }
  break;
case 388:
#line 1354 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (PInvokeAttr)yyVals[-3+yyTop] | PInvokeAttr.bestfit_off; }
  break;
case 389:
#line 1355 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (PInvokeAttr)yyVals[-3+yyTop] | PInvokeAttr.charmaperror_on; }
  break;
case 390:
#line 1356 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (PInvokeAttr)yyVals[-3+yyTop] | PInvokeAttr.charmaperror_off; }
  break;
case 394:
#line 1364 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new ParamAttr(); }
  break;
case 395:
#line 1365 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (ParamAttr)yyVals[-3+yyTop] | ParamAttr.In; }
  break;
case 396:
#line 1366 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (ParamAttr)yyVals[-3+yyTop] | ParamAttr.Out; }
  break;
case 397:
#line 1367 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (ParamAttr)yyVals[-3+yyTop] | ParamAttr.Opt; }
  break;
case 398:
#line 1370 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new ImplAttr(); }
  break;
case 399:
#line 1371 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (ImplAttr)yyVals[-1+yyTop] | ImplAttr.Native; }
  break;
case 400:
#line 1372 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (ImplAttr)yyVals[-1+yyTop] | ImplAttr.IL; }
  break;
case 401:
#line 1373 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (ImplAttr)yyVals[-1+yyTop] | ImplAttr.IL; }
  break;
case 402:
#line 1374 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (ImplAttr)yyVals[-1+yyTop] | ImplAttr.Optil; }
  break;
case 403:
#line 1375 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { /* should this reset? */ }
  break;
case 404:
#line 1376 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (ImplAttr)yyVals[-1+yyTop] | ImplAttr.Unmanaged; }
  break;
case 405:
#line 1377 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (ImplAttr)yyVals[-1+yyTop] | ImplAttr.ForwardRef; }
  break;
case 406:
#line 1378 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (ImplAttr)yyVals[-1+yyTop] | ImplAttr.PreserveSig; }
  break;
case 407:
#line 1379 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (ImplAttr)yyVals[-1+yyTop] | ImplAttr.Runtime; }
  break;
case 408:
#line 1380 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (ImplAttr)yyVals[-1+yyTop] | ImplAttr.InternalCall; }
  break;
case 409:
#line 1381 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (ImplAttr)yyVals[-1+yyTop] | ImplAttr.Synchronized; }
  break;
case 410:
#line 1382 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (ImplAttr)yyVals[-1+yyTop] | ImplAttr.NoInlining; }
  break;
case 411:
#line 1383 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (ImplAttr)yyVals[-1+yyTop] | ImplAttr.NoOptimization; }
  break;
case 412:
#line 1384 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (ImplAttr)yyVals[-1+yyTop] | ImplAttr.AggressiveInlining; }
  break;
case 415:
  case_415();
  break;
case 416:
  case_416();
  break;
case 417:
#line 1405 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new ParamDef((ParamAttr)yyVals[-1+yyTop], null, (BaseTypeRef)yyVals[0+yyTop]); }
  break;
case 418:
#line 1406 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new ParamDef((ParamAttr)yyVals[-2+yyTop], (string)yyVals[0+yyTop], (BaseTypeRef)yyVals[-1+yyTop]); }
  break;
case 419:
  case_419();
  break;
case 420:
  case_420();
  break;
case 421:
  case_421();
  break;
case 422:
#line 1426 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new ArrayList(0); }
  break;
case 423:
  case_423();
  break;
case 424:
  case_424();
  break;
case 425:
  case_425();
  break;
case 426:
  case_426();
  break;
case 431:
#line 1462 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.CurrentMethodDef.AddInstr(new EmitByteInstr((int)yyVals[0+yyTop], tokenizer.Location)); }
  break;
case 432:
#line 1463 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.CurrentMethodDef.SetMaxStack((int)yyVals[0+yyTop]); }
  break;
case 433:
  case_433();
  break;
case 434:
  case_434();
  break;
case 435:
  case_435();
  break;
case 436:
#line 1482 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.CurrentMethodDef.ZeroInit(); }
  break;
case 440:
#line 1486 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.CurrentTypeDef.AddOverride(codegen.CurrentMethodDef, (BaseTypeRef)yyVals[-2+yyTop], (string)yyVals[0+yyTop]); }
  break;
case 441:
#line 1487 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.CurrentTypeDef.AddOverride(codegen.CurrentMethodDef.Signature, (BaseMethodRef)yyVals[0+yyTop]); }
  break;
case 442:
  case_442();
  break;
case 444:
  case_444();
  break;
case 446:
#line 1518 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.CurrentMethodDef.AddLabel((string)yyVals[-1+yyTop]); }
  break;
case 449:
#line 1521 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { AddSecDecl(yyVals[0+yyTop], false); }
  break;
case 452:
  case_452();
  break;
case 455:
  case_455();
  break;
case 456:
  case_456();
  break;
case 457:
#line 1546 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new Local(-1, (BaseTypeRef)yyVals[0+yyTop]); }
  break;
case 458:
#line 1547 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new Local(-1, (string)yyVals[0+yyTop], (BaseTypeRef)yyVals[-1+yyTop]); }
  break;
case 459:
#line 1548 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new Local((int)yyVals[-1+yyTop], (BaseTypeRef)yyVals[0+yyTop]); }
  break;
case 460:
#line 1549 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new Local((int)yyVals[-2+yyTop], (string)yyVals[0+yyTop], (BaseTypeRef)yyVals[-1+yyTop]); }
  break;
case 461:
#line 1552 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = yyVals[-1+yyTop]; }
  break;
case 462:
  case_462();
  break;
case 463:
  case_463();
  break;
case 465:
  case_465();
  break;
case 466:
  case_466();
  break;
case 467:
  case_467();
  break;
case 468:
#line 1599 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new TryBlock((HandlerBlock)yyVals[0+yyTop], tokenizer.Location); }
  break;
case 469:
  case_469();
  break;
case 470:
  case_470();
  break;
case 471:
  case_471();
  break;
case 472:
  case_472();
  break;
case 473:
  case_473();
  break;
case 474:
  case_474();
  break;
case 475:
  case_475();
  break;
case 476:
  case_476();
  break;
case 477:
  case_477();
  break;
case 478:
  case_478();
  break;
case 479:
  case_479();
  break;
case 480:
#line 1679 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  {

						  }
  break;
case 481:
  case_481();
  break;
case 482:
  case_482();
  break;
case 483:
#line 1694 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.CurrentMethodDef.AddInstr( new SimpInstr((Op)yyVals[0+yyTop], tokenizer.Location)); }
  break;
case 484:
#line 1695 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.CurrentMethodDef.AddInstr( new IntInstr((IntOp)yyVals[-1+yyTop], (int)yyVals[0+yyTop], tokenizer.Location)); }
  break;
case 485:
  case_485();
  break;
case 486:
#line 1703 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.CurrentMethodDef.AddInstr( new IntInstr((IntOp)yyVals[-1+yyTop], (int)yyVals[0+yyTop], tokenizer.Location)); }
  break;
case 487:
  case_487();
  break;
case 488:
#line 1711 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.CurrentMethodDef.AddInstr(new IntInstr((IntOp)yyVals[-1+yyTop], (int)yyVals[0+yyTop], tokenizer.Location)); }
  break;
case 489:
  case_489();
  break;
case 490:
  case_490();
  break;
case 491:
  case_491();
  break;
case 492:
  case_492();
  break;
case 493:
  case_493();
  break;
case 494:
  case_494();
  break;
case 495:
  case_495();
  break;
case 496:
#line 1783 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.CurrentMethodDef.AddInstr(new MethodInstr((MethodOp)yyVals[-1+yyTop], (BaseMethodRef)yyVals[0+yyTop], tokenizer.Location)); }
  break;
case 497:
  case_497();
  break;
case 498:
  case_498();
  break;
case 499:
#line 1800 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.CurrentMethodDef.AddInstr(new TypeInstr((TypeOp)yyVals[-1+yyTop], (BaseTypeRef)yyVals[0+yyTop], tokenizer.Location)); }
  break;
case 500:
  case_500();
  break;
case 501:
  case_501();
  break;
case 502:
  case_502();
  break;
case 503:
  case_503();
  break;
case 504:
  case_504();
  break;
case 505:
#line 1838 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.CurrentMethodDef.AddInstr(new SwitchInstr((ArrayList)yyVals[-1+yyTop], tokenizer.Location)); }
  break;
case 506:
  case_506();
  break;
case 507:
  case_507();
  break;
case 509:
  case_509();
  break;
case 510:
  case_510();
  break;
case 511:
  case_511();
  break;
case 512:
  case_512();
  break;
case 515:
#line 1912 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = yyVals[0+yyTop]; }
  break;
case 516:
  case_516();
  break;
case 517:
#line 1918 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = codegen.GetGlobalFieldRef((BaseTypeRef)yyVals[-1+yyTop], (string)yyVals[0+yyTop]); }
  break;
case 518:
#line 1921 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.CurrentTypeDef.EndEventDef(); }
  break;
case 519:
  case_519();
  break;
case 521:
#line 1933 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new FeatureAttr(); }
  break;
case 522:
#line 1934 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (FeatureAttr)yyVals[-1+yyTop] & FeatureAttr.Rtspecialname; }
  break;
case 523:
#line 1935 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (FeatureAttr)yyVals[-1+yyTop] & FeatureAttr.Specialname; }
  break;
case 526:
#line 1942 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.CurrentTypeDef.CurrentEvent.AddAddon((MethodRef)yyVals[-1+yyTop]); }
  break;
case 527:
#line 1943 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.CurrentTypeDef.CurrentEvent.AddRemoveon((MethodRef)yyVals[-1+yyTop]); }
  break;
case 528:
#line 1944 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.CurrentTypeDef.CurrentEvent.AddFire((MethodRef)yyVals[-1+yyTop]); }
  break;
case 529:
#line 1945 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.CurrentTypeDef.CurrentEvent.AddOther((MethodRef)yyVals[-1+yyTop]); }
  break;
case 530:
  case_530();
  break;
case 533:
#line 1955 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.CurrentTypeDef.EndPropertyDef(); }
  break;
case 534:
  case_534();
  break;
case 535:
#line 1968 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new FeatureAttr(); }
  break;
case 536:
#line 1969 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (FeatureAttr)yyVals[-1+yyTop] | FeatureAttr.Rtspecialname; }
  break;
case 537:
#line 1970 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (FeatureAttr)yyVals[-1+yyTop] | FeatureAttr.Specialname; }
  break;
case 538:
#line 1971 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (FeatureAttr)yyVals[-1+yyTop] | FeatureAttr.Instance; }
  break;
case 541:
#line 1978 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.CurrentTypeDef.CurrentProperty.AddSet((MethodRef)yyVals[0+yyTop]); }
  break;
case 542:
#line 1979 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.CurrentTypeDef.CurrentProperty.AddGet((MethodRef)yyVals[0+yyTop]); }
  break;
case 543:
#line 1980 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.CurrentTypeDef.CurrentProperty.AddOther((MethodRef)yyVals[0+yyTop]); }
  break;
case 544:
  case_544();
  break;
case 547:
#line 1990 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new CustomAttr((BaseMethodRef)yyVals[0+yyTop], null); }
  break;
case 549:
#line 1992 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new CustomAttr((BaseMethodRef)yyVals[-2+yyTop], new ByteArrConst((byte[])yyVals[0+yyTop])); }
  break;
case 550:
#line 1993 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new CustomAttr((BaseMethodRef)yyVals[-4+yyTop], (PEAPI.Constant)yyVals[-1+yyTop]); }
  break;
case 554:
  case_554();
  break;
case 555:
#line 2007 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new ByteArrConst((byte[])yyVals[0+yyTop]); }
  break;
case 556:
#line 2008 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new StringConst((string)yyVals[-1+yyTop]); }
  break;
case 557:
  case_557();
  break;
case 559:
  case_559();
  break;
case 561:
#line 2028 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new StringConst(((TypeRef)yyVals[-1+yyTop]).FullName); }
  break;
case 562:
  case_562();
  break;
case 563:
  case_563();
  break;
case 564:
#line 2058 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = TypeSpecToPermPair(yyVals[-4+yyTop], yyVals[-3+yyTop], (ArrayList)yyVals[-1+yyTop]); }
  break;
case 565:
#line 2059 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = TypeSpecToPermPair(yyVals[-1+yyTop], yyVals[0+yyTop], null); }
  break;
case 566:
  case_566();
  break;
case 567:
  case_567();
  break;
case 568:
#line 2073 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new MIPermissionSet((PEAPI.SecurityAction)yyVals[-4+yyTop], (ArrayList)yyVals[-1+yyTop]); }
  break;
case 569:
  case_569();
  break;
case 570:
  case_570();
  break;
case 571:
#line 2090 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new MIPermission((BaseTypeRef)yyVals[-4+yyTop], (ArrayList)yyVals[-1+yyTop]); }
  break;
case 572:
  case_572();
  break;
case 573:
  case_573();
  break;
case 574:
  case_574();
  break;
case 575:
  case_575();
  break;
case 576:
#line 2119 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new NameValuePair((string)yyVals[-2+yyTop], (PEAPI.Constant)yyVals[0+yyTop]); }
  break;
case 577:
#line 2122 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = MemberTypes.Property; }
  break;
case 578:
#line 2123 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = MemberTypes.Field; }
  break;
case 579:
  case_579();
  break;
case 580:
  case_580();
  break;
case 581:
#line 2140 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new NameValuePair((string)yyVals[-2+yyTop], yyVals[0+yyTop]); }
  break;
case 584:
#line 2145 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = yyVals[-1+yyTop]; }
  break;
case 586:
#line 2147 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = ClassRefToObject(yyVals[-5+yyTop], (byte)(int)yyVals[-1+yyTop]); }
  break;
case 587:
#line 2148 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = ClassRefToObject(yyVals[-5+yyTop], (short)(int)yyVals[-1+yyTop]); }
  break;
case 588:
#line 2149 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = ClassRefToObject(yyVals[-5+yyTop], (int)yyVals[-1+yyTop]); }
  break;
case 589:
#line 2150 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = ClassRefToObject(yyVals[-3+yyTop], (int)yyVals[-1+yyTop]); }
  break;
case 590:
#line 2153 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = PEAPI.SecurityAction.Request; }
  break;
case 591:
#line 2154 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = PEAPI.SecurityAction.Demand; }
  break;
case 592:
#line 2155 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = PEAPI.SecurityAction.Assert; }
  break;
case 593:
#line 2156 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = PEAPI.SecurityAction.Deny; }
  break;
case 594:
#line 2157 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = PEAPI.SecurityAction.PermitOnly; }
  break;
case 595:
#line 2158 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = PEAPI.SecurityAction.LinkDemand; }
  break;
case 596:
#line 2159 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = PEAPI.SecurityAction.InheritDemand; }
  break;
case 597:
#line 2160 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = PEAPI.SecurityAction.RequestMinimum; }
  break;
case 598:
#line 2161 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = PEAPI.SecurityAction.RequestOptional; }
  break;
case 599:
#line 2162 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = PEAPI.SecurityAction.RequestRefuse; }
  break;
case 600:
#line 2163 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = PEAPI.SecurityAction.PreJitGrant; }
  break;
case 601:
#line 2164 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = PEAPI.SecurityAction.PreJitDeny; }
  break;
case 602:
#line 2165 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = PEAPI.SecurityAction.NonCasDemand; }
  break;
case 603:
#line 2166 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = PEAPI.SecurityAction.NonCasLinkDemand; }
  break;
case 604:
#line 2167 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = PEAPI.SecurityAction.NonCasInheritance; }
  break;
case 605:
#line 2171 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { }
  break;
case 606:
#line 2172 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.SetModuleName((string)yyVals[0+yyTop]); }
  break;
case 607:
#line 2173 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.ExternTable.AddModule((string)yyVals[0+yyTop]); }
  break;
case 608:
#line 2176 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.SetFileRef(new FileRef((string)yyVals[-5+yyTop], (byte [])yyVals[-1+yyTop], (bool)yyVals[-6+yyTop], (bool)yyVals[0+yyTop])); }
  break;
case 609:
  case_609();
  break;
case 610:
#line 2184 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = true; }
  break;
case 611:
#line 2185 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = false; }
  break;
case 612:
#line 2188 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = false; }
  break;
case 613:
#line 2189 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = true; }
  break;
case 614:
  case_614();
  break;
case 615:
  case_615();
  break;
case 616:
#line 2207 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new PEAPI.AssemAttr(); }
  break;
case 617:
#line 2211 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = ((PEAPI.AssemAttr)yyVals[-1+yyTop]) | PEAPI.AssemAttr.Retargetable; }
  break;
case 620:
#line 2218 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.ThisAssembly.SetPublicKey((byte [])yyVals[0+yyTop]); }
  break;
case 621:
#line 2219 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.ThisAssembly.SetVersion((int)yyVals[-6+yyTop], (int)yyVals[-4+yyTop], (int)yyVals[-2+yyTop], (int)yyVals[0+yyTop]); }
  break;
case 622:
#line 2220 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.ThisAssembly.SetLocale((string)yyVals[0+yyTop]); }
  break;
case 624:
#line 2222 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.ThisAssembly.SetHashAlgorithm((int)yyVals[0+yyTop]); }
  break;
case 625:
#line 2223 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.ThisAssembly.AddCustomAttribute((CustomAttr)yyVals[0+yyTop]); }
  break;
case 626:
#line 2224 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { AddSecDecl(yyVals[0+yyTop], true); }
  break;
case 633:
  case_633();
  break;
case 634:
  case_634();
  break;
case 637:
#line 2255 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.CurrentAssemblyRef.SetVersion((int)yyVals[-6+yyTop], (int)yyVals[-4+yyTop], (int)yyVals[-2+yyTop], (int)yyVals[0+yyTop]); }
  break;
case 638:
#line 2256 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.CurrentAssemblyRef.SetPublicKey((byte [])yyVals[0+yyTop]); }
  break;
case 639:
#line 2257 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.CurrentAssemblyRef.SetPublicKeyToken((byte [])yyVals[0+yyTop]); }
  break;
case 640:
#line 2258 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.CurrentAssemblyRef.SetLocale((string)yyVals[0+yyTop]); }
  break;
case 642:
#line 2260 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.CurrentAssemblyRef.SetHash((byte [])yyVals[0+yyTop]); }
  break;
case 643:
  case_643();
  break;
case 646:
#line 2272 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { current_extern = new KeyValuePair<string, TypeAttr>((string)yyVals[0+yyTop], (TypeAttr)yyVals[-1+yyTop]); }
  break;
case 647:
#line 2275 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = 0; }
  break;
case 648:
#line 2276 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (TypeAttr)yyVals[-1+yyTop] | TypeAttr.Private; }
  break;
case 649:
#line 2277 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (TypeAttr)yyVals[-1+yyTop] | TypeAttr.Public; }
  break;
case 650:
#line 2278 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (TypeAttr)yyVals[-1+yyTop] | TypeAttr.Forwarder; }
  break;
case 651:
#line 2279 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (TypeAttr)yyVals[-2+yyTop] | TypeAttr.NestedPublic; }
  break;
case 652:
#line 2280 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (TypeAttr)yyVals[-2+yyTop] | TypeAttr.NestedPrivate; }
  break;
case 653:
#line 2281 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (TypeAttr)yyVals[-2+yyTop] | TypeAttr.NestedFamily; }
  break;
case 654:
#line 2282 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (TypeAttr)yyVals[-2+yyTop] | TypeAttr.NestedAssembly;}
  break;
case 655:
#line 2283 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (TypeAttr)yyVals[-2+yyTop] | TypeAttr.NestedFamAndAssem; }
  break;
case 656:
#line 2284 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (TypeAttr)yyVals[-2+yyTop] | TypeAttr.NestedFamOrAssem; }
  break;
case 662:
#line 2294 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { codegen.ExternTable.AddClass(current_extern.Key, current_extern.Value, (string)yyVals[0+yyTop]); }
  break;
case 664:
  case_664();
  break;
case 666:
#line 2311 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = ManifestResource.PublicResource; }
  break;
case 667:
#line 2312 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = ManifestResource.PrivateResource; }
  break;
case 674:
#line 2325 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = String.Format("{0}{1}", yyVals[-2+yyTop], yyVals[0+yyTop]); }
  break;
case 675:
  case_675();
  break;
case 678:
  case_678();
  break;
case 679:
  case_679();
  break;
case 680:
  case_680();
  break;
case 681:
  case_681();
  break;
case 682:
#line 2364 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { }
  break;
case 683:
#line 2367 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { tokenizer.InByteArray = true; }
  break;
case 684:
  case_684();
  break;
case 685:
#line 2374 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = new byte[0]; }
  break;
case 686:
  case_686();
  break;
case 687:
  case_687();
  break;
case 688:
  case_688();
  break;
case 689:
#line 2395 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = true; }
  break;
case 690:
#line 2396 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = false; }
  break;
case 694:
#line 2404 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
  { yyVal = (string)yyVals[-2+yyTop] + '.' + (string)yyVals[0+yyTop]; }
  break;
#line default
					}
					yyTop -= yyLen[yyN];
					yyState = yyStates[yyTop];
					int yyM = yyLhs[yyN];
					if (yyState == 0 && yyM == 0)
					{
						if (debug != null) debug.shift(0, yyFinal);
						yyState = yyFinal;
						if (yyToken < 0)
						{
							yyToken = yyLex.advance() ? yyLex.token() : 0;
							if (debug != null) debug.lex(yyState, yyToken, yyname(yyToken), yyLex.value());
						}
						if (yyToken == 0)
						{
							if (debug != null) debug.accept(yyVal);
							return yyVal;
						}
						goto continue_yyLoop;
					}
					if ((yyN = yyGindex[yyM]) != 0 && (yyN += yyState) >= 0 && yyN < yyTable.Length && yyCheck[yyN] == yyState)
						yyState = yyTable[yyN];
					else
						yyState = yyDgoto[yyM];
					if (debug != null) debug.shift(yyStates[yyTop], yyState);
					goto continue_yyLoop;
				continue_yyDiscarded:;  // implements the named-loop continue: 'continue yyDiscarded'
				}
			continue_yyLoop:;       // implements the named-loop continue: 'continue yyLoop'
			}
		}
/*
 All more than 3 lines long rules are wrapped into a method
*/
void case_17()
#line 398 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								if (codegen.CurrentCustomAttrTarget != null)
										codegen.CurrentCustomAttrTarget.AddCustomAttribute((CustomAttr)yyVals[0+yyTop]);
						  }

void case_41()
#line 445 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								codegen.BeginTypeDef((TypeAttr)yyVals[-4+yyTop], (string)yyVals[-3+yyTop], yyVals[-1+yyTop] as BaseClassRef, yyVals[0+yyTop] as ArrayList, null, (GenericParameters)yyVals[-2+yyTop]);
								if (is_value_class)
										codegen.CurrentTypeDef.MakeValueClass();
								if (is_enum_class)
										codegen.CurrentTypeDef.MakeEnumClass();
						  }

void case_42()
#line 455 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								/* Reset some flags*/
								is_value_class = false;
								is_enum_class = false;
								yyVal = new TypeAttr();
						  }

void case_71()
#line 496 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var al = new ArrayList();
								al.Add(yyVals[0+yyTop]);
								yyVal = al;
						  }

void case_72()
#line 502 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var al = (ArrayList)yyVals[-2+yyTop];

								al.Add(yyVals[0+yyTop]);
								yyVal = al;
						  }

void case_77()
#line 519 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								GenericArguments ga = new GenericArguments();
								ga.Add((BaseTypeRef)yyVals[0+yyTop]);
								yyVal = ga;
						  }

void case_78()
#line 525 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								((GenericArguments)yyVals[-2+yyTop]).Add((BaseTypeRef)yyVals[0+yyTop]);
								yyVal = yyVals[-2+yyTop];
						  }

void case_81()
#line 537 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var al = new ArrayList();
								al.Add(yyVals[0+yyTop]);
								yyVal = al;
						  }

void case_82()
#line 543 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var al = (ArrayList)yyVals[-2+yyTop];
								al.Add(yyVals[0+yyTop]);
								yyVal = al;
						  }

void case_85()
#line 553 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								if (yyVals[0+yyTop] != null)
										yyVal = ((BaseClassRef)yyVals[-1+yyTop]).GetGenericTypeInst((GenericArguments)yyVals[0+yyTop]);
								else
										yyVal = yyVals[-1+yyTop];
						  }

void case_86()
#line 560 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var gpar = new GenParam((int)yyVals[0+yyTop], "", GenParamType.Var);
								yyVal = new GenericParamRef(gpar, yyVals[0+yyTop].ToString());
						  }

void case_87()
#line 565 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var gpar = new GenParam((int)yyVals[0+yyTop], "", GenParamType.MVar);
								yyVal = new GenericParamRef(gpar, yyVals[0+yyTop].ToString());
						  }

void case_88()
#line 570 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								int num = -1;
								string name = (string)yyVals[0+yyTop];
								if (codegen.CurrentTypeDef != null)
										num = codegen.CurrentTypeDef.GetGenericParamNum(name);
								var gpar = new GenParam(num, name, GenParamType.Var);
								yyVal = new GenericParamRef(gpar, name);
						  }

void case_89()
#line 579 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								int num = -1;
								string name = (string)yyVals[0+yyTop];
								if (codegen.CurrentMethodDef != null)
										num = codegen.CurrentMethodDef.GetGenericParamNum(name);
								var gpar = new GenParam(num, name, GenParamType.MVar);
								yyVal = new GenericParamRef(gpar, name);
						  }

void case_90()
#line 590 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								GenericParameter gp = new GenericParameter((string)yyVals[0+yyTop], (PEAPI.GenericParamAttributes)yyVals[-2+yyTop], (ArrayList)yyVals[-1+yyTop]);
								GenericParameters colln = new GenericParameters();
								colln.Add(gp);
								yyVal = colln;
						  }

void case_91()
#line 597 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								GenericParameters colln = (GenericParameters)yyVals[-4+yyTop];
								colln.Add(new GenericParameter((string)yyVals[0+yyTop], (PEAPI.GenericParamAttributes)yyVals[-2+yyTop], (ArrayList)yyVals[-1+yyTop]));
								yyVal = colln;
						  }

void case_99()
#line 616 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								  if (codegen.CurrentMethodDef != null)
										  codegen.CurrentCustomAttrTarget = codegen.CurrentMethodDef.GetGenericParam((string)yyVals[0+yyTop]);
								  else
										  codegen.CurrentCustomAttrTarget = codegen.CurrentTypeDef.GetGenericParam((string)yyVals[0+yyTop]);
								  if (codegen.CurrentCustomAttrTarget == null)
										  Report.Error(String.Format("Type parameter '{0}' undefined.", (string)yyVals[0+yyTop]));
						  }

void case_100()
#line 625 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								  int index = ((int)yyVals[-1+yyTop]);
								  if (codegen.CurrentMethodDef != null)
										  codegen.CurrentCustomAttrTarget = codegen.CurrentMethodDef.GetGenericParam(index - 1);
								  else
										  codegen.CurrentCustomAttrTarget = codegen.CurrentTypeDef.GetGenericParam(index - 1);
								  if (codegen.CurrentCustomAttrTarget == null)
										  Report.Error(String.Format("Type parameter '{0}' index out of range.", index));
						  }

void case_101()
#line 637 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var class_list = new ArrayList();
								class_list.Add(yyVals[0+yyTop]);
								yyVal = class_list;
						  }

void case_102()
#line 643 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var class_list = (ArrayList)yyVals[-2+yyTop];
								class_list.Add(yyVals[0+yyTop]);
						  }

void case_105()
#line 654 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								if (codegen.IsThisAssembly((string)yyVals[-2+yyTop]))
										yyVal = codegen.GetTypeRef((string)yyVals[0+yyTop]);
								else
										yyVal = codegen.ExternTable.GetTypeRef((string)yyVals[-2+yyTop], (string)yyVals[0+yyTop], false);
						  }

void case_106()
#line 661 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								if (codegen.IsThisModule((string)yyVals[-2+yyTop]))
										yyVal = codegen.GetTypeRef((string)yyVals[0+yyTop]);
								else
										yyVal = codegen.ExternTable.GetModuleTypeRef((string)yyVals[-2+yyTop], (string)yyVals[0+yyTop], false);
						  }

void case_107()
#line 668 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								PrimitiveTypeRef prim = PrimitiveTypeRef.GetPrimitiveType((string)yyVals[0+yyTop]);

								if (prim != null && !codegen.IsThisAssembly("mscorlib"))
										yyVal = prim;
								else
										yyVal = codegen.GetTypeRef((string)yyVals[0+yyTop]);
						  }

void case_118()
#line 691 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								if (codegen.CurrentCustomAttrTarget != null)
										codegen.CurrentCustomAttrTarget.AddCustomAttribute((CustomAttr)yyVals[0+yyTop]);
						  }

void case_122()
#line 699 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
#if true
								throw new NotImplementedException("GCS modified state machine");
#else
								/**/
								/* My copy of the spec does not have a type_list but it seems pretty crucial*/
								/**/
								var owner = (BaseTypeRef)yyVals[-11+yyTop];
								ArrayList arg_list = (ArrayList)yyVals[-2+yyTop];
								BaseTypeRef[] param_list;
								BaseMethodRef decl;

								if (arg_list != null)
										param_list = (BaseTypeRef[])arg_list.ToArray(typeof(BaseTypeRef));
								else
										param_list = new BaseTypeRef[0];

								decl = owner.GetMethodRef((BaseTypeRef)yyVals[-6+yyTop], (CallConv)yyVals[-7+yyTop], (string)yyVals[-9+yyTop], param_list, 0);

								/* NOTICE: owner here might be wrong*/
								string sig = MethodDef.CreateSignature(owner, (CallConv)yyVals[-7+yyTop], (string)yyVals[-3+yyTop], param_list, 0, false);
								codegen.CurrentTypeDef.AddOverride(sig, decl);
#endif
						  }

void case_125()
#line 729 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var class_ref = (BaseClassRef)yyVals[0+yyTop];
								class_ref.MakeValueClass();
								yyVal = GetTypeRef(class_ref);
						  }

void case_126()
#line 735 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var ext_ref = codegen.ExternTable.GetTypeRef((string)yyVals[-3+yyTop], (string)yyVals[-1+yyTop], true);
								if (yyVals[0+yyTop] != null)
										yyVal = ext_ref.GetGenericTypeInst((GenericArguments)yyVals[0+yyTop]);
								else
										yyVal = GetTypeRef(ext_ref);
						  }

void case_127()
#line 743 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var t_ref = codegen.GetTypeRef((string)yyVals[-1+yyTop]);
								t_ref.MakeValueClass();
								if (yyVals[0+yyTop] != null)
										yyVal = t_ref.GetGenericTypeInst((GenericArguments)yyVals[0+yyTop]);
								else
										yyVal = GetTypeRef(t_ref);
						  }

void case_128()
#line 752 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var base_type = (BaseTypeRef)yyVals[-2+yyTop];
								base_type.MakeArray();
								yyVal = base_type;
						  }

void case_129()
#line 758 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var base_type = (BaseTypeRef)yyVals[-3+yyTop];
								var bound_list = (ArrayList)yyVals[-1+yyTop];
								base_type.MakeBoundArray(bound_list);
								yyVal = base_type;
						  }

void case_130()
#line 765 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var base_type = (BaseTypeRef)yyVals[-1+yyTop];
								base_type.MakeManagedPointer();
								yyVal = base_type;
						  }

void case_131()
#line 771 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var base_type = (BaseTypeRef)yyVals[-1+yyTop];
								base_type.MakeUnmanagedPointer();
								yyVal = base_type;
						  }

void case_132()
#line 777 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var base_type = (BaseTypeRef)yyVals[-1+yyTop];
								base_type.MakePinned();
								yyVal = base_type;
						  }

void case_133()
#line 783 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var base_type = (BaseTypeRef)yyVals[-4+yyTop];
								var class_ref = (BaseTypeRef)yyVals[-1+yyTop];
								base_type.MakeCustomModified(codegen, CustomModifier.modreq, class_ref);
								yyVal = base_type;
						  }

void case_134()
#line 790 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var base_type = (BaseTypeRef)yyVals[-4+yyTop];
								var class_ref = (BaseTypeRef)yyVals[-1+yyTop];
								base_type.MakeCustomModified(codegen, CustomModifier.modopt, class_ref);
								yyVal = base_type;
						  }

void case_151()
#line 815 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								/* TODO: Is this the proper full name*/
								yyVal = new PrimitiveTypeRef(PrimitiveType.NativeInt, "System.IntPtr");
						  }

void case_160()
#line 830 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var bound_list = new ArrayList();
								bound_list.Add(yyVals[0+yyTop]);
								yyVal = bound_list;
						  }

void case_161()
#line 836 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var bound_list = (ArrayList)yyVals[-2+yyTop];
								bound_list.Add(yyVals[0+yyTop]);
						  }

void case_162()
#line 843 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								/* This is shortref for no lowerbound or size*/
								yyVal = new DictionaryEntry(TypeRef.Ellipsis, TypeRef.Ellipsis);
						  }

void case_163()
#line 848 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								/* No lower bound or size*/
								yyVal = new DictionaryEntry(TypeRef.Ellipsis, TypeRef.Ellipsis);
						  }

void case_164()
#line 853 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								/* Only size specified */
								int size = (int)yyVals[0+yyTop];
								if (size < 0)
										yyVal = new DictionaryEntry(0, TypeRef.Ellipsis);	/* size cannot be < 0, so emit as (0, ...) ilasm.net emits it like this */
								else
										yyVal = new DictionaryEntry(TypeRef.Ellipsis, size);
						  }

void case_165()
#line 862 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								/* lower and upper bound*/
								int lower = (int)yyVals[-2+yyTop];
								int upper = (int)yyVals[0+yyTop];
								if (lower > upper)
										Report.Error("Lower bound " + lower + " must be <= upper bound " + upper);

								yyVal = new DictionaryEntry(yyVals[-2+yyTop], yyVals[0+yyTop]);
						  }

void case_166()
#line 872 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								/* Just lower bound*/
								yyVal = new DictionaryEntry(yyVals[-1+yyTop], TypeRef.Ellipsis);
						  }

void case_204()
#line 920 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								/*FIXME: Allowed only for methods, !fields*/
								yyVal = new NativeArray((NativeType)yyVals[-5+yyTop], (int)yyVals[-3+yyTop], (int)yyVals[-1+yyTop]);
						  }

void case_205()
#line 925 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								/*FIXME: Allowed only for methods, !fields*/
								yyVal = new NativeArray((NativeType)yyVals[-4+yyTop], -1, (int)yyVals[-1+yyTop]);
						  }

void case_218()
#line 942 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								if (yyVals[0+yyTop] == null)
										yyVal = new SafeArray();
								else
										yyVal = new SafeArray((SafeArrayType)yyVals[0+yyTop]);
						  }

void case_276()
#line 1012 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								FieldDef field_def = new FieldDef((FieldAttr)yyVals[-5+yyTop], (string)yyVals[-3+yyTop], (BaseTypeRef)yyVals[-4+yyTop]);
								codegen.AddFieldDef(field_def);
								codegen.CurrentCustomAttrTarget = field_def;
								if (yyVals[-6+yyTop] != null)
										field_def.SetOffset((uint)(int)yyVals[-6+yyTop]);
								if (yyVals[-2+yyTop] != null)
										field_def.AddDataValue((string)yyVals[-2+yyTop]);
								if (yyVals[-1+yyTop] != null)
										field_def.SetValue((Constant)yyVals[-1+yyTop]);
						  }

void case_291()
#line 1045 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
							   codegen.AddFieldMarshalInfo((NativeType)yyVals[-1+yyTop]);
							   yyVal = (FieldAttr)yyVals[-4+yyTop] | FieldAttr.HasFieldMarshal;
						  }

void case_316()
#line 1084 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								/* ******** THIS IS NOT IN THE DOCUMENTATION ******** //*/
								yyVal = new StringConst((string)yyVals[0+yyTop]);
						  }

void case_322()
#line 1100 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								 var l = (List<BoolConst>)yyVals[-1+yyTop];
								 l.Add(new BoolConst((bool)yyVals[0+yyTop]));
								 yyVal = l;
						  }

void case_323()
#line 1108 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								DataDef datadef = (DataDef)yyVals[-1+yyTop];
								if (yyVals[0+yyTop] is ArrayList)
								{
										var const_list = (ArrayList)yyVals[0+yyTop];
										DataConstant[] const_arr = new DataConstant[const_list.Count];

										for (int i=0; i<const_arr.Length; i++)
												const_arr[i] = (DataConstant)const_list[i];

										datadef.PeapiConstant = new ArrayConstant(const_arr);
								}
								else
								{
										datadef.PeapiConstant = (PEAPI.Constant)yyVals[0+yyTop];
								}
								codegen.AddDataDef(datadef);
						  }

void case_331()
#line 1142 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var dataitem_list = new ArrayList();
								dataitem_list.Add(yyVals[0+yyTop]);
								yyVal = dataitem_list;
						  }

void case_332()
#line 1148 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var list = (ArrayList)yyVals[-2+yyTop];
								list.Add(yyVals[0+yyTop]);
						  }

void case_335()
#line 1157 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								/*DataDef def = codegen.CurrentTypeDef.GetDataDef((string)$3);*/
								/*$$ = new AddressConstant((DataConstant)def.PeapiConstant);*/
						  }

void case_337()
#line 1163 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								/* ******** THIS IS NOT IN THE SPECIFICATION ******** //*/
								yyVal = new ByteArrConst((byte[])yyVals[0+yyTop]);
						  }

void case_338()
#line 1168 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								double d = (double)yyVals[-2+yyTop];
								FloatConst float_const = new FloatConst((float)d);
								if (yyVals[0+yyTop] != null)
										yyVal = new RepeatedConstant(float_const, (int)yyVals[0+yyTop]);
								else
										yyVal = float_const;
						  }

void case_339()
#line 1177 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								DoubleConst double_const = new DoubleConst((double)yyVals[-2+yyTop]);
								if (yyVals[0+yyTop] != null)
										yyVal = new RepeatedConstant(double_const, (int)yyVals[0+yyTop]);
								else
										yyVal = double_const;
						  }

void case_340()
#line 1185 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								IntConst int_const = new IntConst((long)yyVals[-2+yyTop]);
								if (yyVals[0+yyTop] != null)
										yyVal = new RepeatedConstant(int_const, (int)yyVals[0+yyTop]);
								else
										yyVal = int_const;
						  }

void case_341()
#line 1193 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								IntConst int_const = new IntConst((int)yyVals[-2+yyTop]);
								if (yyVals[0+yyTop] != null)
										yyVal = new RepeatedConstant(int_const, (int)yyVals[0+yyTop]);
								else
										yyVal = int_const;
						  }

void case_342()
#line 1201 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								int i = (int)yyVals[-2+yyTop];
								IntConst int_const = new IntConst((short)i);
								if (yyVals[0+yyTop] != null)
										yyVal = new RepeatedConstant(int_const, (int)yyVals[0+yyTop]);
								else
										yyVal = int_const;
						  }

void case_343()
#line 1210 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								int i = (int)yyVals[-2+yyTop];
								IntConst int_const = new IntConst((sbyte)i);
								if (yyVals[0+yyTop] != null)
										yyVal = new RepeatedConstant(int_const, (int)yyVals[0+yyTop]);
								else
										yyVal = int_const;
						  }

void case_344()
#line 1219 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								FloatConst float_const = new FloatConst(0F);
								if (yyVals[0+yyTop] != null)
										yyVal = new RepeatedConstant(float_const, (int)yyVals[0+yyTop]);
								else
										yyVal = float_const;
						  }

void case_345()
#line 1227 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								DoubleConst double_const = new DoubleConst(0);
								if (yyVals[0+yyTop] != null)
										yyVal = new RepeatedConstant(double_const, (int)yyVals[0+yyTop]);
								else
										yyVal = double_const;
						  }

void case_346()
#line 1235 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								IntConst int_const = new IntConst((long)0);
								if (yyVals[0+yyTop] != null)
										yyVal = new RepeatedConstant(int_const, (int)yyVals[0+yyTop]);
								else
										yyVal = int_const;
						  }

void case_347()
#line 1243 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								IntConst int_const = new IntConst((int)0);
								if (yyVals[0+yyTop] != null)
										yyVal = new RepeatedConstant(int_const, (int)yyVals[0+yyTop]);
								else
										yyVal = int_const;
						  }

void case_348()
#line 1251 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								IntConst int_const = new IntConst((short)0);
								if (yyVals[0+yyTop] != null)
										yyVal = new RepeatedConstant(int_const, (int)yyVals[0+yyTop]);
								else
										yyVal = int_const;
						  }

void case_349()
#line 1259 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								IntConst int_const = new IntConst((sbyte)0);
								if (yyVals[0+yyTop] != null)
										yyVal = new RepeatedConstant(int_const, (int)yyVals[0+yyTop]);
								else
										yyVal = int_const;
						  }

void case_351()
#line 1272 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								CallConv cc = (CallConv)yyVals[-8+yyTop];
								if (yyVals[-4+yyTop] != null)
										cc |= CallConv.Generic;

								MethodDef methdef = new MethodDef( codegen, (MethAttr)yyVals[-9+yyTop], cc, (ImplAttr)yyVals[0+yyTop], (string)yyVals[-5+yyTop], (BaseTypeRef)yyVals[-6+yyTop], (ArrayList)yyVals[-2+yyTop], tokenizer.Reader.Location, (GenericParameters)yyVals[-4+yyTop], codegen.CurrentTypeDef);
								if (pinvoke_info)
								{
										ExternModule mod = codegen.ExternTable.AddModule(pinvoke_mod);
										methdef.AddPInvokeInfo(pinvoke_attr, mod, pinvoke_meth);
										pinvoke_info = false;
								}
						  }

void case_352()
#line 1286 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								MethodDef methdef = new MethodDef( codegen, (MethAttr)yyVals[-12+yyTop], (CallConv)yyVals[-11+yyTop], (ImplAttr)yyVals[0+yyTop], (string)yyVals[-4+yyTop], (BaseTypeRef)yyVals[-9+yyTop], (ArrayList)yyVals[-2+yyTop], tokenizer.Reader.Location, null, codegen.CurrentTypeDef);

								if (pinvoke_info)
								{
										ExternModule mod = codegen.ExternTable.AddModule(pinvoke_mod);
										methdef.AddPInvokeInfo(pinvoke_attr, mod, pinvoke_meth);
										pinvoke_info = false;
								}
								methdef.AddRetTypeMarshalInfo((NativeType)yyVals[-6+yyTop]);
						  }

void case_373()
#line 1320 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								pinvoke_info = true;
								pinvoke_mod = (string)yyVals[-4+yyTop];
								pinvoke_meth = (string)yyVals[-2+yyTop];
								pinvoke_attr = (PInvokeAttr)yyVals[-1+yyTop];
						  }

void case_374()
#line 1327 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								pinvoke_info = true;
								pinvoke_mod = (string)yyVals[-2+yyTop];
								pinvoke_meth = null;
								pinvoke_attr = (PInvokeAttr)yyVals[-1+yyTop];
						  }

void case_375()
#line 1334 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								pinvoke_info = true;
								pinvoke_mod = null;
								pinvoke_meth = null;
								pinvoke_attr = (PInvokeAttr)yyVals[-1+yyTop];
						  }

void case_415()
#line 1392 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var sig_list = new ArrayList();
								sig_list.Add(yyVals[0+yyTop]);
								yyVal = sig_list;
						  }

void case_416()
#line 1398 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var sig_list =(ArrayList)yyVals[-2+yyTop];
								sig_list.Add(yyVals[0+yyTop]);
								yyVal = sig_list;
						  }

void case_419()
#line 1408 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								yyVal = new ParamDef((ParamAttr)0, "...", new SentinelTypeRef());
								/*$$ = ParamDef.Ellipsis;*/
						  }

void case_420()
#line 1413 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								ParamDef param_def = new ParamDef((ParamAttr)yyVals[-5+yyTop], null, (BaseTypeRef)yyVals[-4+yyTop]);
								param_def.AddMarshalInfo((PEAPI.NativeType)yyVals[-1+yyTop]);
								yyVal = param_def;
						  }

void case_421()
#line 1419 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								ParamDef param_def = new ParamDef((ParamAttr)yyVals[-6+yyTop], (string)yyVals[0+yyTop], (BaseTypeRef)yyVals[-5+yyTop]);
								param_def.AddMarshalInfo((PEAPI.NativeType)yyVals[-2+yyTop]);
								yyVal = param_def;
						  }

void case_423()
#line 1428 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var type_list = new ArrayList();
								/* type_list.Add(TypeRef.Ellipsis);*/
								type_list.Add(new SentinelTypeRef());
								yyVal = type_list;
						  }

void case_424()
#line 1435 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var type_list = (ArrayList)yyVals[-2+yyTop];
								/* type_list.Add(TypeRef.Ellipsis);*/
								type_list.Add(new SentinelTypeRef());
								yyVal = type_list;
						  }

void case_425()
#line 1442 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var type_list = new ArrayList();
								type_list.Add(yyVals[-1+yyTop]);
								yyVal = type_list;
						  }

void case_426()
#line 1448 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var type_list = (ArrayList)yyVals[-4+yyTop];
								type_list.Add(yyVals[-1+yyTop]);
						  }

void case_433()
#line 1465 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								if (yyVals[-1+yyTop] != null)
										codegen.CurrentMethodDef.AddLocals((ArrayList)yyVals[-1+yyTop]);
						  }

void case_434()
#line 1470 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								if (yyVals[-1+yyTop] != null)
								{
										codegen.CurrentMethodDef.AddLocals((ArrayList)yyVals[-1+yyTop]);
										codegen.CurrentMethodDef.InitLocals();
								}
						  }

void case_435()
#line 1478 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								codegen.CurrentMethodDef.EntryPoint();
								codegen.HasEntryPoint = true;
						  }

void case_442()
#line 1489 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var owner = (BaseTypeRef)yyVals[-10+yyTop];
								var arg_list = (ArrayList)yyVals[-1+yyTop];
								BaseTypeRef[] param_list;
								BaseMethodRef methref;

								if (arg_list != null)
										param_list = (BaseTypeRef[])arg_list.ToArray(typeof(BaseTypeRef));
								else
										param_list = new BaseTypeRef[0];

								if (owner.UseTypeSpec)
										methref = new TypeSpecMethodRef(owner, (CallConv)yyVals[-12+yyTop], (BaseTypeRef)yyVals[-11+yyTop], (string)yyVals[-8+yyTop], param_list, (int)yyVals[-5+yyTop]);
								else
										methref = owner.GetMethodRef((BaseTypeRef)yyVals[-11+yyTop], (CallConv)yyVals[-12+yyTop], (string)yyVals[-8+yyTop], param_list, (int)yyVals[-5+yyTop]);
								codegen.CurrentTypeDef.AddOverride(codegen.CurrentMethodDef.Signature, methref);
						  }

void case_444()
#line 1508 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								int index = (int)yyVals[-2+yyTop];
								ParamDef param = codegen.CurrentMethodDef.GetParam(index);
								codegen.CurrentCustomAttrTarget = param;
								if (param == null)
										Report.Warning(tokenizer.Location, String.Format("invalid param index ({0}) with .param", index));
								else if (yyVals[0+yyTop] != null)
										param.AddDefaultValue((Constant)yyVals[0+yyTop]);
						  }

void case_452()
#line 1525 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								if (codegen.CurrentCustomAttrTarget != null)
										codegen.CurrentCustomAttrTarget.AddCustomAttribute((CustomAttr)yyVals[0+yyTop]);
						  }

void case_455()
#line 1534 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var local_list = new ArrayList();
								local_list.Add(yyVals[0+yyTop]);
								yyVal = local_list;
						  }

void case_456()
#line 1540 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var local_list = (ArrayList)yyVals[-2+yyTop];
								local_list.Add(yyVals[0+yyTop]);
						  }

void case_462()
#line 1556 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								/* This is a reference to a global method in another*/
								/* assembly. This is not supported in the MS version of ilasm*/
						  }

void case_463()
#line 1561 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								string module = (string)yyVals[-1+yyTop];
								if (codegen.IsThisModule(module))
								{
									/* This is not handled yet.*/
								}
								else
								{
									yyVal = codegen.ExternTable.GetModuleTypeRef((string)yyVals[-1+yyTop], "<Module>", false);
								}
						  }

void case_465()
#line 1576 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								yyVal = new HandlerBlock((LabelInfo)yyVals[-2+yyTop], codegen.CurrentMethodDef.AddLabel());
								codegen.CurrentMethodDef.EndLocalsScope();
						  }

void case_466()
#line 1583 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								yyVal = codegen.CurrentMethodDef.AddLabel();
								codegen.CurrentMethodDef.BeginLocalsScope();
						  }

void case_467()
#line 1590 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								TryBlock try_block = (TryBlock)yyVals[-1+yyTop];
								var clause_list = (ArrayList)yyVals[0+yyTop];
								foreach (object clause in clause_list)
										try_block.AddSehClause((ISehClause)clause);
								codegen.CurrentMethodDef.AddInstr(try_block);
						  }

void case_469()
#line 1601 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								LabelInfo from = codegen.CurrentMethodDef.AddLabelRef((string)yyVals[-2+yyTop]);
								LabelInfo to = codegen.CurrentMethodDef.AddLabelRef((string)yyVals[0+yyTop]);
								yyVal = new TryBlock(new HandlerBlock(from, to), tokenizer.Location);
						  }

void case_470()
#line 1607 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								LabelInfo from = codegen.CurrentMethodDef.AddLabel((int)yyVals[-2+yyTop]);
								LabelInfo to = codegen.CurrentMethodDef.AddLabel((int)yyVals[0+yyTop]);
								yyVal = new TryBlock(new HandlerBlock(from, to), tokenizer.Location);
						  }

void case_471()
#line 1615 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var clause_list = new ArrayList();
								clause_list.Add(yyVals[0+yyTop]);
								yyVal = clause_list;
						  }

void case_472()
#line 1621 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var clause_list = (ArrayList)yyVals[-1+yyTop];
								clause_list.Add(yyVals[0+yyTop]);
						  }

void case_473()
#line 1628 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								if (yyVals[-1+yyTop].GetType() == typeof(PrimitiveTypeRef))
										Report.Error("Exception not be of a primitive type.");

								var type = (BaseTypeRef)yyVals[-1+yyTop];
								CatchBlock cb = new CatchBlock(type);
								cb.SetHandlerBlock((HandlerBlock)yyVals[0+yyTop]);
								yyVal = cb;
						  }

void case_474()
#line 1638 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								FinallyBlock fb = new FinallyBlock();
								fb.SetHandlerBlock((HandlerBlock)yyVals[0+yyTop]);
								yyVal = fb;
						  }

void case_475()
#line 1644 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								FaultBlock fb = new FaultBlock();
								fb.SetHandlerBlock((HandlerBlock)yyVals[0+yyTop]);
								yyVal = fb;
						  }

void case_476()
#line 1650 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								FilterBlock fb = (FilterBlock)yyVals[-1+yyTop];
								fb.SetHandlerBlock((HandlerBlock)yyVals[0+yyTop]);
						  }

void case_477()
#line 1657 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								HandlerBlock block = (HandlerBlock)yyVals[0+yyTop];
								FilterBlock fb = new FilterBlock(block);
								yyVal = fb;
						  }

void case_478()
#line 1663 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								LabelInfo from = codegen.CurrentMethodDef.AddLabelRef((string)yyVals[0+yyTop]);
								FilterBlock fb = new FilterBlock(new HandlerBlock(from, null));
								yyVal = fb;
						  }

void case_479()
#line 1669 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								LabelInfo from = codegen.CurrentMethodDef.AddLabel((int)yyVals[0+yyTop]);
								FilterBlock fb = new FilterBlock(new HandlerBlock(from, null));
								yyVal = fb;
						  }

void case_481()
#line 1681 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								LabelInfo from = codegen.CurrentMethodDef.AddLabelRef((string)yyVals[-2+yyTop]);
								LabelInfo to = codegen.CurrentMethodDef.AddLabelRef((string)yyVals[0+yyTop]);
								yyVal = new HandlerBlock(from, to);
						  }

void case_482()
#line 1687 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								LabelInfo from = codegen.CurrentMethodDef.AddLabel((int)yyVals[-2+yyTop]);
								LabelInfo to = codegen.CurrentMethodDef.AddLabel((int)yyVals[0+yyTop]);
								yyVal = new HandlerBlock(from, to);
						  }

void case_485()
#line 1697 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								int slot = codegen.CurrentMethodDef.GetNamedLocalSlot((string)yyVals[0+yyTop]);
								if (slot < 0)
										Report.Error(String.Format("Undeclared identifier '{0}'", (string)yyVals[0+yyTop]));
								codegen.CurrentMethodDef.AddInstr( new IntInstr((IntOp)yyVals[-1+yyTop], slot, tokenizer.Location));
						  }

void case_487()
#line 1705 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								int pos = codegen.CurrentMethodDef.GetNamedParamPos((string)yyVals[0+yyTop]);
								if (pos < 0)
										Report.Error(String.Format("Undeclared identifier '{0}'", (string)yyVals[0+yyTop]));
								codegen.CurrentMethodDef.AddInstr( new IntInstr((IntOp)yyVals[-1+yyTop], pos, tokenizer.Location));
						  }

void case_489()
#line 1713 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								int slot = codegen.CurrentMethodDef.GetNamedLocalSlot((string)yyVals[0+yyTop]);
								if (slot < 0)
										Report.Error(String.Format("Undeclared identifier '{0}'", (string)yyVals[0+yyTop]));
								codegen.CurrentMethodDef.AddInstr(new IntInstr((IntOp)yyVals[-1+yyTop], slot, tokenizer.Location));
						  }

void case_490()
#line 1720 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								if (yyVals[-1+yyTop] is MiscInstr)
								{
										switch ((MiscInstr)yyVals[-1+yyTop])
										{
										case MiscInstr.ldc_i8:
										codegen.CurrentMethodDef.AddInstr(new LdcInstr((MiscInstr)yyVals[-1+yyTop], (long)yyVals[0+yyTop], tokenizer.Location));
										break;
										}
								}
						  }

void case_491()
#line 1732 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								switch ((MiscInstr)yyVals[-1+yyTop])
								{
								case MiscInstr.ldc_r4:
								case MiscInstr.ldc_r8:
										 codegen.CurrentMethodDef.AddInstr(new LdcInstr((MiscInstr)yyVals[-1+yyTop], (double)yyVals[0+yyTop], tokenizer.Location));
										 break;
								}
						  }

void case_492()
#line 1742 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								long l = (long)yyVals[0+yyTop];
								switch ((MiscInstr)yyVals[-1+yyTop])
								{
										case MiscInstr.ldc_r4:
										case MiscInstr.ldc_r8:
										codegen.CurrentMethodDef.AddInstr(new LdcInstr((MiscInstr)yyVals[-1+yyTop], (double)l, tokenizer.Location));
										break;
								}
						  }

void case_493()
#line 1753 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								byte[] fpdata;
								switch ((MiscInstr)yyVals[-1+yyTop])
								{
										case MiscInstr.ldc_r4:
												fpdata = (byte[])yyVals[0+yyTop];
												if (!BitConverter.IsLittleEndian)
														System.Array.Reverse(fpdata, 0, 4);
												float s = BitConverter.ToSingle(fpdata, 0);
												codegen.CurrentMethodDef.AddInstr(new LdcInstr((MiscInstr)yyVals[-1+yyTop], s, tokenizer.Location));
												break;
										case MiscInstr.ldc_r8:
												fpdata = (byte [])yyVals[0+yyTop];
												if (!BitConverter.IsLittleEndian)
														System.Array.Reverse(fpdata, 0, 8);
												double d = BitConverter.ToDouble(fpdata, 0);
												codegen.CurrentMethodDef.AddInstr(new LdcInstr((MiscInstr)yyVals[-1+yyTop], d, tokenizer.Location));
												break;
								}
						  }

void case_494()
#line 1774 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								LabelInfo target = codegen.CurrentMethodDef.AddLabel((int)yyVals[0+yyTop]);
								codegen.CurrentMethodDef.AddInstr(new BranchInstr((BranchOp)yyVals[-1+yyTop], target, tokenizer.Location));
						  }

void case_495()
#line 1779 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								LabelInfo target = codegen.CurrentMethodDef.AddLabelRef((string)yyVals[0+yyTop]);
								codegen.CurrentMethodDef.AddInstr(new BranchInstr((BranchOp)yyVals[-1+yyTop], target, tokenizer.Location));
						  }

void case_497()
#line 1785 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{

								var owner = (BaseTypeRef)yyVals[-2+yyTop];
								GenericParamRef gpr = yyVals[-3+yyTop] as GenericParamRef;
								if (gpr != null && codegen.CurrentMethodDef != null)
										codegen.CurrentMethodDef.ResolveGenParam((PEAPI.GenParam)gpr.PeapiType);
								IFieldRef fieldref = owner.GetFieldRef((BaseTypeRef)yyVals[-3+yyTop], (string)yyVals[0+yyTop]);

								codegen.CurrentMethodDef.AddInstr(new FieldInstr((FieldOp)yyVals[-4+yyTop], fieldref, tokenizer.Location));
						  }

void case_498()
#line 1796 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								GlobalFieldRef fieldref = codegen.GetGlobalFieldRef((BaseTypeRef)yyVals[-1+yyTop], (string)yyVals[0+yyTop]);
								codegen.CurrentMethodDef.AddInstr(new FieldInstr((FieldOp)yyVals[-2+yyTop], fieldref, tokenizer.Location));
						  }

void case_500()
#line 1802 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								if ((MiscInstr)yyVals[-1+yyTop] == MiscInstr.ldstr)
										codegen.CurrentMethodDef.AddInstr(new LdstrInstr((string)yyVals[0+yyTop], tokenizer.Location));
						  }

void case_501()
#line 1807 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								byte[] bs = (byte[])yyVals[0+yyTop];
								if ((MiscInstr)yyVals[-3+yyTop] == MiscInstr.ldstr)
										codegen.CurrentMethodDef.AddInstr(new LdstrInstr(bs, tokenizer.Location));
						  }

void case_502()
#line 1813 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								byte[] bs = (byte[])yyVals[0+yyTop];
								if ((MiscInstr)yyVals[-2+yyTop] == MiscInstr.ldstr)
										codegen.CurrentMethodDef.AddInstr(new LdstrInstr(bs, tokenizer.Location));
						  }

void case_503()
#line 1819 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var arg_list = (ArrayList)yyVals[-1+yyTop];
								BaseTypeRef[] arg_array = null;
								if (arg_list != null)
										arg_array = (BaseTypeRef[])arg_list.ToArray(typeof(BaseTypeRef));
								codegen.CurrentMethodDef.AddInstr(new CalliInstr((CallConv)yyVals[-4+yyTop], (BaseTypeRef)yyVals[-3+yyTop], arg_array, tokenizer.Location));
						  }

void case_504()
#line 1827 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								if ((MiscInstr)yyVals[-1+yyTop] == MiscInstr.ldtoken)
								{
										if (yyVals[0+yyTop] is BaseMethodRef)
												codegen.CurrentMethodDef.AddInstr(new LdtokenInstr((BaseMethodRef)yyVals[0+yyTop], tokenizer.Location));
										else if (yyVals[0+yyTop] is IFieldRef)
												codegen.CurrentMethodDef.AddInstr(new LdtokenInstr((IFieldRef)yyVals[0+yyTop], tokenizer.Location));
										else
												codegen.CurrentMethodDef.AddInstr(new LdtokenInstr((BaseTypeRef)yyVals[0+yyTop], tokenizer.Location));
								}
						  }

void case_506()
#line 1842 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var arg_list = (ArrayList)yyVals[-1+yyTop];
								GenericArguments ga = (GenericArguments)yyVals[-3+yyTop];
								BaseTypeRef[] param_list;
								if (arg_list != null)
										param_list = (BaseTypeRef[])arg_list.ToArray(typeof(BaseTypeRef));
								else
										param_list = new BaseTypeRef[0];
								BaseMethodRef methref = codegen.GetGlobalMethodRef((BaseTypeRef)yyVals[-5+yyTop], (CallConv)yyVals[-6+yyTop], (string)yyVals[-4+yyTop], param_list, (ga != null ? ga.Count : 0));
								if (ga != null)
										methref = methref.GetGenericMethodRef(ga);
								yyVal = methref;
						  }

void case_507()
#line 1856 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var owner = (BaseTypeRef)yyVals[-6+yyTop];
								var arg_list = (ArrayList)yyVals[-1+yyTop];
								GenericArguments ga = (GenericArguments)yyVals[-3+yyTop];
								BaseTypeRef[] param_list;
								BaseMethodRef methref;
								if (arg_list != null)
										param_list = (BaseTypeRef[])arg_list.ToArray(typeof(BaseTypeRef));
								else
										param_list = new BaseTypeRef[0];
								if (codegen.IsThisAssembly("mscorlib"))
								{
										PrimitiveTypeRef prim = owner as PrimitiveTypeRef;
										if (prim != null && prim.SigMod == "")
												owner = codegen.GetTypeRef(prim.Name);
								}
								if (owner.UseTypeSpec)
										methref = new TypeSpecMethodRef(owner, (CallConv)yyVals[-8+yyTop], (BaseTypeRef)yyVals[-7+yyTop], (string)yyVals[-4+yyTop], param_list, (ga != null ? ga.Count : 0));
								else
										methref = owner.GetMethodRef((BaseTypeRef)yyVals[-7+yyTop], (CallConv)yyVals[-8+yyTop], (string)yyVals[-4+yyTop], param_list, (ga != null ? ga.Count : 0));

								if (ga != null)
										methref = methref.GetGenericMethodRef(ga);
								yyVal = methref;
						  }

void case_509()
#line 1885 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var label_list = new ArrayList();
								label_list.Add(codegen.CurrentMethodDef.AddLabelRef((string)yyVals[0+yyTop]));
								yyVal = label_list;
						  }

void case_510()
#line 1891 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var label_list = new ArrayList();
								label_list.Add(yyVals[0+yyTop]);
								yyVal = label_list;
						  }

void case_511()
#line 1897 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var label_list = (ArrayList)yyVals[-2+yyTop];
								label_list.Add(codegen.CurrentMethodDef.AddLabelRef((string)yyVals[0+yyTop]));
						  }

void case_512()
#line 1902 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var label_list = (ArrayList)yyVals[-2+yyTop];
								label_list.Add(yyVals[0+yyTop]);
						  }

void case_516()
#line 1914 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var owner = (BaseTypeRef)yyVals[-2+yyTop];
								yyVal = owner.GetFieldRef((BaseTypeRef)yyVals[-3+yyTop], (string)yyVals[0+yyTop]);
						  }

void case_519()
#line 1925 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								EventDef event_def = new EventDef((FeatureAttr)yyVals[-2+yyTop], (BaseTypeRef)yyVals[-1+yyTop], (string)yyVals[0+yyTop]);
								codegen.CurrentTypeDef.BeginEventDef(event_def);
								codegen.CurrentCustomAttrTarget = event_def;
						  }

void case_530()
#line 1947 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								if (codegen.CurrentCustomAttrTarget != null)
										codegen.CurrentCustomAttrTarget.AddCustomAttribute((CustomAttr)yyVals[0+yyTop]);
						  }

void case_534()
#line 1959 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								PropertyDef prop_def = new PropertyDef((FeatureAttr)yyVals[-6+yyTop], (BaseTypeRef)yyVals[-5+yyTop], (string)yyVals[-4+yyTop], (ArrayList)yyVals[-2+yyTop]);
								codegen.CurrentTypeDef.BeginPropertyDef(prop_def);
								codegen.CurrentCustomAttrTarget = prop_def;
								if (yyVals[0+yyTop] != null)
										prop_def.AddInitValue((Constant)yyVals[0+yyTop]);
						  }

void case_544()
#line 1982 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								if (codegen.CurrentCustomAttrTarget != null)
										codegen.CurrentCustomAttrTarget.AddCustomAttribute((CustomAttr)yyVals[0+yyTop]);
						  }

void case_554()
#line 2003 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var l = (List<BoolConst>)yyVals[-1+yyTop];
								yyVal = new ArrayConstant(l?.ToArray()) { ExplicitSize = (int)yyVals[-4+yyTop] };
						  }

void case_557()
#line 2010 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var c = yyVals[0+yyTop] as Constant;
								yyVal = c ?? new ArrayConstant(((List<DataConstant>)yyVals[0+yyTop]).ToArray());
						  }

void case_559()
#line 2018 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var l = yyVals[-1+yyTop] as List<DataConstant>;
								if (l == null)
										l = new List<DataConstant>() { (DataConstant)yyVals[-1+yyTop] };
								l.Add((DataConstant)yyVals[0+yyTop]);
								yyVal = l;
						  }

void case_562()
#line 2032 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var owner = (BaseTypeRef)yyVals[-5+yyTop];
								var arg_list = (ArrayList)yyVals[-1+yyTop];
								BaseTypeRef[] param_list;

								if (arg_list != null)
										param_list = (BaseTypeRef[])arg_list.ToArray(typeof(BaseTypeRef));
								else
										param_list = new BaseTypeRef[0];

								yyVal = owner.GetMethodRef((BaseTypeRef)yyVals[-6+yyTop], (CallConv)yyVals[-7+yyTop], (string)yyVals[-3+yyTop], param_list, 0);
						  }

void case_563()
#line 2045 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var arg_list = (ArrayList)yyVals[-1+yyTop];
								BaseTypeRef[] param_list;

								if (arg_list != null)
										param_list = (BaseTypeRef[])arg_list.ToArray(typeof(BaseTypeRef));
								else
										param_list = new BaseTypeRef[0];

								yyVal = codegen.GetGlobalMethodRef((BaseTypeRef)yyVals[-4+yyTop], (CallConv)yyVals[-5+yyTop], (string)yyVals[-3+yyTop], param_list, 0);
						  }

void case_566()
#line 2061 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								System.Text.UnicodeEncoding ue = new System.Text.UnicodeEncoding();
								PermissionSetAttribute psa = new PermissionSetAttribute((System.Security.Permissions.SecurityAction)(short)yyVals[-2+yyTop]);
								psa.XML = ue.GetString((byte [])yyVals[0+yyTop]);
								yyVal = new PermPair((PEAPI.SecurityAction)yyVals[-2+yyTop], psa.CreatePermissionSet());
						  }

void case_567()
#line 2068 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								PermissionSetAttribute psa = new PermissionSetAttribute((System.Security.Permissions.SecurityAction)(short)yyVals[-1+yyTop]);
								psa.XML = (string)yyVals[0+yyTop];
								yyVal = new PermPair((PEAPI.SecurityAction)yyVals[-1+yyTop], psa.CreatePermissionSet());
						  }

void case_569()
#line 2077 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var list = new ArrayList();
								list.Add(yyVals[0+yyTop]);
								yyVal = list;
						  }

void case_570()
#line 2083 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var list = (ArrayList)yyVals[-2+yyTop];
								list.Add(yyVals[0+yyTop]);
								yyVal = list;
						  }

void case_572()
#line 2094 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var list = new ArrayList();
								list.Add(yyVals[0+yyTop]);
								yyVal = list;
						  }

void case_573()
#line 2100 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var list = (ArrayList)yyVals[-1+yyTop];
								list.Add(yyVals[0+yyTop]);
								yyVal = list;
						  }

void case_574()
#line 2108 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var pair = (NameValuePair)yyVals[0+yyTop];
								yyVal = new PermissionMember((MemberTypes)yyVals[-2+yyTop], (BaseTypeRef)yyVals[-1+yyTop], pair.Name, pair.Value);
						  }

void case_575()
#line 2113 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var pair = (NameValuePair)yyVals[0+yyTop];
								yyVal = new PermissionMember((MemberTypes)yyVals[-3+yyTop], (BaseTypeRef)yyVals[-1+yyTop], pair.Name, pair.Value);
						  }

void case_579()
#line 2127 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var pairs = new ArrayList();
								pairs.Add(yyVals[0+yyTop]);
								yyVal = pairs;
						  }

void case_580()
#line 2133 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var pairs = (ArrayList)yyVals[-2+yyTop];
								pairs.Add(yyVals[0+yyTop]);
								yyVal = pairs;
						  }

void case_609()
#line 2178 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								/* We need to compute the hash ourselves. :-(*/
								/* AssemblyName an = AssemblyName.GetName((string)$3);*/
						  }

void case_614()
#line 2193 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								codegen.CurrentCustomAttrTarget = null;
								codegen.CurrentDeclSecurityTarget = null;
						  }

void case_615()
#line 2200 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								codegen.SetThisAssembly((string)yyVals[0+yyTop], (PEAPI.AssemAttr)yyVals[-1+yyTop]);
								codegen.CurrentCustomAttrTarget = codegen.ThisAssembly;
								codegen.CurrentDeclSecurityTarget = codegen.ThisAssembly;
						  }

void case_633()
#line 2238 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								System.Reflection.AssemblyName asmb_name = new System.Reflection.AssemblyName();
								asmb_name.Name = (string)yyVals[0+yyTop];
								codegen.BeginAssemblyRef((string)yyVals[0+yyTop], asmb_name, (PEAPI.AssemAttr)yyVals[-1+yyTop]);
						  }

void case_634()
#line 2244 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								System.Reflection.AssemblyName asmb_name = new System.Reflection.AssemblyName();
								asmb_name.Name = (string)yyVals[-2+yyTop];
								codegen.BeginAssemblyRef((string)yyVals[0+yyTop], asmb_name, (PEAPI.AssemAttr)yyVals[-3+yyTop]);
						  }

void case_643()
#line 2262 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								if (codegen.CurrentCustomAttrTarget != null)
										codegen.CurrentCustomAttrTarget.AddCustomAttribute((CustomAttr)yyVals[0+yyTop]);
						  }

void case_664()
#line 2301 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								FileStream s = new FileStream((string)yyVals[0+yyTop], FileMode.Open, FileAccess.Read);
								byte [] buff = new byte [s.Length];
								s.Read(buff, 0, (int)s.Length);
								s.Close();
								codegen.AddManifestResource(new ManifestResource((string)yyVals[0+yyTop], buff, yyVals[-1+yyTop] == null ? 0 : (uint)yyVals[-1+yyTop]));
						  }

void case_675()
#line 2329 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								long l = (long)yyVals[0+yyTop];
								byte[] intb = BitConverter.GetBytes(l);
								yyVal = BitConverter.ToInt32(intb, BitConverter.IsLittleEndian ? 0 : 4);
						  }

void case_678()
#line 2341 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								int i = (int)yyVals[-1+yyTop];
								byte[] intb = BitConverter.GetBytes(i);
								yyVal = (double)BitConverter.ToSingle(intb, 0);
						  }

void case_679()
#line 2347 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								long l = (long)yyVals[-1+yyTop];
								byte[] intb = BitConverter.GetBytes(l);
								yyVal = (double)BitConverter.ToSingle(intb, BitConverter.IsLittleEndian ? 0 : 4);
						  }

void case_680()
#line 2353 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								byte[] intb = BitConverter.GetBytes((long)yyVals[-1+yyTop]);
								yyVal = BitConverter.ToDouble(intb, 0);
						  }

void case_681()
#line 2358 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								byte[] intb = BitConverter.GetBytes((int)yyVals[-1+yyTop]);
								yyVal = (double)BitConverter.ToSingle(intb, 0);
						  }

void case_684()
#line 2368 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								yyVal = yyVals[-1+yyTop];
								tokenizer.InByteArray = false;
						  }

void case_686()
#line 2376 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var byte_list = (ArrayList)yyVals[0+yyTop];
								yyVal = byte_list.ToArray(typeof(byte));
						  }

void case_687()
#line 2383 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var byte_list = new ArrayList();
								byte_list.Add(Convert.ToByte(yyVals[0+yyTop]));
								yyVal = byte_list;
						  }

void case_688()
#line 2389 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"
{
								var byte_list = (ArrayList)yyVals[-1+yyTop];
								byte_list.Add(Convert.ToByte(yyVals[0+yyTop]));
						  }

#line default
static readonly short[] yyLhs =
{
   -1,    0,    1,    1,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
   19,   19,   19,   19,   20,   20,   20,    8,   21,   21,   21,   21,   21,    4,   23,    3,   25,   27,   27,   27,   27,   27,   27,   27,   27,
   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   27,   29,   29,   30,   30,   32,   32,   28,   28,
   34,   34,   35,   35,   37,   37,   38,   38,   31,   31,   31,   31,   31,   31,   31,   33,   33,   40,   40,   40,   40,   40,   40,   41,   42,
   42,   43,   43,   44,   44,   39,   39,   39,   26,   26,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   45,   36,
   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   36,   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,
   54,   54,   54,   54,   54,   54,   54,   54,   54,   54,   52,   52,   55,   55,   55,   55,   55,   50,   50,   50,   56,   56,   56,   56,   56,
   56,   56,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,
   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,   57,
   57,   57,   57,   57,   57,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,
   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   59,   53,
   53,    6,   60,   60,   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,   61,   62,   62,   63,   63,   66,   66,
   66,   66,   66,   66,   66,   66,   66,   66,   66,   66,   66,   66,   66,   69,   69,   65,   65,   65,   71,   71,   72,   73,   73,    7,   74,
   74,   76,   76,   76,   75,   75,   77,   77,   78,   78,   78,   78,   78,   78,   78,   78,   78,   78,   78,   78,   78,   78,   78,   78,   78,
    5,   79,   79,   81,   81,   81,   81,   81,   81,   81,   81,   81,   81,   81,   81,   81,   81,   81,   81,   81,   81,   81,   81,   81,   81,
   81,   84,   84,   84,   84,   84,   84,   84,   84,   84,   84,   84,   84,   84,   84,   84,   49,   49,   49,   82,   82,   82,   82,   83,   83,
   83,   83,   83,   83,   83,   83,   83,   83,   83,   83,   83,   83,   83,   51,   51,   85,   85,   86,   86,   86,   86,   86,   87,   87,   87,
   87,   87,   88,   88,   80,   80,   89,   89,   89,   89,   89,   89,   89,   89,   89,   89,   89,   89,   89,   89,   89,   89,   89,   89,   89,
   89,   89,   89,   89,   90,   90,   90,   95,   95,   95,   95,   96,   48,   48,   48,   92,   97,   93,   98,   98,   98,   99,   99,  100,  100,
  100,  100,  102,  102,  102,  101,  101,  101,   94,   94,   94,   94,   94,   94,   94,   94,   94,   94,   94,   94,   94,   94,   94,   94,   94,
   94,   94,   94,   94,   94,   94,   91,   91,  104,  104,  104,  104,  104,  103,  103,  105,  105,  105,   46,  106,  106,  108,  108,  108,  107,
  107,  109,  109,  109,  109,  109,  109,  109,   47,  110,  112,  112,  112,  112,  111,  111,  113,  113,  113,  113,  113,  113,   16,   16,   16,
   16,  114,  114,  116,  116,  116,  116,  116,  117,  117,  118,  118,  115,  115,   15,   15,   15,   15,   15,  121,  121,  122,  123,  123,  124,
  124,  126,  125,  125,  120,  120,  127,  128,  128,  128,  128,  128,  128,  128,  128,  119,  119,  119,  119,  119,  119,  119,  119,  119,  119,
  119,  119,  119,  119,  119,   14,   14,   14,    9,    9,  129,  129,  130,  130,   10,  131,  134,  134,  132,  132,  135,  135,  135,  135,  135,
  135,  135,  136,  136,  136,  136,  136,   11,  137,  137,  138,  138,  139,  139,  139,  139,  139,  139,  139,  139,   12,  140,  142,  142,  142,
  142,  142,  142,  142,  142,  142,  142,  141,  141,  143,  143,  143,  143,   13,  144,  146,  146,  146,  145,  145,  147,  147,  147,   58,   58,
   17,   18,   67,   67,   67,   67,   67,  148,  150,   70,  149,  149,  151,  151,   68,   68,   22,   22,   24,   24,   24,   64,   64,  133,  133,
};

static readonly short[] yyLen =
{
    2,    1,    0,    2,    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,    2,    2,    3,    2,    2,    1,    1,
    3,    2,    5,    4,    2,    4,    6,    7,    0,    2,    2,    2,    2,    4,    2,    4,    6,    0,    2,    2,    3,    3,    3,    3,    3,
    3,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    0,    2,    0,    1,    2,    3,    0,    3,
    0,    3,    1,    3,    0,    3,    1,    3,    1,    1,    3,    2,    3,    2,    3,    3,    5,    0,    2,    2,    2,    2,    2,    1,    3,
    5,    1,    3,    1,    3,    4,    5,    1,    0,    2,    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,    2,    2,   13,    1,    1,
    3,    6,    3,    3,    4,    2,    2,    2,    5,    5,    7,    1,    1,    1,    1,    1,    1,    1,    2,    1,    2,    1,    2,    1,    2,
    1,    2,    3,    2,    1,    1,    1,    1,    1,    1,    1,    3,    0,    1,    1,    3,    2,    2,    2,    1,    0,    1,    1,    2,    2,
    2,    2,    0,    6,    5,    5,    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,    2,    1,    2,    1,    2,    1,    2,
    1,    2,    3,    4,    6,    5,    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,    2,    4,    1,    2,    2,    1,    2,
    1,    2,    1,    2,    1,    0,    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,    2,    2,    2,    2,    1,    3,    2,    2,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    2,    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    8,    0,    3,    0,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    5,    2,    2,    0,    2,    0,    2,    4,    4,
    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    4,    1,    2,    1,    1,    1,    1,    4,    1,    1,    2,    2,    4,
    2,    0,    1,    1,    3,    1,    1,    3,    5,    5,    4,    3,    2,    5,    5,    5,    5,    5,    5,    2,    2,    2,    2,    2,    2,
    4,   11,   14,    0,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    8,    6,
    5,    0,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    4,    4,    4,    4,    1,    1,    1,    0,    4,    4,    4,    0,    2,
    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    0,    1,    1,    3,    2,    3,    1,    6,    7,    0,    1,    3,
    3,    5,    0,    1,    0,    2,    2,    2,    4,    5,    1,    1,    4,    6,    4,    4,    3,   15,    1,    5,    1,    2,    1,    1,    1,
    1,    1,    1,    1,    0,    1,    3,    1,    2,    2,    3,    3,    3,    4,    1,    3,    1,    2,    2,    4,    4,    1,    2,    3,    2,
    2,    2,    2,    2,    2,    1,    4,    4,    1,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,    5,    3,    2,
    2,    4,    3,    6,    2,    4,    7,    9,    0,    1,    1,    3,    3,    1,    1,    2,    5,    3,    4,    4,    3,    0,    2,    2,    0,
    2,    3,    3,    3,    3,    1,    1,    1,    4,    8,    0,    2,    2,    2,    0,    2,    2,    2,    2,    1,    1,    1,    3,    5,    5,
    7,    0,    3,    0,    7,    2,    4,    1,    1,    2,    1,    4,    8,    6,    6,    3,    4,    3,    6,    1,    3,    5,    1,    2,    3,
    4,    3,    1,    1,    1,    3,    3,    1,    1,    4,    1,    6,    6,    6,    4,    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    2,    3,    8,    4,    0,    2,    0,    1,    4,    4,    0,    2,    0,    2,    3,    8,    2,    3,    3,
    1,    1,    3,    8,    2,    3,    1,    4,    5,    7,    0,    2,    8,    3,    3,    2,    3,    3,    1,    1,    4,    4,    0,    2,    2,
    2,    3,    3,    3,    3,    3,    3,    0,    2,    2,    3,    1,    3,    4,    3,    0,    2,    2,    0,    2,    4,    3,    1,    1,    3,
    1,    1,    1,    4,    4,    4,    4,    1,    0,    4,    0,    1,    1,    2,    1,    1,    1,    1,    1,    3,    1,    0,    1,    0,    2,
};

static readonly short[] yyDefRed =
{
    2,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  353,    0,  665,    0,    0,    0,    0,    0,    0,    3,    4,    5,
    6,    7,    8,    9,   10,   11,   12,   13,   14,   15,   16,   17,   23,   24,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  616,
  647,    0,  676,   21,  675,   19,    0,    0,  328,  327,    0,    0,  279,    0,    0,    0,    0,    0,  691,  692,  695,    0,  693,    0,    0,
    0,  590,  591,  592,  593,  594,  595,  596,  597,  598,  599,  600,  601,  602,  603,  604,    0,    0,   22,   18,    0,    2,  108,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  323,  330,  429,  618,  635,  657,  668,  616,  699,    0,    0,   58,   43,   44,   66,   65,
   51,   53,   54,   55,   56,   57,   59,   60,   61,   62,   63,    0,   64,   52,    0,    0,    0,    0,    0,  157,  158,  137,  138,  139,  140,
  141,  142,    0,  144,  146,  148,  150,    0,    0,  154,  156,  155,    0,   84,  159,    0,  124,    0,   83,    0,  136,    0,    0,  171,  172,
    0,    0,  169,    0,    0,    0,    0,   20,  611,    0,    0,   25,    0,  354,  355,  356,  357,  369,  370,  368,  358,  359,  360,  361,  364,
  362,  363,  365,  366,  372,    0,  371,  367,  394,    0,    0,  666,  667,    0,    0,    0,    0,  673,    0,    0,    0,    0,    0,    0,  331,
    0,    0,  349,    0,  348,    0,  347,    0,  346,    0,  344,    0,  345,    0,    0,  683,    0,  337,    0,    0,    0,    0,    0,    0,  617,
    0,  649,  648,    0,  650,    0,   46,   45,   47,   48,   49,   50,   92,    0,    0,    0,    0,   86,   88,    0,    0,  153,  151,  143,  145,
  147,  149,    0,    0,    0,    0,    0,  552,  131,  130,  132,    0,    0,    0,  167,  168,  173,  174,  175,  176,    0,    0,  324,  278,    0,
  287,  280,  281,  282,  288,  289,  290,  283,  284,  285,  286,  292,  293,    0,  613,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  566,    0,   33,   38,   40,   42,  521,    0,    0,    0,  535,    0,  111,  110,  114,  115,  116,  118,  117,  123,  119,  109,  112,  113,    0,
    0,  329,    0,    0,    0,    0,    0,    0,  677,    0,    0,    0,    0,    0,    0,    0,  336,  466,  350,  483,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  435,    0,    0,    0,    0,    0,    0,    0,  436,  453,  449,  452,  450,  451,    0,  445,
  430,  443,  447,  448,  429,    0,  614,    0,    0,    0,    0,  626,  625,  619,  632,    0,    0,    0,    0,    0,  644,  643,  636,  645,    0,
    0,    0,  661,  658,  663,    0,    0,  672,  669,    0,  651,  652,  653,  654,  655,  656,    0,    0,    0,    0,    0,    0,   87,   89,  125,
  152,    0,    0,   85,    0,  127,  128,  163,    0,    0,  160,    0,    0,    0,    0,  392,  391,    0,    0,    0,    0,    0,  549,    0,    0,
    0,    0,   27,    0,    0,    0,    0,    0,    0,    0,    0,  579,    0,    0,  569,  674,    0,    0,    0,  121,    0,    0,  120,  524,  539,
  332,  335,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  682,  687,    0,    0,  488,  489,  490,  492,  491,  493,  494,  495,    0,
  496,    0,  499,    0,    0,    0,    0,    0,  513,  504,  514,    0,  484,  485,  486,  487,  431,    0,    0,    0,  432,    0,    0,    0,    0,
    0,  468,    0,  446,    0,    0,    0,    0,    0,    0,  471,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,   92,   74,    0,   93,   94,   95,   97,   96,    0,   68,    0,   41,    0,    0,    0,    0,    0,    0,    0,    0,  129,
    0,  275,    0,  274,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  560,    0,
    0,  558,    0,  217,    0,    0,    0,    0,    0,  182,  183,  184,  185,  186,  187,  188,  189,  190,  191,  192,    0,  194,  196,  198,  200,
  206,  207,  208,  209,  210,  211,  212,  213,  214,  215,  216,    0,  220,  223,  225,  229,  227,    0,    0,    0,    0,   31,    0,    0,  375,
  383,  384,  385,  386,  378,  379,  380,    0,  377,  381,  382,    0,    0,    0,    0,    0,    0,    0,    0,    0,  564,    0,    0,  568,    0,
    0,   34,   35,   36,   37,  522,  523,    0,    0,    0,    0,   99,  538,  536,  537,    0,    0,    0,  343,  342,  341,  340,    0,    0,    0,
    0,  338,  339,  334,  333,  684,  688,    0,    0,    0,    0,  502,    0,    0,  515,    0,  510,  509,    0,    0,    0,    0,    0,  455,    0,
    0,    0,  441,    0,    0,    0,    0,    0,  465,    0,  479,  478,  477,    0,  480,  474,  475,  472,  476,  624,  623,  620,    0,  642,  641,
  638,  639,    0,    0,    0,    0,    0,    0,    0,    0,    0,   98,   90,   71,    0,    0,    0,    0,   76,    0,  165,  161,  133,  134,    0,
  423,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  555,    0,  550,    0,  559,  228,  224,
  222,    0,    0,    0,  226,  193,  195,  197,  199,  221,  246,  232,  233,  234,  235,  236,  237,  238,  239,  240,  241,  260,    0,  250,  251,
  252,  253,  254,  255,  256,  257,  258,  231,  261,  262,  263,  264,  265,  266,  267,  268,  269,  270,  271,  272,  273,    0,    0,  291,  201,
  295,    0,    0,    0,    0,  374,    0,    0,  395,  396,  397,    0,    0,  689,  690,    0,    0,    0,  582,  581,  580,    0,  570,   32,    0,
    0,    0,    0,  518,    0,    0,    0,    0,  530,  531,  532,  525,  533,    0,    0,    0,  544,  545,  546,  540,  678,  679,  681,  680,    0,
    0,    0,  501,    0,    0,    0,    0,  505,    0,    0,    0,  458,  433,    0,    0,    0,    0,  440,    0,  470,  469,  439,  473,    0,    0,
    0,    0,  670,    0,   80,    0,   72,  419,    0,    0,    0,  415,    0,  126,    0,    0,  563,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  242,  243,  244,  245,  259,    0,    0,  249,  248,  202,    0,
    0,    0,  317,    0,  297,  313,  315,  697,  276,  608,    0,  387,  388,  389,  390,    0,    0,    0,    0,  578,  577,    0,  572,    0,    0,
  100,    0,    0,    0,    0,    0,  542,  543,  541,    0,    0,  497,    0,    0,  512,  511,    0,  461,  456,  460,  434,    0,  444,    0,    0,
    0,    0,   91,    0,  135,    0,    0,    0,  428,  425,  424,    0,    0,  312,  310,  306,  304,  302,  300,  298,  301,  299,  311,  307,  305,
  303,  561,  309,  308,  556,    0,    0,    0,  247,    0,    0,  203,    0,  314,  373,    0,    0,    0,    0,    0,    0,    0,  571,  573,    0,
    0,    0,    0,  526,  528,  529,  527,    0,    0,  503,  516,  438,    0,  482,  481,    0,    0,    0,  418,  416,  562,    0,    0,    0,  179,
  180,  205,    0,    0,  398,  584,    0,    0,    0,  589,    0,    0,  574,    0,    0,    0,    0,    0,    0,    0,    0,  426,  321,    0,    0,
  178,  204,    0,    0,    0,    0,    0,  575,    0,    0,  534,    0,  506,    0,  621,  637,    0,  554,  322,    0,  412,  404,  399,  401,  400,
  402,  403,  405,  407,  408,  409,  410,  411,  406,  586,  587,  588,    0,  318,  576,    0,    0,    0,    0,  398,    0,    0,  507,    0,  421,
    0,    0,    0,    0,  319,    0,    0,  122,    0,    0,  442,
};

static readonly short[] yyDgoto =
{
    1,    2,   22,   23,   24,   25,   26,   27,   28,   29,   30,   31,   32,   33,   34,   35,   36,  458,   53,   37,   38,  491,   72,   39,  165,
   40,  222,   51,  263,  444,  590,  166,  591,  441, 1140,  595,  215,  587,  785,  168,  442,  787,  399,    0,  169,  346,  347,  348,  924,  925,
  524,  958,  459,  602,  170,  460,  177,  667,  484,  871,   62,  181,  669,  877, 1008, 1004,  623,  361,  893, 1006,  242, 1189, 1148, 1149,   41,
  109,   60,  223,  110,   42,  243,   67,  801, 1153,  479,  960,  961,  802, 1059,  400,  747,  525,  764,  402,  403,  748,  749,  404,  405,  559,
  560,  765,  561,  534,  743,  535,  349,  716,  492,  911,  350,  717,  496,  919,   57,  178,  624,  625,  626,   91,  485,  488,  489, 1021, 1022,
 1023, 1137,  486,  894,   64,  315,   43,  244,   49,  118,  413,    0,   44,  245,  422,   45,  246,  119,  428,   46,  247,   74,  433,  513,  514,
  365,  515,
};

static readonly short[] yySindex =
{
    0,    0, 6883, -279, -248, -123,  -95,  -87, -330,  -48, -346,  -15,  -95,    0, -152,    0,  547, 1189, 1189, -123,  -95,    6,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,   81,  120,  468,  157,  225,  238,  244,  250, -109, -263,    0,
    0, 1006,    0,    0,    0,    0, 5302,  407,    0,    0,   94,  -95,    0,  -95, -158,  173,   72, 1849,    0,    0,    0,  547,    0,  263,  210,
  263,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 5336, -106,    0,    0,  -95,    0,    0,  860,  276,
  264,  272,  338,  388,  398,  415,  274,  285,  -53,    0,    0,    0,    0,    0,    0,    0,    0,    0, -241, -230,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  962,    0,    0, -118,  294, -140,   32, -213,    0,    0,    0,    0,    0,    0,
    0,    0,  506,    0,    0,    0,    0,  407,  452,    0,    0,    0,  509,    0,    0,  263,    0,  279,    0,  287,    0,  407,  407,    0,    0,
  619, 5302,    0,  324,  360,  374, 4459,    0,    0, -178,  370,    0,  -95,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  381,    0,    0,    0,  263,  547,    0,    0,  263,  457,  187,  397,    0,   -9,  380,  409, 4818, 5360,  179,    0,
   94,  -95,    0,  -95,    0,  -95,    0, -123,    0, -194,    0, -194,    0,  414,  423,    0,  458,    0, 6404,  732,  531,  189,   50, -241,    0,
  287,    0,    0,  965,    0,  263,    0,    0,    0,    0,    0,    0,    0,  302,  547,   -1,  366,    0,    0,  452,  256,    0,    0,    0,    0,
    0,    0, 5302,  460,  547,  191, -134,    0,    0,    0,    0,  469,  483,  547,    0,    0,    0,    0,    0,    0, 2893,  255,    0,    0,  485,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,   34,    0,  444,  519,  577,  599, 5362,  263,  547,   80,  599,  452,
    0,  601,    0,    0,    0,    0,    0, 5336,  -95,  357,    0,  -95,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  600,
  604,    0,  860,  603,  606,  608,  621,  623,    0,  620,  632,  636,  647,  599,  599,  665,    0,    0,    0,    0,  366, -123,  -84,  366,  407,
 5302, 5336, -223,  407, 5082,  677,  366,  366,  -95,    0,  667, -228,  -95, 5469, -171,  543,  -95,    0,    0,    0,    0,    0,    0,  690,    0,
    0,    0,    0,    0,    0,  500,    0,  437,  -88,  708,  -95,    0,    0,    0,    0,  717,  -71,  723,  729,  -95,    0,    0,    0,    0,  643,
  650,  547,    0,    0,    0,  663,  547,    0,    0, -172,    0,    0,    0,    0,    0,    0, -160, -147, -197,  592,  116,  547,    0,    0,    0,
    0,  337, 5302,    0,  145,    0,    0,    0,  741,  127,    0, 2574, 2574,  263,  417,    0,    0,  263,  768,  776, 2038,  380,    0, 6174,  681,
  781,  803,    0, -166, -107,  317,  484,  159,  547,  196,  251,    0,  790,  217,    0,    0,   90, 2609,  795,    0,  -83, 4958,    0,    0,    0,
    0,    0,  -48,  -48,  -48,  -48,  311,  352,  -48,  -48,  -64,   87,    0,    0,  800,  665,    0,    0,    0,    0,    0,    0,    0,    0, 5302,
    0,  872,    0,   66,  380, 5302,  407, 5302,    0,    0,    0,  366,    0,    0,    0,    0,    0,  -95, 5589,  816,    0,  407,  828,  -95,  586,
  594,    0,  841,    0, 6491, 5302,  543, -182, -182,  500,    0, -182,  -95,  458,  380,  458,  850,  458,  458,  380,  458,  458,  854,  547,  547,
  263,  547, -188,  547,    0,    0, 5302,    0,    0,    0,    0,    0,   94,    0, -197,    0,  858,  547,  287,  865,  -76,  187,  547,  -95,    0,
  -78,    0,  873,    0,  876,  327,  869,  453,  883,  892,  894,  895,  899,  902,  903,  904,  920,  923,  926,  927,  929,  458,  930,    0,  934,
 1747,    0,  662,    0,  738,  743,  949,  133,  753,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0, -295,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 6111,    0,    0,    0,    0,    0,  312,   94,  944,  458,    0,  599,  181,    0,
    0,    0,    0,    0,    0,    0,    0,  951,    0,    0,    0,  957,  966,  967,  976,  964,  263,  968,  547, -168,    0,  599,  980,    0,  452,
   94,    0,    0,    0,    0,    0,    0,    0,  547,  327,  -95,    0,    0,    0,    0,  233, 1339,  637,    0,    0,    0,    0,  989,  990,  991,
  993,    0,    0,    0,    0,    0,    0, 2893,    0,  977,  458,    0,  -41, 5302,    0,  872,    0,    0,  347,  999,  470,   34,  508,    0, 5302,
 5589, 5302,    0,  327, 1001,  -95,   94,  -95,    0, -113,    0,    0,    0,  366,    0,    0,    0,    0,    0,    0,    0,    0,  -95,    0,    0,
    0,    0,  -95,  263,  263,  287,  -95,  287, -147,  187,  553,    0,    0,    0, -197,  287,  988, 5302,    0,  191,    0,    0,    0,    0, 1002,
    0, 5362,  576,  -95,  216, -123, -123, -123, -123,   -7,   -7, -123, -123, -123, -123, 5302, -123, -123,    0, 1014,    0, 1005,    0,    0,    0,
    0,  599, 1011, 1013,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  429,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0, -148, -126,    0,    0,
    0, -221, 1009,  959,  380,    0,  310,  356,    0,    0,    0, 6174, 1016,    0,    0, 1017, 1020,  380,    0,    0,    0,  331,    0,    0,  263,
  779, 1023,  232,    0,  407,  407,  407,  407,    0,    0,    0,    0,    0,  407,  407,  407,    0,    0,    0,    0,    0,    0,    0,    0, 1019,
  460,   94,    0,  869, 4614,    0, 1022,    0,  366,  931, 1028,    0,    0, 5589,   34,  626, 4614,    0,  944,    0,    0,    0,    0,  786,  791,
 1033, 1034,    0,   94,    0, 5302,    0,    0, 1037, 5362, 1046,    0,  187,    0,  869,   34,    0, 1036, 1051, 1059, 1061, 1066, 1067, 1076, 1078,
 1089, 1098, 1099, 1107, 1110, 1111, 1112,  291, 1113, 1115, 1116,    1,  -95,  -95,    0,    0,    0,    0,    0, 1119,  599,    0,    0,    0,  -95,
 -159,  458,    0,  380,    0,    0,    0,    0,    0,    0,  430,    0,    0,    0,    0,  339,  988,  -95,  -23,    0,    0, -203,    0, 2909,  407,
    0,  869, 1009, 1009, 1009, 1009,    0,    0,    0,  327, 1125,    0,  639,   94,    0,    0,   94,    0,    0,    0,    0, 1126,    0,  -95,   94,
  -95,  -95,    0,  187,    0,   -8,  988,  661,    0,    0,    0, 5362, 1133,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  599, 1136, 1137,    0,  380, 1138,    0,  -95,    0,    0,  327, 1139, 1141, 1154, 1155, 1156, 1159,    0,    0,  452,
 1150, 5302,  666,    0,    0,    0,    0,  460,  869,    0,    0,    0,  327,    0,    0, 1158, 1160, 1162,    0,    0,    0,   34,  216,  177,    0,
    0,    0, 1165, 1166,    0,    0,  -95,  -95,  -95,    0, 1150, 1164,    0,  872,  944, 1174,  673, 1149,  -95,  -95, 6174,    0,    0, 1182,  216,
    0,    0,  988, 1525, 1184, 1185, 1192,    0, 2070, 1171,    0,  869,    0, 5623,    0,    0,  486,    0,    0, 1193,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 1190,    0,    0,  327,  680,  470,   94,    0, 1206, 1198,    0, 1207,    0,
 1525, 1210,  988, 1197,    0, 1216, 1218,    0,  869,  697,    0,
};

static readonly short[] yyRindex =
{
    0,    0, 1491, -184, 1590,    0,    0, 4852,  365, 4588, -144,    0,    0,    0, 1248,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0, -184,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 5697,    0,    0, 1432,    0,    0,    0,    0, 3534, 3728, 5697,    0,    0,    0,    0,    0, 1539,    0,
 1225,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 4363, 4363, 4363, 4363, 4363, 4363,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0, -211,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 5697,    0,    0,    0,    0,    0,    0,    0, 1180,    0,    0,    0, 1762,    0, 5697, 5697,    0,    0,
    0,    0,    0, 3192,    0,    0,    0,    0,    0,  622,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1830,    0,    0,    0, 1226,    0, 3420, 3922,    0,    0, 4037,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1229,    0,    0,    0,    0, 1230,    0,    0,    0,    0,    0,    0,    0, -163,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 2635,    0, 2926, 1233,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 2121, 3639, 3817, 1239,    0,  889,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 1238,    0,    0,    0,    0,    0,    0,    0,    0, 5697,
    0,    0,    0, 5697,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 1246,    0,    0,    0,    0,    0,    0,    0,  492,    0, 1252,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  138,    0,    0,    0,    0, 1471, 1233,    0,    0,  130,    0,    0, 1254, 3306,    0,  578,  321,
    0,    0,    0, 1239,    0,    0,    0,    0, 4143,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0, 4363, 4363, 4363, 4363,    0,    0, 4363, 4363,    0,    0,    0,    0,    0, 1255,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0, 6578,    0, 5697,    0,    0,    0,    0,  726,    0,    0,    0,    0,    0,    0,  737,    0,    0, 5697,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0, 6665,    0,    0,    0,    0,  947,    0,    0,    0,    0,  544,    0,    0,    0,    0,    0,
  464,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 1259,    0, 2053, 4743,    0,  -18,    0,  378,    0,
  425,    0,    0,    0,    0,    0, 5018,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1260,    0,    0,    0,    0,    0,    0,    0,  625,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0, -131,    0,    0,    0,    0,    0,    0,    0,  761,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 5966, 1263, 4248,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  -66,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 6071,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  759,    0,    0,    0,
  737,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  960,  975,  390,    0, 1270,  492,  763,    0,    0,    0,    0,    0, 2344, 5196,    0,    0, 2926,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  123,    0,    0,    0,
    0,    0,  705, 2412, 1239,    0,    0,    0,    0,    0,    0,  578,    0,    0,    0,  789,    0,  826,    0,    0,    0,    0,    0,    0, 1271,
    0,    0,    0,    0, 5697, 5697, 5697, 5697,    0,    0,    0,    0,    0, 5697, 5697, 5697,    0,    0,    0,    0,    0,    0,    0,    0,    0,
 1272,    0,    0, 5018,    0, 6188,    0,    0,    0, 6752,    0,    0,    0,    0,  835,    0,    0,    0, 6839,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 1274,    0,   12,    0, 5018,  845,    0, 5730,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0, 4476,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 5196,    0,    0,    0,    0,    0,    0,    0, 5697,
    0, 5018, 1971, 1971, 1971, 1971,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  847,    0,  862, 5730,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  641,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0, 1272, 5018,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  845,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 1283,    0,    0, 1272,    0,    0,  578,    0,    0,    0, 1280,
    0,    0, 5196, 1287,    0,    0,    0,    0,    0,    0,    0, 5018,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  870,    0,    0,    0,    0,    0,    0,
 1288,    0, 5196,    0,    0,    0,    0,    0, 5018,    0,    0,
};

static readonly short[] yyGindex =
{
    0, 1460,    0, 1335,    0, 1338, 1340, -185,    0,    0,    0,    0,    0,    0,    0, -176, -196,   -6,   22, -201, -189,    0,  -47,    0,  -12,
    0,    0,    0,  879,    0,    0, -419,    0,    0, -262,    0,  -24,  778,    0, -150,  985,  612, 1350,    0, -117,    0,    0,    0,  -86, -288,
  -45, -929,    0, 1117, -451,  973,    0, -846,  -67,    0,  -21,    0,    0, -890,   64,    0, -842, -227, -777,  416, -145,    0,    0,    0,    0,
    0,    0,    0,  -54,    0, 1176,    0, -205,  384, -463,    0,  525, -893,  454,    0,  834, -471, -214,    0,    0,  649,    0,    0,    0,    0,
 1026, -492,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  963, 1572,    0,    0,  896,    0,  573,
    0,  461,  901,    0,    0,  720,    0,    0, 1552, 1487,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0, 1090,    0,
    0,    0,    0,
};

static readonly short[] yyTable =
{
   55,  250,   73,  319,   75,  216,   66,  469,  278,  362,  603,  603,  176,  179,   94,  673,  453,   68,  455,   69,   70,  343,  208,  265,  588,
  219,  342,  969,   68,  401,   69,   70,  167,  344, 1005, 1037,  217,  340,  217,  139, 1015,   93,  396,  543,  224,  280,  341,  395,  412,  421,
  427,  432,  184, 1047,  397,  180,   73,  182,  393,  209,  739,   68,  213,   69,   70, 1097,  766,  394,  411,  768,  358, 1057,  140,  325,  698,
  752,  698,  698,  210,  227,  229,  231,  233,  235,  237,  367,  141, 1091,  210,  220,   68,  217,   69,   70,  268,   54,  366,  322,  548,   47,
   68,  140,   69,   70,   67,   58,   68,  255,   69,   70,  288, 1086,  277,  579,  610,  326,  610,  610,   68,  449,   69,  994, 1087,   54,  581,
  995,  289,  290,  580,   54,   50,  434,  996, 1102,  582,  267,  456,   54,  230,  583,   52,  230,  230,  266,  998,  520,  314,  445,  210,  230,
  457,  472,  295,  217,  367,  999,  281,  313,  830,  831,  832,  833,  454,  251,  252,  674,  283,  284,   54,  262,  788,  217,  218,  584,  487,
   68,  551,   69,  353,   52,  358,  317,  781,  834,   56,   54,  710,  240,  217,  253,  563,   59,  693,  692,  693,  693,  398,  792,  320,  578,
  693,  520,   63,  693,  482,  672,  457,  568,  728,  468,   73,   73,  793,  693,  693, 1141,  693,  326,  240,  448,  354,   61,  355, 1169,  356,
  241,   71,  521,  281,  471,  928,  821,  608,  609,  610,  611,  612,  613,  283,  284,   54,  614,  615,  616,  617,   65,  493,  270,  271, 1160,
   68,  478,   69,  451,  357,   77,   52,  358,  324,   67,  447,  281,  240,  359,  360,  272,  474,  947, 1191,  446,   77,  283,  284, 1205, 1080,
   95,  463,  675,  676,  677,  678,  288,  326,  467,   48,   78,  544,  619,  620,  528,  527, 1001,   68,  533,   69,  481,  510,  511,  500, 1166,
   78,  117,  547,  281,  249,  158,  679,  680,  681,  682,  529,  601,  601,  283,  284, 1209, 1005,  799,  429, 1019, 1020,  294,  683,  517,  684,
  685,  523,  494, 1002,  593,  497,  254,  186,  530,  538,  540,  495,  240,  763,  997,  401,  564,  762,  550,  735, 1147,  187,  322,   96,  569,
  483,  526,   68,  396,   69,  585,  230,  163,  395,  729,  430,  698,  288,  265,  516,  397,  593,  522,  326,  393,  956,    7, 1168,  359,  360,
  537,  539,  541,  394,  431, 1117,  545,  183,  736,  549,  552,  592,   97,  888,  889,  285,  286,  287,  518,  519,  218,  610,  599,  288,  103,
  600,  393,  218,  586,  566,  103,  708,  763,  164,  103,  103,  164,  103,  572,  575,  597, 1010,  393,  770,  577,  771,  900,  773,  774,  111,
  775,  776,  288,  596,  693, 1093, 1094, 1095, 1027, 1028, 1029, 1030,  693,  693,  693,  734,  288, 1031, 1032, 1033,  707,  185,  351,  711, 1150,
  359,  360,  352,  880,   48,  686,  281,  423,  326,  780,  700,  782,  285,  286,  287,  942,  283,  284,   68,  691,   69,   70,  715,  288,  694,
  790,  818,  326,  452,  733,  794,  718,  719,  720,  721,  698,  738,  726,  727,  742,  699,   68,  112,   69,   70,  285,  286,  287,  210,  424,
  732,  751,  281, 1026,  425,  113,  737,  398,  740,  761,    7,  114,  283,  284,  217,  909,  917,  115,  426,  746,  908,  916,  470,  695,  696,
  878,  240,  910,  918,  210,  741,  759,  963,   61,  269,  226,  744,  285,  286,  287,  786,   61,  754,  228,  891,  701,  702,  225,  281,  487,
  760,  282,   68,  238,   69,   70,  769,  784,  283,  284,  281,  778,  779, 1076,  239,  675,  676,  677,  678,  288,  283,  284, 1100,  722,  723,
   68,  790,   69,   70,  827,  828,  872,  975,  977,  873,   68,  959,   69,   70,  294,  927,  874,  795,  691,  679,  680,  681,  682,  294,  294,
  703,  704,  296,  211,  212,  879,  281,   61,  872,  230,  683, 1090,  684,  685,  724,  725,  594,  284,  874,  932,  933,  875,  612,  326,   68,
  326,   69,  892,  265,   54,  316,  294,  326,  103,  103,  103,  294,  264,  297,  294,  294,  465,  294,  294,  298,  326,  294,  466,  166,  294,
  294,  166,  318,  898,  931,  294,  294,   61,  671,  232,  264,  326,  294, 1035,  294,  294,  294,   61,  323,  234,  294,  294,  294,  294,  294,
   68,  294,   69,   70,  327,   54,  294,  294,  294,   61,  363,  236,  456,  890,  294,  285,  286,  287,  930,  364,  162,  899,  691,  162,  936,
  671,  457, 1089,  902,  901,  696,  687,  688,  689,  945,   68,  671,   69,   70,  929,   68,  949,   69,   70,  671,  467,  140,  803,  443,  804,
  939,  746,  941,   68,  240,   69,   70,  659,   54,  450,   98,  285,  286,  287,  935,  461,  691,   68,  686,   69,   70, 1107,  452,   99,  944,
   79,  946,   79,  281,  462,  872,  473,  948, 1193,  986,  321,  296, 1061,  283,  284,  874,  950,   68,  962,   69,   70,  951,  888,  889,  659,
  952,  475,  965,  279,  659,  937,  938,  285,  286,  287,  659,  171,  172,  173,  174,  175,  982,  476,  659,  285,  286,  287,  968,  465,  414,
  321,   68, 1128,   69,  466,   68,   54,   69,   70, 1003,  367,  959,  640,  264,  675,  676,  677,  678,  326,  326,  326,  326,  326,  326, 1142,
  954,  955,  970,  971,  972,  973,  974,  976,  978,  979,  980,  981,  477,  983,  984,  285,  286,  287,  679,  680,  681,  682,  177,  966,  967,
  177,  959,    7, 1019, 1020, 1046, 1088,  177,  217,  683,  490,  684,  685,  415,  495,  640, 1000,  498,  416, 1011, 1012,  499,  690,  326,  326,
  501,  640,  326,  502, 1036,  503,  640,  989,  990,  991,  992, 1040,  417,  418,  694,  612,  506, 1044,  504,  181,  505,  419,  181, 1045,  938,
  640,  640, 1196,  507,  181,  912,  786,  993,  508,  640,  219, 1109,  967,  219,  746, 1013, 1014,  467, 1058,  509,  219,  100,  101,  102,  103,
  104,  105, 1039, 1084,  467,  512, 1053,  612, 1120,  967, 1055,  542,  612, 1139,  967,  612,  612,  420,  612,  612, 1162,  967,  959,  536, 1135,
  612,  612, 1197,  967,  612,  640,  612,  612,    7,  273,  274,  275,  276,  612,  553,  612,  612,  612,  913, 1210,  967,   11,   12,  696,  612,
  612,  106,  107,  562, 1101,  108, 1081, 1082,  612,  612,  914,  565,  285,  286,  287,  612, 1110,  686, 1085, 1111,  567,  915,  959,  508,  508,
  406,  570, 1114,  291,  292,  293,  294,  571, 1118,  454,  454, 1092, 1096, 1123,  589,  696,  555,  556,  557,  558,  696,  573,  691,  696,  696,
  598,  696,  696,  574,  296,  696,  457,  457,  696,  696,   81,   81, 1121,  296,  696,  696,  576, 1113,  605, 1115, 1116,  696,  606,  696,  696,
  696,  668, 1159,    7,  696,  696,  696,  696,  696,  670,  696,  583,  583,  671,  407,  696,  696,  696,  697,  408,  709,  296,  730,  696, 1058,
  265,  296, 1138,  691,  296,  296, 1127,  296,  296,   17,   18,  296,  750,  409,  296,  296, 1103, 1104, 1105, 1106,  296,  296,  410,  585,  585,
  691,  755,  296,  753,  296,  296,  296,  459,  459,  756,  296,  296,  296,  296,  296,  757,  296,  427,  427,   82,   82,  296,  296,  296,  772,
 1154, 1155, 1156,  777,  296,   68,  789,   69,   70,  417,  417,  791, 1164, 1165,  596,   99,  464,  420,  420,  823,  797, 1199,  694,  798,  694,
  694,  283,  284,  800,  805,  141,  694,  694,  694,  694,  694,  694,  694,  806,  694,  807,  808,  694,  694,  694,  809,  694,  694,  810,  811,
  812,  694,  694,  691,  694,  103,  694,  694,  694,  694,  694, 1198,  694,  694,  694,  694,  813,  694,  694,  814,  694,  694,  815,  816,  694,
  817,  819,  820,  694,  694,  829,  824,  694,  694,  694,  694,  694,  694,  694,  694,  622,  694,  694,  694,  825,  826,  694,  876,  694,  694,
  881,  694,  694,  662,  694,  694,  882,  694,  694,  694,  886,  883,  884,  694,  694,  694,  694,  694,  660,  694,  694,  885,  896,  605,  694,
  694,  694,  926,  694,  694,  262,  694,  694,  694,  694,  694,  920,  921,  922,   68,  923,   69,   70,  622,  934,  662,  943,  957,  964,  985,
  662,  804,  694,  142,  622,  987,  662,  988,  314,  622,  660, 1007, 1016, 1017,  662,  660, 1018, 1024, 1025, 1034,  694,  660, 1038, 1042,  622,
  622, 1048, 1041,  622,  660,  143, 1049, 1050, 1051, 1054,  694,  694,  622,  100,  101,  102,  103,  104,  105, 1056, 1060, 1062,  694,  144,  145,
  146,  147,  148,  149,  150,  151, 1063,  152, 1064,  153,  154,  155,  156, 1065, 1066,  694,  694,  694,  694,  694,  694,  694,  694, 1067,  694,
 1068,  694,  694,  694,  694,  256,  257,  258,  435,  436,  437, 1069,  259,  260,  261,  438,  439,  440,  106,  107, 1070, 1071,  108,  157,  158,
  285,  286,  287,  159, 1072,  160,  161, 1073, 1074, 1075, 1077,  120, 1078, 1079, 1083,  694,  694,  694,  694,  694,  694, 1108,  694,  694,  121,
  122, 1112,  694,  123, 1122,  124, 1124, 1125, 1126,  694, 1136, 1129,  125, 1130,  126,  127,  128,  129,  130,  131,  132,  133,  162,  134,  135,
  136,  163,  164, 1131, 1132, 1133, 1134, 1143, 1145, 1144, 1151, 1163, 1152,  103,  694,  103,  103, 1158,  694,  694, 1161, 1190,  103,  103,  103,
  103,  103,  103,  103, 1167,  103, 1184, 1185,  103,  103,  103, 1195,  103,  103, 1186, 1194, 1201,  103,  103, 1202,  103,  104,  103,  103,  103,
  103,  103, 1203,  103,  103,  103,  103, 1204,  103,  103, 1206,  103,  103, 1207, 1208,  103,    1,   39,  664,  103,  103,  615,  646,  103,  103,
  103,  103,  103,  103,  103,  103,  162,  103,  103,  103,  685,  376,  103,  633,  103,  103,  605,  103,  103,   69,  103,  103,  553,  103,  103,
  103,   70,  686,  557,  103,  103,  103,  103,  103,   73,  103,  103,  634,  519,  606,  103,  103,  103,   75,  103,  103,  414,  103,  103,  103,
  296,  103,  320,  137,  351,  352,  221,  337,  605,  138,  338,  953,  339,  605,  783, 1052,  605,  605,  103,  605,  605,  887,  345,  796, 1188,
 1146,  605,  605, 1200,  604,  554, 1119,  605,  605,  940,  767,  103, 1043,  822,  605,   92,  605,  605,  605, 1098,  897, 1157,  895, 1009,  116,
  605,  605,  103,  248,    0,  731,    0,  903,    0,  605,  605,    0,    0,  103,    0,    0,  605,    0,    0,    0,    0,    0,    0,  376,  376,
  376,  376,    0,    0,    0,    0,  103,  103,  103,  103,  103,  103,  103,  103,    0,  103,    0,  103,  103,  103,  103,    0,  904,    0,    0,
    0,    0,  376,  376,  376,  376,    0,    0,    0,    0,    7,    0,    0,    0,    0,    0,    0,    0,  376,  905,  376,  376,    0,   11,   12,
    0,    0,    0,    0,    0,    0,  103,  103,  103,  103,  103,  103,  906,  103,  103,    0,    0,    0,    0,    0,    0,    0,  907,    0,  325,
  103,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  325,   76,   77,   78,   79,   80,   81,   82,   83,   84,   85,   86,   87,
   88,   89,   90,    0,  104,  103,  104,  104,    0,  103,  103,    0,    0,  104,  104,  104,  104,  104,  104,  104,    0,  104,    0,    0,  104,
  104,  104,    0,  104,  104,    0,    0,    0,  104,  104,    0,  104,  107,  104,  104,  104,  104,  104,    0,  104,  104,  104,  104,    0,  104,
  104,    0,  104,  104,    0,    0,  104,    0,    0,    0,  104,  104,    0,    0,  104,  104,  104,  104,  104,  104,  104,  104,    0,  104,  104,
  104,  376,    0,  104,    0,  104,  104,  606,  104,  104,    0,  104,  104,    0,  104,  104,  104,    0,    0,    0,  104,  104,  104,  104,  104,
    0,  104,  104,    0,    0,  607,  104,  104,  104,    0,  104,  104,    0,  104,  104,  104,    0,  104,    0,    0,    0,    0,    0,   42,  606,
   42,   42,    0,    0,  606,    0,    0,  606,  606,  104,  606,  606,    0,    0,    0,    0,    0,  606,  606,    0,    0,    0,    0,  606,  606,
    0,    0,  104,    0,    0,  606,    0,  606,  606,  606,  325,  325,  325,  325,  325,  325,  606,  606,  104,    0,    0,    0, 1170,    0,    0,
  606,  606,    0,    0,  104,    0,    0,  606, 1171,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  104,  104,  104,
  104,  104,  104,  104,  104,    0,  104,    0,  104,  104,  104,  104,    0,    0,    0,  325,  325,    0,    0,  325,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0, 1172, 1173, 1174, 1175, 1176, 1177, 1178, 1179, 1180, 1181, 1182,    0,   42,    0,  104,  104,  104,
  104,  104,  104,    0,  104,  104,    0,    0,   42,   42,    0,    0,   42,    0,   42,    0,  104,    0,    0,    0,    0,   42,    0,   42,   42,
   42,   42,   42,   42,   42,   42,    0,   42,   42,   42,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  107,  104,  107,  107,    0,
  104,  104,    0,    0,  107,  107,  107,    0,  107,  107,  107,    0,  107,    0,    0,  107,  107,  107,    0,    0,  107,    0,    0,    0,  107,
  107,    0,  107,  105,  107,  107,  107,  107,  107,    0,  107,  107,  107,  107,    0,  107,  107,    0,  107,  107,    0, 1183,  107,    0,    0,
    0,  107,  107,    0,    0,  107,  107,  107,  107,  107,  107,  107,  107,    0,  107,  107,  107,    0,    0,  107,    0,  107,  107,  607,  107,
  107,    0,  107,  107,    0,  107,  107,  107,    0,    0,    0,  107,  107,  107,  107,  107,    0,  107,  107,    0,    0,  609,  107,  107,  107,
    0,  107,  107,    0,  107,  107,  107,    0,    0,    0,    0,    0,   42,    0,    0,  607,    0,    0,   42,    0,  607,    0,    0,  607,  607,
  107,  607,  607,    0,    0,    0,    0,    0,  607,  607,    0,    0,    0,    0,  607,  607,    0,    0,  107,    0,    0,  607,    0,  607,  607,
  607,    0,    0,    0,    0,    0,    0,  607,  607,  107,    0,    0,    0,    0,    0,    0,  607,  607,    0,    0,  107,    0,    0,  607,  821,
  608,  609,  610,  611,  612,  613,    0,    0,    0,  614,  615,  616,  617,  107,  107,  107,  107,  107,  107,  107,  107,    0,  107,    0,  107,
  107,  107,  107,  171,  172,  173,  174,  175,    0,    0,    0,    0,    0,    0,  696,    0,  188,  189,  190,  191,    0,  192,  193,  194,  195,
  196,  197,  198,    0,  618,  619,  620,    0,    0,  199,    0,    0,    0,  107,  107,  107,  107,  107,  107,    0,  107,  107,  200,  201,  202,
  203,  204,  205,    0,  696,    0,    0,  107,    0,    0,    0,    0,    0,    0,    0,    0,    0,  696,    0,    0,    0,    0,    0,    0,    0,
    0,  696,    0,    0,    0,  696,  696,    0,    0,    0,    0,  105,  107,  105,  105,    0,  107,  107,    0,  696,  105,  105,  105,    0,  105,
  105,  105,    0,  105,  696,    0,  105,  105,  105,    0,    0,  105,    0,    0,    0,  105,  105,    0,  105,  106,  105,  105,  105,  105,  105,
    0,  105,  105,  105,  105,    0,  105,  105,    0,  105,  105,    0,    0,  105,    0,    0,    0,  105,  105,    0,    0,  105,  105,  105,  105,
  105,  105,  105,  105,    0,  105,  105,  105,    0,    0,  105,    0,  105,  105,  609,  105,  105,    0,  105,  105,    0,  105,  105,  105,    0,
  206,  207,  105,  105,  105,  105,  105,    0,  105,  105,    0,    0,  612,  105,  105,  105,    0,  105,  105,    0,  105,  105,  105,    0,    0,
    0,    0,    0,    0,    0,    0,  609,    0,    0,    0,    0,  609,    0,    0,  609,  609,  105,  609,  609,    0,    0,    0,    0,    0,  609,
  609,    0,    0,    0,    0,  609,  609,    0,    0,  105,    0,    0,  609,    0,  609,  609,  609,    0,    0,    0,    0,    0,    0,  609,  609,
  105,    0,    0,    0,    0,    0,    0,  609,  609,    0,    0,  105,    0,    0,  609,  607,  608,  609,  610,  611,  612,  613,    0,    0,    0,
  614,  615,  616,  617,  105,  105,  105,  105,  105,  105,  105,  105,    0,  105,    0,  105,  105,  105,  105,    0,    0,    0,  821,  608,  609,
  610,  611,  612,  613,    0,    0,    0,  614,  615,  616,  617,    0,    0,    0,    0,    0,    0,    0,    0,    0,  618,  619,  620,    0,    0,
  621,    0,    0,    0,  105,  105,  105,  105,  105,  105,    0,  105,  105,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  105,    0,
    0,    0,    0,  619,  620,    0,    0, 1001,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  622,    0,    0,    0,    0,    0,    0,
    0,    0,  106,  105,  106,  106,    0,  105,  105,    0,    0,  106,  106,  106,    0,  106,  106,  106,    0,  106,    0,    0,  106,  106,  106,
 1187,    0,  106,    0,    0,    0,  106,  106,    0,  106,   75,  106,  106,  106,  106,  106,    0,  106,  106,  106,  106,    0,  106,  106,    0,
  106,  106,    0,    0,  106,    0,    0,    0,  106,  106,    0,    0,  106,  106,  106,  106,  106,  106,  106,  106,    0,  106,  106,  106,    0,
    0,  106,    0,  106,  106,  612,  106,  106,    0,  106,  106,    0,  106,  106,  106,    0,    0,    0,  106,  106,  106,  106,  106,    0,  106,
  106,    0,    0,    0,  106,  106,  106,    0,  106,  106,    0,  106,  106,  106,    0,    0,    0,    0,    0,    0,    0,    0,  612,    0,    0,
    0,    0,  612,    0,    0,  612,  612,  106,  612,  612,    0,    0,    0,    0,    0,  612,  612,    0,    0,    0,    0,  612,  612,    0,    0,
  106,    0,    0,  612,    0,  612,  612,  612,    0,    0,    0,    0,    0,    0,  612,  612,  106,    0,    0,    0,    0,    0,    0,  612,  612,
    0,    0,  106,    0,    0,  612,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  106,  106,  106,  106,  106,
  106,  106,  106,    0,  106,    0,  106,  106,  106,  106,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,   68,    0,   69,   70,    0,    0,    0,    0,    0,    0,    0,  140,    0,  106,  106,  106,  106,  106,
  106,    0,  106,  106,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  106,    0,    0,   68,    0,   69,   70,    0,    0,    0,    0,
    0,    0,    0,  214,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  141,   75,  106,   75,   75,    0,  106,  106,
    0,    0,   75,   75,   75,    0,   75,   75,   75,    0,   75,    0,    0,    0,   75,   75,    0,    0,   75,    0,    0,    0,    0,   75,    0,
   75,   75,   75,   75,   75,   75,   75,    0,   75,   75,   75,   75,    0,   75,   75,    0,   75,   75,    0,    0,   75,    0,    0,    0,   75,
   75,    0,    0,   75,   75,   75,   75,   75,   75,   75,   75,    0,   75,   75,   75,    0,    0,   75,    0,   75,   75,    0,   75,   75,    0,
   75,   75,    0,   75,   75,   75,    0,    0,    0,   75,   75,   75,   75,   75,    0,   75,   75,    0,    0,    0,   75,   75,   75,    0,   75,
   75,    0,   75,   75,   75,    0,  705,  143,  706,    0,    0,    0,    0,    0,    0,  142,    0,    0,    0,    0,    0,    0,    0,   75,    0,
  144,  145,  146,  147,  148,  149,  150,  151,    0,  152,    0,  153,  154,  155,  156,    0,   75,  143,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,   75,    0,    0,  144,  145,  146,  147,  148,  149,  150,  151,   75,  152,    0,  153,  154,  155,  156,
    0,    0,    0,    0,    0,  159,    0,  160,  161,    0,    0,   75,   75,   75,   75,   75,   75,   75,   75,    0,   75,    0,   75,   75,   75,
   75,    0,    0,    0,    0,    0,    0,    0,    0,    0,  157,  158,    0,    0,    0,  159,    0,  160,  161,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  164,    0,    0,    0,    0,    0,    0,   75,   75,   75,   75,   75,   75,    0,   75,   75,    0,    0,    0,    0,    0,
    0,   68,    0,   69,   70,   75,    0,    0,    0,  162,    0,    0,  464,  163,  164,    0,    0,    0,    0,    0,    0,    0,  283,  284,    0,
    0,  141,    0,    0,    0,    0,    0,    0,    0,   75,   75,   75,   75,    0,   75,   75,    0,  547,   75,   75,   75,    0,   75,   75,   75,
    0,   75,    0,    0,    0,   75,   75,  465,    0,   75,    0,    0,    0,  466,   75,    0,   75,    0,   75,   75,   75,   75,   75,    0,   75,
   75,   75,   75,    0,   75,   75,    0,   75,   75,    0,    0,   75,    0,    0,    0,   75,   75,    0,    0,   75,   75,   75,   75,   75,   75,
   75,   75,    0,   75,   75,   75,    0,    0,   75,    0,   75,   75,    0,   75,   75,    0,   75,   75,    0,   75,   75,   75,    0,    0,    0,
   75,   75,   75,   75,   75,    0,   75,   75,    0,    0,    0,   75,   75,   75,    0,   75,   75,    0,   75,   75,   75,    0,    0,    0,  142,
    0,    0,    0,    0,    0,    0,  548,    0,    0,    0,    0,    0,    0,    0,   75,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  143,    0,    0,    0,    0,    0,   75,    0,    0,    0,    0,    0,    0,    0,    0,    0,  143,    0,  144,  145,  146,  147,  148,  149,
  150,  151,    0,  152,    0,  153,  154,  155,  156,   75,  144,  145,  146,  147,  148,  149,  150,  151,    0,  152,    0,  153,  154,  155,  156,
    0,    0,   75,   75,   75,   75,   75,   75,   75,   75,    0,   75,    0,   75,   75,   75,   75,    0,    0,  157,  158,  285,  286,  287,  159,
    0,  160,  161,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  159,    0,  160,  161,    0,  464,    0,    0,    0,    0,
    0,    0,   75,   75,   75,   75,   75,   75,    0,   75,   75,    0,    0,    0,    0,    0,    0,    0,  162,    0,    0,   75,  163,  164,    0,
  547,    0,  547,    0,    0,    0,    0,    0,    0,  547,  547,    0, 1099,    0,  164,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,   75,    0,    0,    0,   75,   75,  547,    0,  547,  547,  547,  547,  547,    0,  547,  547,  547,  547,    0,  547,  547,    0,  547,  547,
  547,    0,  547,    0,    0,    0,    0,  547,    0,    0,  547,  547,    0,  547,  547,  547,  547,  547,    0,  547,  547,  547,  547,  547,  547,
    0,  547,  547,    0,  547,  547,    0,  547,  547,   29,  547,  547,  547,    0,    0,  547,  547,  547,  547,  547,  547,    0,  547,  547,  547,
  547,  547,  547,  547,  547,    0,  547,  547,    0,  547,  547,  547,    0,    0,  548,    0,  548,    0,    0,    0,    0,    0,    0,  548,  548,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  548,    0,  548,  548,
  548,  548,  548,  547,  548,  548,  548,  548,    0,  548,  548,    0,  548,  548,  548,    0,  548,    0,    0,    0,    0,  548,    0,    0,  548,
  548,    0,  548,  548,  548,  548,  548,    0,  548,  548,  548,  548,  548,  548,   30,  548,  548,    0,  548,  548,    0,  548,  548,    0,  548,
  548,  548,    0,    0,  548,  548,  548,  548,  548,  548,    0,  548,  548,  548,  548,  548,  548,  548,  548,    0,  548,  548,    0,  548,  548,
  548,    0,    0,  464,    0,  464,  464,    0,    0,    0,    0,    0,  464,  464,    0,    0,  464,    0,    0,    0,  464,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  464,    0,  464,  464,  464,  464,  464,  548,  464,  464,  464,  464,    0,  464,  464,
    0,  464,  464,   26,    0,  464,    0,    0,    0,    0,  464,    0,    0,  464,  464,    0,  464,  464,  464,  464,  464,    0,  464,  464,  464,
    0,    0,  464,    0,  464,  464,    0,  464,  464,    0,  464,  464,    0,  464,  464,  464,    0,    0,    0,  464,  464,  464,  464,  464,    0,
  464,  464,    0,    0,    0,  464,  464,  464,    0,  464,  464,    0,  464,  464,  464,    0,    0,   29,    0,   29,    0,    0,    0,    0,    0,
    0,   29,   29,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,   28,    0,    0,    0,    0,    0,    0,   29,
    0,   29,   29,   29,   29,   29,    0,   29,   29,   29,   29,    0,   29,   29,    0,   29,   29,   29,    0,   29,    0,    0,    0,    0,   29,
    0,    0,   29,   29,    0,   29,   29,   29,   29,   29,    0,   29,   29,   29,   29,   29,    0,    0,   29,   29,    0,    0,   29,    0,   29,
   29,    0,   29,   29,   29,    0,    0,   29,   29,   29,   29,   29,   29,    0,   29,    0,    0,   29,   29,   29,   29,   29,   30,   29,   30,
    0,   29,   29,   29,    0,    0,   30,   30,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  565,    0,    0,
    0,    0,    0,    0,   30,    0,   30,   30,   30,   30,   30,    0,   30,   30,   30,   30,    0,   30,   30,    0,   30,   30,   30,    0,   30,
    0,    0,    0,    0,   30,    0,    0,   30,   30,    0,   30,   30,   30,   30,   30,    0,   30,   30,   30,   30,   30,    0,    0,   30,   30,
    0,    0,   30,    0,   30,   30,    0,   30,   30,   30,    0,   26,   30,   30,   30,   30,   30,   30,    0,   30,   26,   26,   30,   30,   30,
   30,   30,    0,   30,    0,    0,   30,   30,   30,    0,    0,    0,    0,    0,    0,    0,    0,    0,   26,    0,   26,   26,   26,   26,   26,
    0,   26,   26,   26,   26,    0,   26,   26,    0,   26,   26,   26,  567,   26,    0,    0,    0,    0,   26,    0,    0,   26,   26,    0,   26,
   26,   26,   26,   26,    0,   26,   26,   26,   26,   26,    0,    0,   26,   26,    0,    0,   26,    0,   26,   26,    0,   26,   26,   26,    0,
   28,   26,   26,   26,   26,   26,   26,    0,   26,   28,   28,   26,   26,   26,   26,   26,    0,   26,    0,    0,   26,   26,   26,    0,    0,
    0,    0,    0,    0,    0,    0,    0,   28,    0,   28,   28,   28,   28,   28,    0,   28,   28,   28,   28,    0,   28,   28,    0,   28,   28,
   28,    0,   28,    0,    0,    0,    0,   28,    0,    0,   28,   28,    0,   28,   28,   28,   28,   28,  462,   28,   28,   28,   28,   28,    0,
    0,   28,   28,    0,    0,   28,    0,   28,   28,    0,   28,   28,   28,    0,    0,   28,   28,   28,   28,   28,   28,    0,   28,    0,    0,
   28,   28,   28,   28,   28,  565,   28,  565,    0,   28,   28,   28,    0,    0,  565,  565,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  565,    0,  565,  565,  565,  565,  565,    0,  565,  565,  565,  565,    0,
  565,  565,    0,  565,  565,    0,    0,  565,    0,    0,    0,    0,  565,    0,    0,  565,  565,    0,  565,  565,  565,  565,  565,  463,  565,
  565,  565,    0,    0,  565,    0,  565,  565,    0,  565,  565,    0,  565,  565,    0,  565,  565,  565,    0,    0,    0,  565,  565,  565,  565,
  565,    0,  565,  565,    0,    0,    0,  565,  565,  565,    0,  565,  565,    0,  565,  565,  565,    0,    0,    0,  567,    0,  567,    0,    0,
    0,    0,    0,    0,  567,  567,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  567,    0,  567,  567,  567,  567,  567,    0,  567,  567,  567,  567,    0,  567,  567,    0,  567,  567,    0,    0,  567,    0,    0,
    0,    0,  567,    0,    0,  567,  567,    0,  567,  567,  567,  567,  567,  277,  567,  567,  567,    0,    0,  567,    0,  567,  567,    0,  567,
  567,    0,  567,  567,    0,  567,  567,  567,    0,    0,    0,  567,  567,  567,  567,  567,    0,  567,  567,    0,    0,    0,  567,  567,  567,
    0,  567,  567,    0,  567,  567,  567,    0,    0,    0,  462,  462,    0,    0,  462,    0,    0,    0,  462,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  462,    0,  462,  462,  462,  462,  462,    0,  462,  462,  462,  462,    0,  462,  462,    0,  462,
  462,    0,    0,  462,    0,    0,    0,    0,  462,    0,    0,  462,  462,    0,  462,  462,  462,  462,  462,    0,  462,  462,  462,    0,    0,
  462,  316,  462,  462,    0,  462,  462,    0,  462,  462,    0,  462,  462,  462,    0,    0,    0,  462,  462,  462,  462,  462,    0,  462,  462,
    0,    0,    0,  462,  462,  462,    0,  462,  462,    0,  462,  462,  462,    0,    0,  463,  463,    0,    0,  463,    0,    0,    0,  463,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  463,    0,  463,  463,  463,  463,  463,    0,  463,  463,  463,  463,
    0,  463,  463,    0,  463,  463,    0,    0,  463,    0,    0,    0,    0,  463,    0,    0,  463,  463,    0,  463,  463,  463,  463,  463,    0,
  463,  463,  463,    0,    0,  463,    0,  463,  463,    0,  463,  463,    0,  463,  463,    0,  463,  463,  463,    0,    0,    0,  463,  463,  463,
  463,  463,    0,  463,  463,    0,    0,    0,  463,  463,  463,    0,  463,  463,    0,  463,  463,  463,    0,    0,    0,  277,    0,  277,    0,
    0,    0,    0,    0,    0,  277,  277,    0,    0,    0,    0,  277,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  277,    0,  277,  277,  277,  277,  277,    0,  277,  277,  277,  277,    0,  277,  277,    0,  277,  277,    0,    0,  277,    0,
    0,    0,    0,  277,    0,    0,  277,  277,    0,  277,  277,  277,  277,  277,    0,  277,  277,  277,    0,    0,    0,    0,  277,  277,    0,
    0,  277,    0,  277,  277,    0,  277,  277,  277,    0,    0,    0,  277,  277,  277,  277,  277,   68,  277,   69,   70,    0,    0,  277,  277,
  277,    0,  277,  140,    0,  277,  277,  277,    0,  316,    0,  316,    0,    0,    0,    0,    0,  141,  316,  316,    0,    0,    0,    0,    0,
    0,    0,    0,  316,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  316,    0,  316,  316,  316,  316,  316,    0,  316,
  316,  316,  316,    0,  316,  316,    0,  316,  316,    0,    0,  316,    0,    0,    0,    0,  316,    0,    0,  316,  316,    0,  316,  316,  316,
  316,  316,    0,  316,  316,  316,    0,    0,    0,    0,  316,  316,    0,    0,  316,    0,  316,  316,    0,  316,  316,  316,    0,    0,    0,
  316,  316,  316,  316,  316,    0,  316,    0,    0,    0,    0,  316,  316,  316,    0,  316,    0,    0,  316,  316,  316,  277,  299,  277,  277,
    0,  300,  301,  302,  303,  304,  305,    0,  306,  307,  308,  309,  310,  311,  312,  142,    0,    0,    0,    0,    0,  277,   68,    0,   69,
   70,    0,    0,    0,    0,    0,    0,    0,  464,    0,    0,    0,    0,    0,    0,    0,    0,  143,  594,  284,    0,    0,  141,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  144,  145,  146,  147,  148,  149,  150,  151,    0,  152,    0,  153,  154,  155,  156,
    0,    0,    0,  465,    0,    0,    0,    0,    0,  466,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  157,  158,    0,    0,    0,  159,    0,  160,  161,    0,    0,    0,    0,    0,    0,
    0,  277,    0,    0,    0,  277,  277,  277,  277,  277,  277,    0,  277,  277,  277,  277,  277,  277,  277,  277,    0,    0,    0,    0,    0,
    0,  131,    0,  131,  131,    0,    0,    0,    0,  162,    0,    0,  131,  163,  164,    0,    0,    0,    0,    0,  142,  277,  131,  131,    0,
    0,  131,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  277,  277,  277,  277,  277,  277,  277,  277,  143,  277,    0,
  277,  277,  277,  277,    0,    0,    0,  131,    0,    0,    0,    0,    0,  131,    0,  144,  145,  146,  147,  148,  149,  150,  151,    0,  152,
    0,  153,  154,  155,  156,    0,    0,    0,    0,    0,    0,  328,    0,    0,  277,  277,    0,    0,    0,  277,    0,  277,  277,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  551,    0,  551,  551,    0,  157,  158,  285,  286,  287,  159,  551,  160,  161,    0,
    0,    0,    0,    3,    0,    0,    0,    0,    4,    0,  551,    5,    6,  277,    7,    8,    0,  277,  277,    0,    0,    9,   10,    0,  131,
    0,    0,   11,   12,    0,    0,    0,    0,    0,   13,    0,   14,   15,   16,  162,    0,    0,    0,  163,  164,   17,   18,    0,    0,    0,
    0,  131,    0,    0,   19,   20,    0,    0,    0,    0,    0,   21,    0,    0,    0,    0,    0,    0,    0,  131,  131,  131,  131,  131,  131,
  131,  131,    0,  131,    0,  131,  131,  131,  131,    0,    0,    0,    0,    0,    0,    0,   68,    0,   69,   70,    0,    0,    0,    0,    0,
    0,    0,  140,    0,    0,    0,  551,  551,  551,  551,  551,    0,    0,    0,    0,    0,  141,    0,    0,  131,  131,  131,  131,  131,  131,
    0,  131,  131,    0,    0,    0,    0,    0,  551,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  394,    0,  394,  394,    0,    0,    0,    0,    0,  551,    0,  394,    0,    0,  422,  422,    0,  131,    0,    0,    0,  131,  131,    0,
    0,  394,    0,  551,  551,  551,  551,  551,  551,  551,  551,    0,  551,    0,  551,  551,  551,  551,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  712,    0,    0,   68,    0,   69,   70,    0,    0,    0,    0,    0,    0,
    0,  214,    0,  551,  551,  713,    0,  714,  551,    0,  551,  551,    0,    0,  142,  141,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  143,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  551,    0,    0,    0,  551,  551,    0,  144,  145,  146,  147,  148,  149,  150,  151,    0,  152,    0,  153,  154,  155,  156,  394,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,  394,    0,    0,  394,    0,  394,  394,    0,  157,  158,    0,    0,    0,  159,  394,  160,  161,  413,  394,  394,  394,  394,  394,  394,
  394,  394,    0,  394,  394,  394,  394,  394,  394,    0,    0,    0,    0,  142,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  162,    0,    0,    0,  163,  164,    0,  143,    0,    0,    0,  394,  394,    0,    0,    0,  394,
    0,  394,  394,    0,    0,    0,    0,    0,  144,  145,  146,  147,  148,  149,  150,  151,    0,  152,    0,  153,  154,  155,  156,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,   68,    0,   69,   70,    0,    0,    0,    0,  394,    0,    0,  140,  394,  394,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  531,  158,  141,    0,    0,  159,    0,  160,  161,    0,    0,   68,    0,   69,   70,    0,    0,
    0,    0,  394,    0,  532,  214,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  141,   68,    0,   69,   70,    0,
    0,    0,    0,  329,  394,    0,  480,  162,    0,    0,    0,  163,  164,    0,    0,    0,    0,    0,    0,    0,  141,    0,  394,  394,  394,
  394,  394,  394,  394,  394,    0,  394,    0,  394,  394,  394,  394,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  330,    0,    0,    0,    0,    0,    7,    8,    0,    0,  331,    0,    0,    9,    0,    0,    0,    0,    0,   11,   12,    0,  394,  394,    0,
    0,   13,  394,    0,  394,  394,    0,    0,  142,  332,  333,  334,   17,   18,    0,  335,    0,    0,    0,    0,  336,    0,    0,    0,    0,
    0,    0,   68,    0,   69,   70,    0,    0,    0,    0,  143,    0,    0,  214,    0,    0,    0,  142,    0,    0,    0,  394,    0,    0,    0,
  394,  394,  141,  144,  145,  146,  147,  148,  149,  150,  151,    0,  152,    0,  153,  154,  155,  156,  142,  143,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  144,  145,  146,  147,  148,  149,  150,  151,  143,  152,    0,  153,  154,
  155,  156,    0,  157,  158,    0,    0,    0,  159,    0,  160,  161,    0,  144,  145,  146,  147,  148,  149,  150,  151,    0,  152,    0,  153,
  154,  155,  156,    0,    0,    0,    0,    0,    0,    0,    0,    0,  157,  158,    0,    0,    0,  159,    0,  160,  161,    0,   68,    0,   69,
   70,    0,  162,    0,    0,    0,  163,  164,  745,    0,    0,    0,    0,  157,  158,    0,    0,    0,  159,    0,  160,  161,  141,    0,    0,
  142,    0,    0,    0,    0,    0,   68,    0,   69,   70,    0,  162,    0,    0,    0,  163,  164, 1192,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  143,    0,    0,    0,  141,    0,    0,    0,    0,    0,  162,    0,    0,    0,  163,  164,    0,    0,  144,  145,  146,  147,  148,
  149,  150,  151,    0,  152,    0,  153,  154,  155,  156,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  170,    0,  170,  170,    0,    0,    0,    0,    0,    0,    0,  170,    0,    0,    0,  546,  158,    0,    0,    0,
  159,    0,  160,  161,    0,  170,    0,    0,    0,    0,    0,    0,    0,  394,    0,  394,  394,    0,    0,    0,  142,    0,    0,    0,  394,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  394,    0,    0,    0,    0,    0,  162,    0,    0,  143,  163,  164,
    0,    0,    0,    0,  142,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  144,  145,  146,  147,  148,  149,  150,  151,    0,  152,
    0,  153,  154,  155,  156,    0,  143,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  144,
  145,  146,  147,  148,  149,  150,  151,    0,  152,    0,  153,  154,  155,  156,    0,  157,  158,    0,    0,    0,  159,    0,  160,  161,    0,
    0,    0,    0,  170,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  157,
  158,    0,    0,    0,  159,  170,  160,  161,    0,    0,    0,  394,    0,    0,  162,    0,    0,    0,  163,  164,    0,    0,    0,  170,  170,
  170,  170,  170,  170,  170,  170,    0,  170,    0,  170,  170,  170,  170,  394,    0,    0,    0,    0,    0,    0,    0,    0,    0,  162,    0,
    0,    0,  163,  164,    0,    0,  394,  394,  394,  394,  394,  394,  394,  394,    0,  394,    0,  394,  394,  394,  394,    0,    0,  170,  170,
    0,    0,    0,  170,    0,  170,  170,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  393,
    0,  393,    0,    0,    0,    0,  394,  394,  393,  393,    0,  394,  393,  394,  394,    0,    0,    0,    0,    0,    0,    0,  170,    0,    0,
    0,  170,  170,  393,    0,    0,  393,    0,  393,  393,  393,  393,  393,    0,  393,  393,  393,  393,    0,  393,  393,    0,  393,  393,    0,
    0,    0,    0,    0,    0,  394,    0,    0,    0,  394,  394,    0,  393,  393,  393,  393,    0,    0,  393,    0,    0,    0,    0,    0,    0,
  393,  393,    0,    0,  393,    0,  393,    0,    0,    0,    0,    0,    0,    0,    0,  393,    0,  393,  393,  393,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  498,  393,  498,    0,  393,    0,  393,    0,  693,  498,  498,  693,    0,    0,    0,    0,    0,  693,    0,    0,    0,
  693,  693,    0,  693,    0,    0,    0,    0,    0,    0,    0,  498,    0,  498,  498,  498,  498,  498,    0,  498,  498,  498,  498,    0,  498,
  498,    0,  498,  498,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  835,    0,  498,  498,  498,  498,    0,    0,  498,    0,
    0,    0,    0,    0,    0,  498,  498,    0,    0,  498,    0,  498,    0,    0,    0,    0,    0,    0,    0,    0,  498,    0,  498,  498,  498,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  498,    0,    0,  498,    0,  498,    0,    0,    0,    0,    0,  517,    0,  517,    0,
    0,    0,    0,    0,  693,  517,  517,  693,    0,    0,    0,    0,    0,  693,    0,    0,    0,  693,  693,    0,  693,    0,    0,    0,    0,
    0,    0,    0,  517,  393,  517,  517,  517,  517,  517,    0,  517,  517,  517,  517,    0,  517,  517,    0,  517,  517,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  517,  517,  517,  517,    0,    0,  517,    0,    0,    0,    0,    0,    0,  517,  517,    0,
    0,  517,    0,  517,    0,    0,    0,    0,    0,    0,    0,    0,  517,    0,  517,  517,  517,    0,    0,    0,  627,    0,    0,    0,    0,
    0,    0,  517,    0,    0,  517,    0,  517,    0,  836,  837,    0,  838,  839,  840,  841,  842,  843,  844,  845,  846,  847,    0,    0,  693,
  693,  693,  848,  849,  850,  851,  852,  628,    0,    0,  853,  854,  629,  855,  856,    0,    0,    0,  630,    0,    0,  857,    0,    0,  858,
  859,  860,  861,  862,  863,  864,  865,  866,  867,  868,  869,  870,    0,    0,    0,    0,    0,    0,  631,  632,    0,    0,  633,  634,  635,
  636,  637,  638,  639,  640,  641,  642,  643,  644,  645,    0,  646,  647,  648,  649,  650,  651,  652,  653,  654,  655,  656,  657,  658,  659,
  660,  661,  662,  663,  664,    0,    0,    0,  665,    0,    0,    0,   68,    0,   69,    0,    0,    0,    0,    0,    0,  367,  368,    0,    0,
  666,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  693,  693,  693,  369,    0,  370,  371,  372,  373,
  374,    0,  375,  376,  377,  378,    0,  379,  380,    0,  381,  382,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    7,    8,  383,  384,    0,    0,  385,    0,    0,    0,    0,    0,    0,   11,   12,    0,    0,  386,    0,  387,    0,    0,    0,    0,   68,
    0,   69,    0,  388,    0,  389,   17,   18,  367,  758,    0,    0,    0,    0,    0,    0,    0,    0,  390,    0,    0,  391,    0,  392,    0,
    0,    0,    0,    0,    0,    0,  369,    0,  370,  371,  372,  373,  374,    0,  375,  376,  377,  378,    0,  379,  380,    0,  381,  382,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    7,    8,  383,  384,    0,    0,  385,    0,    0,    0,    0,    0,    0,
   11,   12,    0,    0,  386,    0,  387,    0,    0,    0,    0,  500,    0,  500,    0,  388,    0,  389,   17,   18,  500,  500,    0,    0,    0,
    0,    0,    0,    0,    0,  390,    0,    0,  391,    0,  392,    0,    0,    0,    0,    0,    0,    0,  500,    0,  500,  500,  500,  500,  500,
    0,  500,  500,  500,  500,    0,  500,  500,    0,  500,  500,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  500,
  500,  500,  500,    0,    0,  500,    0,    0,    0,    0,    0,    0,  500,  500,    0,    0,  500,    0,  500,    0,    0,    0,    0,  467,    0,
  467,    0,  500,    0,  500,  500,  500,  467,  467,    0,    0,    0,    0,    0,    0,    0,    0,  500,    0,    0,  500,    0,  500,    0,    0,
    0,    0,    0,    0,    0,  467,    0,  467,  467,  467,  467,  467,    0,  467,  467,  467,  467,    0,  467,  467,    0,  467,  467,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  467,  467,  467,  467,    0,    0,  467,    0,    0,    0,    0,    0,    0,  467,
  467,    0,    0,  467,    0,  467,    0,    0,    0,    0,  437,    0,  437,    0,  467,    0,  467,  467,  467,  437,  437,    0,    0,    0,    0,
    0,    0,    0,    0,  467,    0,    0,  467,    0,  467,    0,    0,    0,    0,    0,    0,    0,  437,    0,  437,  437,  437,  437,  437,    0,
  437,  437,  437,  437,    0,  437,  437,    0,  437,  437,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  437,  437,
  437,  437,    0,    0,  437,    0,    0,    0,    0,    0,    0,  437,  437,    0,    0,  437,    0,  437,    0,    0,    0,    0,  296,    0,  296,
    0,  437,    0,  437,  437,  437,  296,  296,    0,    0,    0,    0,    0,    0,    0,    0,  437,    0,    0,  437,    0,  437,    0,    0,    0,
    0,    0,    0,    0,  296,    0,  296,  296,  296,  296,  296,    0,  296,  296,  296,  296,    0,  296,  296,    0,  296,  296,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,  296,  296,  296,  296,    0,    0,  296,    0,    0,    0,    0,    0,    0,  296,  296,
    0,    0,  296,    0,  296,    0,    0,    0,    0,    0,    0,    0,    0,  296,    0,  296,  296,  296,    3,    0,    0,    0,    0,    4,    0,
    0,    5,    6,  296,    7,    8,  296,    0,  296,    0,    0,    9,   10,    0,    0,    0,    0,   11,   12,    0,    0,    0,    0,    0,   13,
    0,   14,   15,   16,    0,    0,    0,    0,    0,    0,   17,   18,    0,    0,    0,    0,    0,    0,    0,   19,   20,    0,    0,    0,    0,
    0,   21,
};

static readonly short[] yyCheck =
{
    6,  118,   14,  208,   16,   91,   12,  295,  158,  236,  461,  462,   57,   60,   20,  478,  278,  258,  280,  260,  261,  222,   67,  140,  443,
   92,  222,  804,  258,  243,  260,  261,   56,  222,  876,  928,  259,  222,  259,   51,  886,   19,  243,  271,   98,  162,  222,  243,  244,  245,
  246,  247,   64,  943,  243,   61,  267,   63,  243,   71,  531,  258,   74,  260,  261,  268,  558,  243,  244,  561,  264,  964,  269,  218,  258,
  546,  260,  261,  266,  100,  101,  102,  103,  104,  105,  267,  283, 1016,  266,   95,  258,  259,  260,  261,  141,  263,  241,  214,  269,  378,
  258,  269,  260,  261,  267,  435,  258,  119,  260,  261,  282,  270,  157,  273,  258,  281,  260,  261,  258,  269,  260,  269,  281,  263,  271,
  273,  171,  172,  288,  263,  378,  248,  280, 1026,  281,  141,  270,  263,  269,  286,  263,  272,  273,  283,  270,  372,  324,  264,  266,  280,
  284,  296,  176,  259,  267,  281,  269,  181,  453,  454,  455,  456,  279,  393,  394,  272,  279,  280,  263,  287,  589,  259,  278,  320,  324,
  258,  390,  260,  225,  263,  264,  187,  370,  478,  271,  263,  269,  271,  259,  419,  278,  521,  258,  481,  260,  261,  243,  273,  210,  371,
  266,  267,  548,  269,  321,  371,  284,  278,  272,  295,  421,  422,  288,  279,  280, 1108,  282,  281,  271,  266,  226,  269,  228, 1152,  230,
  278,  378,  372,  269,  296,  271,  452,  453,  454,  455,  456,  457,  458,  279,  280,  263,  462,  463,  464,  465,  260,  332,  460,  461, 1139,
  258,  318,  260,  277,  232,  273,  263,  264,  267,  422,  266,  269,  271,  457,  458,  478,  313,  759, 1161,  270,  288,  279,  280, 1202,  273,
  269,  288,  384,  385,  386,  387,  282,  281,  295,  563,  273,  514,  508,  509,  512,  376,  512,  258,  379,  260,  319,  363,  364,  352, 1145,
  288,  564,  388,  269,  545,  502,  413,  414,  415,  416,  377,  461,  462,  279,  280, 1208, 1158,  605,  268,  522,  523,    0,  429,  370,  431,
  432,  373,  333,  549,  446,  336,  561,  260,  378,  381,  382,  507,  271,  520,  487,  554,  408,  556,  390,  278, 1122,  274,  464,  267,  416,
  270,  375,  258,  554,  260,  502,  487,  554,  554,  272,  310,  545,  282,  480,  370,  554,  483,  373,  281,  554,  789,  321, 1149,  457,  458,
  381,  382,  383,  554,  329,  388,  387,  540,  528,  390,  391,  270,  267,  556,  557,  503,  504,  505,  371,  372,  272,  540,  270,  282,  269,
  273,  271,  279,  550,  410,  275,  492,  520,  270,  279,  280,  273,  282,  419,  426,  270,  879,  287,  563,  431,  565,  709,  567,  568,  267,
  570,  571,  282,  452,  270,  453,  454,  455,  904,  905,  906,  907,  503,  504,  505,  526,  282,  913,  914,  915,  492,  273,  268,  495,  272,
  457,  458,  273,  272,  563,  562,  269,  268,  281,  576,  370,  578,  503,  504,  505,  753,  279,  280,  258,  481,  260,  261,  496,  282,  278,
  592,  621,  281,  287,  526,  597,  502,  503,  504,  505,  268,  531,  508,  509,  536,  273,  258,  267,  260,  261,  503,  504,  505,  266,  310,
  524,  546,  269,  271,  315,  267,  530,  554,  532,  556,  321,  267,  279,  280,  259,  716,  717,  267,  329,  543,  716,  717,  267,  272,  273,
  670,  271,  716,  717,  266,  536,  555,  794,  269,  502,  271,  542,  503,  504,  505,  587,  269,  548,  271,  694,  455,  456,  271,  269,  699,
  556,  272,  258,  279,  260,  261,  562,  581,  279,  280,  269,  573,  574,  272,  279,  384,  385,  386,  387,  282,  279,  280, 1023,  262,  263,
  258,  693,  260,  261,  446,  447,  269,  809,  810,  272,  258,  791,  260,  261,  268,  735,  279,  598,  605,  413,  414,  415,  416,  277,  278,
  510,  511,  278,  393,  394,  672,  269,  269,  269,  271,  429,  272,  431,  432,  262,  263,  279,  280,  279,  272,  273,  668,    0,  258,  258,
  260,  260,  694,  745,  263,  260,  310,  267,  503,  504,  505,  315,  343,  278,  318,  319,  314,  321,  322,  270,  280,  325,  320,  270,  328,
  329,  273,  271,  700,  740,  334,  335,  269,  268,  271,  343,  281,  341,  925,  343,  344,  345,  269,  271,  271,  349,  350,  351,  352,  353,
  258,  355,  260,  261,  270,  263,  360,  361,  362,  269,  271,  271,  270,  694,  368,  503,  504,  505,  740,  271,  270,  708,  709,  273,  746,
  310,  284,  272,  715,  710,    0,  389,  390,  391,  756,  258,  321,  260,  261,  738,  258,  763,  260,  261,  329,  732,  269,  269,  421,  271,
  749,  750,  751,  258,  271,  260,  261,  268,  263,  478,  267,  503,  504,  505,  745,  271,  753,  258,  562,  260,  261, 1034,  287,  280,  755,
  258,  757,  260,  269,  271,  269,  271,  763,  272,  826,  343,    0,  967,  279,  280,  279,  772,  258,  792,  260,  261,  777,  556,  557,  310,
  781,  332,  801,  269,  315,  272,  273,  503,  504,  505,  321,  379,  380,  381,  382,  383,  815,  273,  329,  503,  504,  505,  803,  314,  268,
  343,  258, 1090,  260,  320,  258,  263,  260,  261,  876,  267, 1016,  268,  343,  384,  385,  386,  387,  453,  454,  455,  456,  457,  458, 1112,
  272,  273,  805,  806,  807,  808,  809,  810,  811,  812,  813,  814,  260,  816,  817,  503,  504,  505,  413,  414,  415,  416,  269,  272,  273,
  272, 1056,  321,  522,  523,  941, 1001,  279,  259,  429,  259,  431,  432,  332,  507,  321,  872,  267,  337,  559,  560,  267,  388,  508,  509,
  272,  332,  512,  272,  926,  272,  337,  453,  454,  455,  456,  933,  356,  357,    0,  268,  271,  939,  272,  269,  272,  365,  272,  272,  273,
  356,  357, 1190,  271,  279,  268,  953,  478,  272,  365,  269,  272,  273,  272,  938,  559,  560,  929,  965,  272,  279,  453,  454,  455,  456,
  457,  458,  933,  995,  941,  265,  955,  310,  272,  273,  959,  269,  315,  272,  273,  318,  319,  411,  321,  322,  272,  273, 1152,  271, 1099,
  328,  329,  272,  273,  332,  411,  334,  335,  321,  453,  454,  455,  456,  341,  274,  343,  344,  345,  331,  272,  273,  334,  335,  268,  352,
  353,  508,  509,  541, 1024,  512,  987,  988,  361,  362,  348,  278,  503,  504,  505,  368, 1038,  562,  999, 1041,  278,  359, 1202,  272,  273,
  268,  278, 1049,  384,  385,  386,  387,  278, 1055,  272,  273, 1017, 1018, 1080,  422,  310,  516,  517,  518,  519,  315,  378, 1034,  318,  319,
  284,  321,  322,  378,  268,  325,  272,  273,  328,  329,  272,  273, 1061,  277,  334,  335,  378, 1048,  275, 1050, 1051,  341,  271,  343,  344,
  345,  370, 1138,  321,  349,  350,  351,  352,  353,  278,  355,  272,  273,  260,  332,  360,  361,  362,  278,  337,  275,  310,  272,  368, 1121,
 1192,  315, 1101, 1090,  318,  319, 1087,  321,  322,  352,  353,  325,  271,  356,  328,  329, 1027, 1028, 1029, 1030,  334,  335,  365,  272,  273,
 1112,  515,  341,  275,  343,  344,  345,  272,  273,  515,  349,  350,  351,  352,  353,  274,  355,  272,  273,  272,  273,  360,  361,  362,  274,
 1131, 1132, 1133,  274,  368,  258,  273,  260,  261,  272,  273,  271, 1143, 1144, 1163,  280,  269,  272,  273,  482,  272, 1193,  258,  272,  260,
  261,  279,  280,  284,  271,  283,  267,  268,  269,  270,  271,  272,  273,  271,  275,  271,  271,  278,  279,  280,  271,  282,  283,  271,  271,
  271,  287,  288, 1190,  290,    0,  292,  293,  294,  295,  296, 1192,  298,  299,  300,  301,  271,  303,  304,  271,  306,  307,  271,  271,  310,
  271,  271,  268,  314,  315,  452,  468,  318,  319,  320,  321,  322,  323,  324,  325,  268,  327,  328,  329,  476,  271,  332,  278,  334,  335,
  274,  337,  338,  268,  340,  341,  274,  343,  344,  345,  271,  270,  270,  349,  350,  351,  352,  353,  268,  355,  356,  270,  267,    0,  360,
  361,  362,  275,  364,  365,  287,  367,  368,  369,  370,  371,  272,  272,  272,  258,  272,  260,  261,  321,  270,  310,  270,  284,  271,  260,
  315,  271,  388,  406,  332,  269,  321,  269,  324,  337,  310,  277,  271,  271,  329,  315,  271,  513,  270,  275,  406,  321,  275,  270,  352,
  353,  515,  371,  356,  329,  433,  515,  274,  274,  272,  421,  422,  365,  453,  454,  455,  456,  457,  458,  273,  284,  270,  433,  451,  452,
  453,  454,  455,  456,  457,  458,  272,  460,  272,  462,  463,  464,  465,  272,  272,  451,  452,  453,  454,  455,  456,  457,  458,  272,  460,
  272,  462,  463,  464,  465,  393,  394,  395,  393,  394,  395,  272,  400,  401,  402,  400,  401,  402,  508,  509,  272,  272,  512,  501,  502,
  503,  504,  505,  506,  272,  508,  509,  272,  272,  272,  272,  380,  272,  272,  270,  501,  502,  503,  504,  505,  506,  271,  508,  509,  393,
  394,  275,  513,  397,  271,  399,  270,  270,  270,  520,  260,  272,  406,  272,  408,  409,  410,  411,  412,  413,  414,  415,  550,  417,  418,
  419,  554,  555,  274,  274,  274,  272,  274,  271,  274,  270,  287,  271,  258,  550,  260,  261,  278,  554,  555,  271,  275,  267,  268,  269,
  270,  271,  272,  273,  272,  275,  272,  272,  278,  279,  280,  271,  282,  283,  272,  272,  260,  287,  288,  271,  290,    0,  292,  293,  294,
  295,  296,  270,  298,  299,  300,  301,  272,  303,  304,  288,  306,  307,  272,  271,  310,    0,  267,  267,  314,  315,  267,  267,  318,  319,
  320,  321,  322,  323,  324,  325,  273,  327,  328,  329,  272,  272,  332,  267,  334,  335,  268,  337,  338,  267,  340,  341,  268,  343,  344,
  345,  267,  272,  268,  349,  350,  351,  352,  353,  271,  355,  356,  267,  267,    0,  360,  361,  362,  271,  364,  365,  272,  367,  368,  369,
  267,  371,  272,  547,  267,  267,   96,  222,  310,  553,  222,  783,  222,  315,  579,  953,  318,  319,  388,  321,  322,  692,  222,  600, 1158,
 1121,  328,  329, 1194,  462,  404, 1056,  334,  335,  750,  559,  406,  938,  625,  341,   18,  343,  344,  345, 1021,  699, 1135,  696,  878,   47,
  352,  353,  422,  116,   -1,  515,   -1,  268,   -1,  361,  362,   -1,   -1,  433,   -1,   -1,  368,   -1,   -1,   -1,   -1,   -1,   -1,  384,  385,
  386,  387,   -1,   -1,   -1,   -1,  451,  452,  453,  454,  455,  456,  457,  458,   -1,  460,   -1,  462,  463,  464,  465,   -1,  308,   -1,   -1,
   -1,   -1,  413,  414,  415,  416,   -1,   -1,   -1,   -1,  321,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  429,  330,  431,  432,   -1,  334,  335,
   -1,   -1,   -1,   -1,   -1,   -1,  501,  502,  503,  504,  505,  506,  348,  508,  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  358,   -1,  267,
  520,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  280,  524,  525,  526,  527,  528,  529,  530,  531,  532,  533,  534,  535,
  536,  537,  538,   -1,  258,  550,  260,  261,   -1,  554,  555,   -1,   -1,  267,  268,  269,  270,  271,  272,  273,   -1,  275,   -1,   -1,  278,
  279,  280,   -1,  282,  283,   -1,   -1,   -1,  287,  288,   -1,  290,    0,  292,  293,  294,  295,  296,   -1,  298,  299,  300,  301,   -1,  303,
  304,   -1,  306,  307,   -1,   -1,  310,   -1,   -1,   -1,  314,  315,   -1,   -1,  318,  319,  320,  321,  322,  323,  324,  325,   -1,  327,  328,
  329,  562,   -1,  332,   -1,  334,  335,  268,  337,  338,   -1,  340,  341,   -1,  343,  344,  345,   -1,   -1,   -1,  349,  350,  351,  352,  353,
   -1,  355,  356,   -1,   -1,    0,  360,  361,  362,   -1,  364,  365,   -1,  367,  368,  369,   -1,  371,   -1,   -1,   -1,   -1,   -1,  258,  310,
  260,  261,   -1,   -1,  315,   -1,   -1,  318,  319,  388,  321,  322,   -1,   -1,   -1,   -1,   -1,  328,  329,   -1,   -1,   -1,   -1,  334,  335,
   -1,   -1,  406,   -1,   -1,  341,   -1,  343,  344,  345,  453,  454,  455,  456,  457,  458,  352,  353,  422,   -1,   -1,   -1,  372,   -1,   -1,
  361,  362,   -1,   -1,  433,   -1,   -1,  368,  383,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  451,  452,  453,
  454,  455,  456,  457,  458,   -1,  460,   -1,  462,  463,  464,  465,   -1,   -1,   -1,  508,  509,   -1,   -1,  512,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  434,  435,  436,  437,  438,  439,  440,  441,  442,  443,   -1,  380,   -1,  501,  502,  503,
  504,  505,  506,   -1,  508,  509,   -1,   -1,  393,  394,   -1,   -1,  397,   -1,  399,   -1,  520,   -1,   -1,   -1,   -1,  406,   -1,  408,  409,
  410,  411,  412,  413,  414,  415,   -1,  417,  418,  419,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  258,  550,  260,  261,   -1,
  554,  555,   -1,   -1,  267,  268,  269,   -1,  271,  272,  273,   -1,  275,   -1,   -1,  278,  279,  280,   -1,   -1,  283,   -1,   -1,   -1,  287,
  288,   -1,  290,    0,  292,  293,  294,  295,  296,   -1,  298,  299,  300,  301,   -1,  303,  304,   -1,  306,  307,   -1,  546,  310,   -1,   -1,
   -1,  314,  315,   -1,   -1,  318,  319,  320,  321,  322,  323,  324,  325,   -1,  327,  328,  329,   -1,   -1,  332,   -1,  334,  335,  268,  337,
  338,   -1,  340,  341,   -1,  343,  344,  345,   -1,   -1,   -1,  349,  350,  351,  352,  353,   -1,  355,  356,   -1,   -1,    0,  360,  361,  362,
   -1,  364,  365,   -1,  367,  368,  369,   -1,   -1,   -1,   -1,   -1,  547,   -1,   -1,  310,   -1,   -1,  553,   -1,  315,   -1,   -1,  318,  319,
  388,  321,  322,   -1,   -1,   -1,   -1,   -1,  328,  329,   -1,   -1,   -1,   -1,  334,  335,   -1,   -1,  406,   -1,   -1,  341,   -1,  343,  344,
  345,   -1,   -1,   -1,   -1,   -1,   -1,  352,  353,  422,   -1,   -1,   -1,   -1,   -1,   -1,  361,  362,   -1,   -1,  433,   -1,   -1,  368,  452,
  453,  454,  455,  456,  457,  458,   -1,   -1,   -1,  462,  463,  464,  465,  451,  452,  453,  454,  455,  456,  457,  458,   -1,  460,   -1,  462,
  463,  464,  465,  379,  380,  381,  382,  383,   -1,   -1,   -1,   -1,   -1,   -1,  268,   -1,  392,  393,  394,  395,   -1,  397,  398,  399,  400,
  401,  402,  403,   -1,  507,  508,  509,   -1,   -1,  410,   -1,   -1,   -1,  501,  502,  503,  504,  505,  506,   -1,  508,  509,  423,  424,  425,
  426,  427,  428,   -1,  308,   -1,   -1,  520,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  321,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  330,   -1,   -1,   -1,  334,  335,   -1,   -1,   -1,   -1,  258,  550,  260,  261,   -1,  554,  555,   -1,  348,  267,  268,  269,   -1,  271,
  272,  273,   -1,  275,  358,   -1,  278,  279,  280,   -1,   -1,  283,   -1,   -1,   -1,  287,  288,   -1,  290,    0,  292,  293,  294,  295,  296,
   -1,  298,  299,  300,  301,   -1,  303,  304,   -1,  306,  307,   -1,   -1,  310,   -1,   -1,   -1,  314,  315,   -1,   -1,  318,  319,  320,  321,
  322,  323,  324,  325,   -1,  327,  328,  329,   -1,   -1,  332,   -1,  334,  335,  268,  337,  338,   -1,  340,  341,   -1,  343,  344,  345,   -1,
  551,  552,  349,  350,  351,  352,  353,   -1,  355,  356,   -1,   -1,    0,  360,  361,  362,   -1,  364,  365,   -1,  367,  368,  369,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  310,   -1,   -1,   -1,   -1,  315,   -1,   -1,  318,  319,  388,  321,  322,   -1,   -1,   -1,   -1,   -1,  328,
  329,   -1,   -1,   -1,   -1,  334,  335,   -1,   -1,  406,   -1,   -1,  341,   -1,  343,  344,  345,   -1,   -1,   -1,   -1,   -1,   -1,  352,  353,
  422,   -1,   -1,   -1,   -1,   -1,   -1,  361,  362,   -1,   -1,  433,   -1,   -1,  368,  452,  453,  454,  455,  456,  457,  458,   -1,   -1,   -1,
  462,  463,  464,  465,  451,  452,  453,  454,  455,  456,  457,  458,   -1,  460,   -1,  462,  463,  464,  465,   -1,   -1,   -1,  452,  453,  454,
  455,  456,  457,  458,   -1,   -1,   -1,  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  507,  508,  509,   -1,   -1,
  512,   -1,   -1,   -1,  501,  502,  503,  504,  505,  506,   -1,  508,  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  520,   -1,
   -1,   -1,   -1,  508,  509,   -1,   -1,  512,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  555,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  258,  550,  260,  261,   -1,  554,  555,   -1,   -1,  267,  268,  269,   -1,  271,  272,  273,   -1,  275,   -1,   -1,  278,  279,  280,
  555,   -1,  283,   -1,   -1,   -1,  287,  288,   -1,  290,    0,  292,  293,  294,  295,  296,   -1,  298,  299,  300,  301,   -1,  303,  304,   -1,
  306,  307,   -1,   -1,  310,   -1,   -1,   -1,  314,  315,   -1,   -1,  318,  319,  320,  321,  322,  323,  324,  325,   -1,  327,  328,  329,   -1,
   -1,  332,   -1,  334,  335,  268,  337,  338,   -1,  340,  341,   -1,  343,  344,  345,   -1,   -1,   -1,  349,  350,  351,  352,  353,   -1,  355,
  356,   -1,   -1,   -1,  360,  361,  362,   -1,  364,  365,   -1,  367,  368,  369,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  310,   -1,   -1,
   -1,   -1,  315,   -1,   -1,  318,  319,  388,  321,  322,   -1,   -1,   -1,   -1,   -1,  328,  329,   -1,   -1,   -1,   -1,  334,  335,   -1,   -1,
  406,   -1,   -1,  341,   -1,  343,  344,  345,   -1,   -1,   -1,   -1,   -1,   -1,  352,  353,  422,   -1,   -1,   -1,   -1,   -1,   -1,  361,  362,
   -1,   -1,  433,   -1,   -1,  368,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  451,  452,  453,  454,  455,
  456,  457,  458,   -1,  460,   -1,  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  258,   -1,  260,  261,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  269,   -1,  501,  502,  503,  504,  505,
  506,   -1,  508,  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  520,   -1,   -1,  258,   -1,  260,  261,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  283,  258,  550,  260,  261,   -1,  554,  555,
   -1,   -1,  267,  268,  269,   -1,  271,  272,  273,   -1,  275,   -1,   -1,   -1,  279,  280,   -1,   -1,  283,   -1,   -1,   -1,   -1,  288,   -1,
  290,    0,  292,  293,  294,  295,  296,   -1,  298,  299,  300,  301,   -1,  303,  304,   -1,  306,  307,   -1,   -1,  310,   -1,   -1,   -1,  314,
  315,   -1,   -1,  318,  319,  320,  321,  322,  323,  324,  325,   -1,  327,  328,  329,   -1,   -1,  332,   -1,  334,  335,   -1,  337,  338,   -1,
  340,  341,   -1,  343,  344,  345,   -1,   -1,   -1,  349,  350,  351,  352,  353,   -1,  355,  356,   -1,   -1,   -1,  360,  361,  362,   -1,  364,
  365,   -1,  367,  368,  369,   -1,  397,  433,  399,   -1,   -1,   -1,   -1,   -1,   -1,  406,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  388,   -1,
  451,  452,  453,  454,  455,  456,  457,  458,   -1,  460,   -1,  462,  463,  464,  465,   -1,  406,  433,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  422,   -1,   -1,  451,  452,  453,  454,  455,  456,  457,  458,  433,  460,   -1,  462,  463,  464,  465,
   -1,   -1,   -1,   -1,   -1,  506,   -1,  508,  509,   -1,   -1,  451,  452,  453,  454,  455,  456,  457,  458,   -1,  460,   -1,  462,  463,  464,
  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  501,  502,   -1,   -1,   -1,  506,   -1,  508,  509,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  555,   -1,   -1,   -1,   -1,   -1,   -1,  501,  502,  503,  504,  505,  506,   -1,  508,  509,   -1,   -1,   -1,   -1,   -1,
   -1,  258,   -1,  260,  261,  520,   -1,   -1,   -1,  550,   -1,   -1,  269,  554,  555,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,  280,   -1,
   -1,  283,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  258,  550,  260,  261,   -1,  554,  555,   -1,    0,  267,  268,  269,   -1,  271,  272,  273,
   -1,  275,   -1,   -1,   -1,  279,  280,  314,   -1,  283,   -1,   -1,   -1,  320,  288,   -1,  290,   -1,  292,  293,  294,  295,  296,   -1,  298,
  299,  300,  301,   -1,  303,  304,   -1,  306,  307,   -1,   -1,  310,   -1,   -1,   -1,  314,  315,   -1,   -1,  318,  319,  320,  321,  322,  323,
  324,  325,   -1,  327,  328,  329,   -1,   -1,  332,   -1,  334,  335,   -1,  337,  338,   -1,  340,  341,   -1,  343,  344,  345,   -1,   -1,   -1,
  349,  350,  351,  352,  353,   -1,  355,  356,   -1,   -1,   -1,  360,  361,  362,   -1,  364,  365,   -1,  367,  368,  369,   -1,   -1,   -1,  406,
   -1,   -1,   -1,   -1,   -1,   -1,    0,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  388,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,   -1,   -1,   -1,   -1,   -1,  406,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,  451,  452,  453,  454,  455,  456,
  457,  458,   -1,  460,   -1,  462,  463,  464,  465,  433,  451,  452,  453,  454,  455,  456,  457,  458,   -1,  460,   -1,  462,  463,  464,  465,
   -1,   -1,  451,  452,  453,  454,  455,  456,  457,  458,   -1,  460,   -1,  462,  463,  464,  465,   -1,   -1,  501,  502,  503,  504,  505,  506,
   -1,  508,  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  506,   -1,  508,  509,   -1,    0,   -1,   -1,   -1,   -1,
   -1,   -1,  501,  502,  503,  504,  505,  506,   -1,  508,  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  550,   -1,   -1,  520,  554,  555,   -1,
  258,   -1,  260,   -1,   -1,   -1,   -1,   -1,   -1,  267,  268,   -1,  553,   -1,  555,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  550,   -1,   -1,   -1,  554,  555,  290,   -1,  292,  293,  294,  295,  296,   -1,  298,  299,  300,  301,   -1,  303,  304,   -1,  306,  307,
  308,   -1,  310,   -1,   -1,   -1,   -1,  315,   -1,   -1,  318,  319,   -1,  321,  322,  323,  324,  325,   -1,  327,  328,  329,  330,  331,  332,
   -1,  334,  335,   -1,  337,  338,   -1,  340,  341,    0,  343,  344,  345,   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,  355,  356,  357,
  358,  359,  360,  361,  362,   -1,  364,  365,   -1,  367,  368,  369,   -1,   -1,  258,   -1,  260,   -1,   -1,   -1,   -1,   -1,   -1,  267,  268,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,   -1,  292,  293,
  294,  295,  296,  411,  298,  299,  300,  301,   -1,  303,  304,   -1,  306,  307,  308,   -1,  310,   -1,   -1,   -1,   -1,  315,   -1,   -1,  318,
  319,   -1,  321,  322,  323,  324,  325,   -1,  327,  328,  329,  330,  331,  332,    0,  334,  335,   -1,  337,  338,   -1,  340,  341,   -1,  343,
  344,  345,   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,  355,  356,  357,  358,  359,  360,  361,  362,   -1,  364,  365,   -1,  367,  368,
  369,   -1,   -1,  258,   -1,  260,  261,   -1,   -1,   -1,   -1,   -1,  267,  268,   -1,   -1,  271,   -1,   -1,   -1,  275,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,   -1,  292,  293,  294,  295,  296,  411,  298,  299,  300,  301,   -1,  303,  304,
   -1,  306,  307,    0,   -1,  310,   -1,   -1,   -1,   -1,  315,   -1,   -1,  318,  319,   -1,  321,  322,  323,  324,  325,   -1,  327,  328,  329,
   -1,   -1,  332,   -1,  334,  335,   -1,  337,  338,   -1,  340,  341,   -1,  343,  344,  345,   -1,   -1,   -1,  349,  350,  351,  352,  353,   -1,
  355,  356,   -1,   -1,   -1,  360,  361,  362,   -1,  364,  365,   -1,  367,  368,  369,   -1,   -1,  258,   -1,  260,   -1,   -1,   -1,   -1,   -1,
   -1,  267,  268,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,    0,   -1,   -1,   -1,   -1,   -1,   -1,  290,
   -1,  292,  293,  294,  295,  296,   -1,  298,  299,  300,  301,   -1,  303,  304,   -1,  306,  307,  308,   -1,  310,   -1,   -1,   -1,   -1,  315,
   -1,   -1,  318,  319,   -1,  321,  322,  323,  324,  325,   -1,  327,  328,  329,  330,  331,   -1,   -1,  334,  335,   -1,   -1,  338,   -1,  340,
  341,   -1,  343,  344,  345,   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,  355,   -1,   -1,  358,  359,  360,  361,  362,  258,  364,  260,
   -1,  367,  368,  369,   -1,   -1,  267,  268,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,    0,   -1,   -1,
   -1,   -1,   -1,   -1,  290,   -1,  292,  293,  294,  295,  296,   -1,  298,  299,  300,  301,   -1,  303,  304,   -1,  306,  307,  308,   -1,  310,
   -1,   -1,   -1,   -1,  315,   -1,   -1,  318,  319,   -1,  321,  322,  323,  324,  325,   -1,  327,  328,  329,  330,  331,   -1,   -1,  334,  335,
   -1,   -1,  338,   -1,  340,  341,   -1,  343,  344,  345,   -1,  258,  348,  349,  350,  351,  352,  353,   -1,  355,  267,  268,  358,  359,  360,
  361,  362,   -1,  364,   -1,   -1,  367,  368,  369,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,   -1,  292,  293,  294,  295,  296,
   -1,  298,  299,  300,  301,   -1,  303,  304,   -1,  306,  307,  308,    0,  310,   -1,   -1,   -1,   -1,  315,   -1,   -1,  318,  319,   -1,  321,
  322,  323,  324,  325,   -1,  327,  328,  329,  330,  331,   -1,   -1,  334,  335,   -1,   -1,  338,   -1,  340,  341,   -1,  343,  344,  345,   -1,
  258,  348,  349,  350,  351,  352,  353,   -1,  355,  267,  268,  358,  359,  360,  361,  362,   -1,  364,   -1,   -1,  367,  368,  369,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,   -1,  292,  293,  294,  295,  296,   -1,  298,  299,  300,  301,   -1,  303,  304,   -1,  306,  307,
  308,   -1,  310,   -1,   -1,   -1,   -1,  315,   -1,   -1,  318,  319,   -1,  321,  322,  323,  324,  325,    0,  327,  328,  329,  330,  331,   -1,
   -1,  334,  335,   -1,   -1,  338,   -1,  340,  341,   -1,  343,  344,  345,   -1,   -1,  348,  349,  350,  351,  352,  353,   -1,  355,   -1,   -1,
  358,  359,  360,  361,  362,  258,  364,  260,   -1,  367,  368,  369,   -1,   -1,  267,  268,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,   -1,  292,  293,  294,  295,  296,   -1,  298,  299,  300,  301,   -1,
  303,  304,   -1,  306,  307,   -1,   -1,  310,   -1,   -1,   -1,   -1,  315,   -1,   -1,  318,  319,   -1,  321,  322,  323,  324,  325,    0,  327,
  328,  329,   -1,   -1,  332,   -1,  334,  335,   -1,  337,  338,   -1,  340,  341,   -1,  343,  344,  345,   -1,   -1,   -1,  349,  350,  351,  352,
  353,   -1,  355,  356,   -1,   -1,   -1,  360,  361,  362,   -1,  364,  365,   -1,  367,  368,  369,   -1,   -1,   -1,  258,   -1,  260,   -1,   -1,
   -1,   -1,   -1,   -1,  267,  268,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  290,   -1,  292,  293,  294,  295,  296,   -1,  298,  299,  300,  301,   -1,  303,  304,   -1,  306,  307,   -1,   -1,  310,   -1,   -1,
   -1,   -1,  315,   -1,   -1,  318,  319,   -1,  321,  322,  323,  324,  325,    0,  327,  328,  329,   -1,   -1,  332,   -1,  334,  335,   -1,  337,
  338,   -1,  340,  341,   -1,  343,  344,  345,   -1,   -1,   -1,  349,  350,  351,  352,  353,   -1,  355,  356,   -1,   -1,   -1,  360,  361,  362,
   -1,  364,  365,   -1,  367,  368,  369,   -1,   -1,   -1,  267,  268,   -1,   -1,  271,   -1,   -1,   -1,  275,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,   -1,  292,  293,  294,  295,  296,   -1,  298,  299,  300,  301,   -1,  303,  304,   -1,  306,
  307,   -1,   -1,  310,   -1,   -1,   -1,   -1,  315,   -1,   -1,  318,  319,   -1,  321,  322,  323,  324,  325,   -1,  327,  328,  329,   -1,   -1,
  332,    0,  334,  335,   -1,  337,  338,   -1,  340,  341,   -1,  343,  344,  345,   -1,   -1,   -1,  349,  350,  351,  352,  353,   -1,  355,  356,
   -1,   -1,   -1,  360,  361,  362,   -1,  364,  365,   -1,  367,  368,  369,   -1,   -1,  267,  268,   -1,   -1,  271,   -1,   -1,   -1,  275,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,   -1,  292,  293,  294,  295,  296,   -1,  298,  299,  300,  301,
   -1,  303,  304,   -1,  306,  307,   -1,   -1,  310,   -1,   -1,   -1,   -1,  315,   -1,   -1,  318,  319,   -1,  321,  322,  323,  324,  325,   -1,
  327,  328,  329,   -1,   -1,  332,   -1,  334,  335,   -1,  337,  338,   -1,  340,  341,   -1,  343,  344,  345,   -1,   -1,   -1,  349,  350,  351,
  352,  353,   -1,  355,  356,   -1,   -1,   -1,  360,  361,  362,   -1,  364,  365,   -1,  367,  368,  369,   -1,   -1,   -1,  258,   -1,  260,   -1,
   -1,   -1,   -1,   -1,   -1,  267,  268,   -1,   -1,   -1,   -1,  273,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  290,   -1,  292,  293,  294,  295,  296,   -1,  298,  299,  300,  301,   -1,  303,  304,   -1,  306,  307,   -1,   -1,  310,   -1,
   -1,   -1,   -1,  315,   -1,   -1,  318,  319,   -1,  321,  322,  323,  324,  325,   -1,  327,  328,  329,   -1,   -1,   -1,   -1,  334,  335,   -1,
   -1,  338,   -1,  340,  341,   -1,  343,  344,  345,   -1,   -1,   -1,  349,  350,  351,  352,  353,  258,  355,  260,  261,   -1,   -1,  360,  361,
  362,   -1,  364,  269,   -1,  367,  368,  369,   -1,  258,   -1,  260,   -1,   -1,   -1,   -1,   -1,  283,  267,  268,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  277,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,   -1,  292,  293,  294,  295,  296,   -1,  298,
  299,  300,  301,   -1,  303,  304,   -1,  306,  307,   -1,   -1,  310,   -1,   -1,   -1,   -1,  315,   -1,   -1,  318,  319,   -1,  321,  322,  323,
  324,  325,   -1,  327,  328,  329,   -1,   -1,   -1,   -1,  334,  335,   -1,   -1,  338,   -1,  340,  341,   -1,  343,  344,  345,   -1,   -1,   -1,
  349,  350,  351,  352,  353,   -1,  355,   -1,   -1,   -1,   -1,  360,  361,  362,   -1,  364,   -1,   -1,  367,  368,  369,  258,  388,  260,  261,
   -1,  392,  393,  394,  395,  396,  397,   -1,  399,  400,  401,  402,  403,  404,  405,  406,   -1,   -1,   -1,   -1,   -1,  283,  258,   -1,  260,
  261,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,  279,  280,   -1,   -1,  283,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  451,  452,  453,  454,  455,  456,  457,  458,   -1,  460,   -1,  462,  463,  464,  465,
   -1,   -1,   -1,  314,   -1,   -1,   -1,   -1,   -1,  320,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  501,  502,   -1,   -1,   -1,  506,   -1,  508,  509,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  388,   -1,   -1,   -1,  392,  393,  394,  395,  396,  397,   -1,  399,  400,  401,  402,  403,  404,  405,  406,   -1,   -1,   -1,   -1,   -1,
   -1,  258,   -1,  260,  261,   -1,   -1,   -1,   -1,  550,   -1,   -1,  269,  554,  555,   -1,   -1,   -1,   -1,   -1,  406,  433,  279,  280,   -1,
   -1,  283,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  451,  452,  453,  454,  455,  456,  457,  458,  433,  460,   -1,
  462,  463,  464,  465,   -1,   -1,   -1,  314,   -1,   -1,   -1,   -1,   -1,  320,   -1,  451,  452,  453,  454,  455,  456,  457,  458,   -1,  460,
   -1,  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,  268,   -1,   -1,  501,  502,   -1,   -1,   -1,  506,   -1,  508,  509,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  258,   -1,  260,  261,   -1,  501,  502,  503,  504,  505,  506,  269,  508,  509,   -1,
   -1,   -1,   -1,  310,   -1,   -1,   -1,   -1,  315,   -1,  283,  318,  319,  550,  321,  322,   -1,  554,  555,   -1,   -1,  328,  329,   -1,  406,
   -1,   -1,  334,  335,   -1,   -1,   -1,   -1,   -1,  341,   -1,  343,  344,  345,  550,   -1,   -1,   -1,  554,  555,  352,  353,   -1,   -1,   -1,
   -1,  433,   -1,   -1,  361,  362,   -1,   -1,   -1,   -1,   -1,  368,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  451,  452,  453,  454,  455,  456,
  457,  458,   -1,  460,   -1,  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  258,   -1,  260,  261,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  269,   -1,   -1,   -1,  379,  380,  381,  382,  383,   -1,   -1,   -1,   -1,   -1,  283,   -1,   -1,  501,  502,  503,  504,  505,  506,
   -1,  508,  509,   -1,   -1,   -1,   -1,   -1,  406,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  258,   -1,  260,  261,   -1,   -1,   -1,   -1,   -1,  433,   -1,  269,   -1,   -1,  272,  273,   -1,  550,   -1,   -1,   -1,  554,  555,   -1,
   -1,  283,   -1,  451,  452,  453,  454,  455,  456,  457,  458,   -1,  460,   -1,  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  379,   -1,   -1,  258,   -1,  260,  261,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  269,   -1,  501,  502,  397,   -1,  399,  506,   -1,  508,  509,   -1,   -1,  406,  283,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  433,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  550,   -1,   -1,   -1,  554,  555,   -1,  451,  452,  453,  454,  455,  456,  457,  458,   -1,  460,   -1,  462,  463,  464,  465,  406,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,  433,   -1,   -1,  258,   -1,  260,  261,   -1,  501,  502,   -1,   -1,   -1,  506,  269,  508,  509,  272,  451,  452,  453,  454,  455,  456,
  457,  458,   -1,  460,  283,  462,  463,  464,  465,   -1,   -1,   -1,   -1,  406,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  550,   -1,   -1,   -1,  554,  555,   -1,  433,   -1,   -1,   -1,  501,  502,   -1,   -1,   -1,  506,
   -1,  508,  509,   -1,   -1,   -1,   -1,   -1,  451,  452,  453,  454,  455,  456,  457,  458,   -1,  460,   -1,  462,  463,  464,  465,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  258,   -1,  260,  261,   -1,   -1,   -1,   -1,  550,   -1,   -1,  269,  554,  555,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  501,  502,  283,   -1,   -1,  506,   -1,  508,  509,   -1,   -1,  258,   -1,  260,  261,   -1,   -1,
   -1,   -1,  406,   -1,  522,  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  283,  258,   -1,  260,  261,   -1,
   -1,   -1,   -1,  268,  433,   -1,  269,  550,   -1,   -1,   -1,  554,  555,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  283,   -1,  451,  452,  453,
  454,  455,  456,  457,  458,   -1,  460,   -1,  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  315,   -1,   -1,   -1,   -1,   -1,  321,  322,   -1,   -1,  325,   -1,   -1,  328,   -1,   -1,   -1,   -1,   -1,  334,  335,   -1,  501,  502,   -1,
   -1,  341,  506,   -1,  508,  509,   -1,   -1,  406,  349,  350,  351,  352,  353,   -1,  355,   -1,   -1,   -1,   -1,  360,   -1,   -1,   -1,   -1,
   -1,   -1,  258,   -1,  260,  261,   -1,   -1,   -1,   -1,  433,   -1,   -1,  269,   -1,   -1,   -1,  406,   -1,   -1,   -1,  550,   -1,   -1,   -1,
  554,  555,  283,  451,  452,  453,  454,  455,  456,  457,  458,   -1,  460,   -1,  462,  463,  464,  465,  406,  433,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  451,  452,  453,  454,  455,  456,  457,  458,  433,  460,   -1,  462,  463,
  464,  465,   -1,  501,  502,   -1,   -1,   -1,  506,   -1,  508,  509,   -1,  451,  452,  453,  454,  455,  456,  457,  458,   -1,  460,   -1,  462,
  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  501,  502,   -1,   -1,   -1,  506,   -1,  508,  509,   -1,  258,   -1,  260,
  261,   -1,  550,   -1,   -1,   -1,  554,  555,  269,   -1,   -1,   -1,   -1,  501,  502,   -1,   -1,   -1,  506,   -1,  508,  509,  283,   -1,   -1,
  406,   -1,   -1,   -1,   -1,   -1,  258,   -1,  260,  261,   -1,  550,   -1,   -1,   -1,  554,  555,  269,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,  433,   -1,   -1,   -1,  283,   -1,   -1,   -1,   -1,   -1,  550,   -1,   -1,   -1,  554,  555,   -1,   -1,  451,  452,  453,  454,  455,
  456,  457,  458,   -1,  460,   -1,  462,  463,  464,  465,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  258,   -1,  260,  261,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  269,   -1,   -1,   -1,  501,  502,   -1,   -1,   -1,
  506,   -1,  508,  509,   -1,  283,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  258,   -1,  260,  261,   -1,   -1,   -1,  406,   -1,   -1,   -1,  269,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  283,   -1,   -1,   -1,   -1,   -1,  550,   -1,   -1,  433,  554,  555,
   -1,   -1,   -1,   -1,  406,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  451,  452,  453,  454,  455,  456,  457,  458,   -1,  460,
   -1,  462,  463,  464,  465,   -1,  433,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  451,
  452,  453,  454,  455,  456,  457,  458,   -1,  460,   -1,  462,  463,  464,  465,   -1,  501,  502,   -1,   -1,   -1,  506,   -1,  508,  509,   -1,
   -1,   -1,   -1,  406,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  501,
  502,   -1,   -1,   -1,  506,  433,  508,  509,   -1,   -1,   -1,  406,   -1,   -1,  550,   -1,   -1,   -1,  554,  555,   -1,   -1,   -1,  451,  452,
  453,  454,  455,  456,  457,  458,   -1,  460,   -1,  462,  463,  464,  465,  433,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  550,   -1,
   -1,   -1,  554,  555,   -1,   -1,  451,  452,  453,  454,  455,  456,  457,  458,   -1,  460,   -1,  462,  463,  464,  465,   -1,   -1,  501,  502,
   -1,   -1,   -1,  506,   -1,  508,  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  258,
   -1,  260,   -1,   -1,   -1,   -1,  501,  502,  267,  268,   -1,  506,  271,  508,  509,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  550,   -1,   -1,
   -1,  554,  555,  287,   -1,   -1,  290,   -1,  292,  293,  294,  295,  296,   -1,  298,  299,  300,  301,   -1,  303,  304,   -1,  306,  307,   -1,
   -1,   -1,   -1,   -1,   -1,  550,   -1,   -1,   -1,  554,  555,   -1,  321,  322,  323,  324,   -1,   -1,  327,   -1,   -1,   -1,   -1,   -1,   -1,
  334,  335,   -1,   -1,  338,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  349,   -1,  351,  352,  353,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  258,  364,  260,   -1,  367,   -1,  369,   -1,  266,  267,  268,  269,   -1,   -1,   -1,   -1,   -1,  275,   -1,   -1,   -1,
  279,  280,   -1,  282,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,   -1,  292,  293,  294,  295,  296,   -1,  298,  299,  300,  301,   -1,  303,
  304,   -1,  306,  307,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  279,   -1,  321,  322,  323,  324,   -1,   -1,  327,   -1,
   -1,   -1,   -1,   -1,   -1,  334,  335,   -1,   -1,  338,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  349,   -1,  351,  352,  353,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  364,   -1,   -1,  367,   -1,  369,   -1,   -1,   -1,   -1,   -1,  258,   -1,  260,   -1,
   -1,   -1,   -1,   -1,  266,  267,  268,  269,   -1,   -1,   -1,   -1,   -1,  275,   -1,   -1,   -1,  279,  280,   -1,  282,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  290,  513,  292,  293,  294,  295,  296,   -1,  298,  299,  300,  301,   -1,  303,  304,   -1,  306,  307,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  321,  322,  323,  324,   -1,   -1,  327,   -1,   -1,   -1,   -1,   -1,   -1,  334,  335,   -1,
   -1,  338,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  349,   -1,  351,  352,  353,   -1,   -1,   -1,  371,   -1,   -1,   -1,   -1,
   -1,   -1,  364,   -1,   -1,  367,   -1,  369,   -1,  448,  449,   -1,  451,  452,  453,  454,  455,  456,  457,  458,  459,  460,   -1,   -1,  503,
  504,  505,  466,  467,  468,  469,  470,  408,   -1,   -1,  474,  475,  413,  477,  478,   -1,   -1,   -1,  419,   -1,   -1,  485,   -1,   -1,  488,
  489,  490,  491,  492,  493,  494,  495,  496,  497,  498,  499,  500,   -1,   -1,   -1,   -1,   -1,   -1,  444,  445,   -1,   -1,  448,  449,  450,
  451,  452,  453,  454,  455,  456,  457,  458,  459,  460,   -1,  462,  463,  464,  465,  466,  467,  468,  469,  470,  471,  472,  473,  474,  475,
  476,  477,  478,  479,  480,   -1,   -1,   -1,  484,   -1,   -1,   -1,  258,   -1,  260,   -1,   -1,   -1,   -1,   -1,   -1,  267,  268,   -1,   -1,
  501,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  503,  504,  505,  290,   -1,  292,  293,  294,  295,
  296,   -1,  298,  299,  300,  301,   -1,  303,  304,   -1,  306,  307,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
  321,  322,  323,  324,   -1,   -1,  327,   -1,   -1,   -1,   -1,   -1,   -1,  334,  335,   -1,   -1,  338,   -1,  340,   -1,   -1,   -1,   -1,  258,
   -1,  260,   -1,  349,   -1,  351,  352,  353,  267,  268,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  364,   -1,   -1,  367,   -1,  369,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,  290,   -1,  292,  293,  294,  295,  296,   -1,  298,  299,  300,  301,   -1,  303,  304,   -1,  306,  307,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  321,  322,  323,  324,   -1,   -1,  327,   -1,   -1,   -1,   -1,   -1,   -1,
  334,  335,   -1,   -1,  338,   -1,  340,   -1,   -1,   -1,   -1,  258,   -1,  260,   -1,  349,   -1,  351,  352,  353,  267,  268,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  364,   -1,   -1,  367,   -1,  369,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,   -1,  292,  293,  294,  295,  296,
   -1,  298,  299,  300,  301,   -1,  303,  304,   -1,  306,  307,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  321,
  322,  323,  324,   -1,   -1,  327,   -1,   -1,   -1,   -1,   -1,   -1,  334,  335,   -1,   -1,  338,   -1,  340,   -1,   -1,   -1,   -1,  258,   -1,
  260,   -1,  349,   -1,  351,  352,  353,  267,  268,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  364,   -1,   -1,  367,   -1,  369,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,  290,   -1,  292,  293,  294,  295,  296,   -1,  298,  299,  300,  301,   -1,  303,  304,   -1,  306,  307,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  321,  322,  323,  324,   -1,   -1,  327,   -1,   -1,   -1,   -1,   -1,   -1,  334,
  335,   -1,   -1,  338,   -1,  340,   -1,   -1,   -1,   -1,  258,   -1,  260,   -1,  349,   -1,  351,  352,  353,  267,  268,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  364,   -1,   -1,  367,   -1,  369,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  290,   -1,  292,  293,  294,  295,  296,   -1,
  298,  299,  300,  301,   -1,  303,  304,   -1,  306,  307,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  321,  322,
  323,  324,   -1,   -1,  327,   -1,   -1,   -1,   -1,   -1,   -1,  334,  335,   -1,   -1,  338,   -1,  340,   -1,   -1,   -1,   -1,  258,   -1,  260,
   -1,  349,   -1,  351,  352,  353,  267,  268,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  364,   -1,   -1,  367,   -1,  369,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,  290,   -1,  292,  293,  294,  295,  296,   -1,  298,  299,  300,  301,   -1,  303,  304,   -1,  306,  307,   -1,   -1,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  321,  322,  323,  324,   -1,   -1,  327,   -1,   -1,   -1,   -1,   -1,   -1,  334,  335,
   -1,   -1,  338,   -1,  340,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  349,   -1,  351,  352,  353,  310,   -1,   -1,   -1,   -1,  315,   -1,
   -1,  318,  319,  364,  321,  322,  367,   -1,  369,   -1,   -1,  328,  329,   -1,   -1,   -1,   -1,  334,  335,   -1,   -1,   -1,   -1,   -1,  341,
   -1,  343,  344,  345,   -1,   -1,   -1,   -1,   -1,   -1,  352,  353,   -1,   -1,   -1,   -1,   -1,   -1,   -1,  361,  362,   -1,   -1,   -1,   -1,
   -1,  368,
};

#line 2417 "D:\Programming\mcs-ilasm\ilasm\parser\ILParser.jay"

}

#line default
	// %token constants
	class Token
	{
  public const int EOF = 257;
  public const int ID = 258;
  public const int QSTRING = 259;
  public const int SQSTRING = 260;
  public const int COMP_NAME = 261;
  public const int INT32 = 262;
  public const int INT64 = 263;
  public const int FLOAT64 = 264;
  public const int HEXBYTE = 265;
  public const int DOT = 266;
  public const int OPEN_BRACE = 267;
  public const int CLOSE_BRACE = 268;
  public const int OPEN_BRACKET = 269;
  public const int CLOSE_BRACKET = 270;
  public const int OPEN_PARENS = 271;
  public const int CLOSE_PARENS = 272;
  public const int COMMA = 273;
  public const int COLON = 274;
  public const int DOUBLE_COLON = 275;
  public const int SEMICOLON = 277;
  public const int ASSIGN = 278;
  public const int STAR = 279;
  public const int AMPERSAND = 280;
  public const int PLUS = 281;
  public const int SLASH = 282;
  public const int BANG = 283;
  public const int ELLIPSIS = 284;
  public const int DASH = 286;
  public const int OPEN_ANGLE_BRACKET = 287;
  public const int CLOSE_ANGLE_BRACKET = 288;
  public const int UNKNOWN = 289;
  public const int INSTR_NONE = 290;
  public const int INSTR_VAR = 291;
  public const int INSTR_I = 292;
  public const int INSTR_I8 = 293;
  public const int INSTR_R = 294;
  public const int INSTR_BRTARGET = 295;
  public const int INSTR_METHOD = 296;
  public const int INSTR_NEWOBJ = 297;
  public const int INSTR_FIELD = 298;
  public const int INSTR_TYPE = 299;
  public const int INSTR_STRING = 300;
  public const int INSTR_SIG = 301;
  public const int INSTR_RVA = 302;
  public const int INSTR_TOK = 303;
  public const int INSTR_SWITCH = 304;
  public const int INSTR_PHI = 305;
  public const int INSTR_LOCAL = 306;
  public const int INSTR_PARAM = 307;
  public const int D_ADDON = 308;
  public const int D_ALGORITHM = 309;
  public const int D_ASSEMBLY = 310;
  public const int D_BACKING = 311;
  public const int D_BLOB = 312;
  public const int D_CAPABILITY = 313;
  public const int D_CCTOR = 314;
  public const int D_CLASS = 315;
  public const int D_COMTYPE = 316;
  public const int D_CONFIG = 317;
  public const int D_IMAGEBASE = 318;
  public const int D_CORFLAGS = 319;
  public const int D_CTOR = 320;
  public const int D_CUSTOM = 321;
  public const int D_DATA = 322;
  public const int D_EMITBYTE = 323;
  public const int D_ENTRYPOINT = 324;
  public const int D_EVENT = 325;
  public const int D_EXELOC = 326;
  public const int D_EXPORT = 327;
  public const int D_FIELD = 328;
  public const int D_FILE = 329;
  public const int D_FIRE = 330;
  public const int D_GET = 331;
  public const int D_HASH = 332;
  public const int D_IMPLICITCOM = 333;
  public const int D_LANGUAGE = 334;
  public const int D_LINE = 335;
  public const int D_XLINE = 336;
  public const int D_LOCALE = 337;
  public const int D_LOCALS = 338;
  public const int D_MANIFESTRES = 339;
  public const int D_MAXSTACK = 340;
  public const int D_METHOD = 341;
  public const int D_MIME = 342;
  public const int D_MODULE = 343;
  public const int D_MRESOURCE = 344;
  public const int D_NAMESPACE = 345;
  public const int D_ORIGINATOR = 346;
  public const int D_OS = 347;
  public const int D_OTHER = 348;
  public const int D_OVERRIDE = 349;
  public const int D_PACK = 350;
  public const int D_PARAM = 351;
  public const int D_PERMISSION = 352;
  public const int D_PERMISSIONSET = 353;
  public const int D_PROCESSOR = 354;
  public const int D_PROPERTY = 355;
  public const int D_PUBLICKEY = 356;
  public const int D_PUBLICKEYTOKEN = 357;
  public const int D_REMOVEON = 358;
  public const int D_SET = 359;
  public const int D_SIZE = 360;
  public const int D_STACKRESERVE = 361;
  public const int D_SUBSYSTEM = 362;
  public const int D_TITLE = 363;
  public const int D_TRY = 364;
  public const int D_VER = 365;
  public const int D_VTABLE = 366;
  public const int D_VTENTRY = 367;
  public const int D_VTFIXUP = 368;
  public const int D_ZEROINIT = 369;
  public const int K_AT = 370;
  public const int K_AS = 371;
  public const int K_AGGRESSIVEINLINING = 372;
  public const int K_IMPLICITCOM = 373;
  public const int K_IMPLICITRES = 374;
  public const int K_NOAPPDOMAIN = 375;
  public const int K_NOPROCESS = 376;
  public const int K_NOMACHINE = 377;
  public const int K_EXTERN = 378;
  public const int K_INSTANCE = 379;
  public const int K_EXPLICIT = 380;
  public const int K_DEFAULT = 381;
  public const int K_VARARG = 382;
  public const int K_UNMANAGED = 383;
  public const int K_CDECL = 384;
  public const int K_STDCALL = 385;
  public const int K_THISCALL = 386;
  public const int K_FASTCALL = 387;
  public const int K_MARSHAL = 388;
  public const int K_IN = 389;
  public const int K_OUT = 390;
  public const int K_OPT = 391;
  public const int K_STATIC = 392;
  public const int K_PUBLIC = 393;
  public const int K_PRIVATE = 394;
  public const int K_FAMILY = 395;
  public const int K_INITONLY = 396;
  public const int K_RTSPECIALNAME = 397;
  public const int K_STRICT = 398;
  public const int K_SPECIALNAME = 399;
  public const int K_ASSEMBLY = 400;
  public const int K_FAMANDASSEM = 401;
  public const int K_FAMORASSEM = 402;
  public const int K_PRIVATESCOPE = 403;
  public const int K_LITERAL = 404;
  public const int K_NOTSERIALIZED = 405;
  public const int K_VALUE = 406;
  public const int K_NOT_IN_GC_HEAP = 407;
  public const int K_INTERFACE = 408;
  public const int K_SEALED = 409;
  public const int K_ABSTRACT = 410;
  public const int K_AUTO = 411;
  public const int K_SEQUENTIAL = 412;
  public const int K_ANSI = 413;
  public const int K_UNICODE = 414;
  public const int K_AUTOCHAR = 415;
  public const int K_BESTFIT = 416;
  public const int K_IMPORT = 417;
  public const int K_SERIALIZABLE = 418;
  public const int K_NESTED = 419;
  public const int K_LATEINIT = 420;
  public const int K_EXTENDS = 421;
  public const int K_IMPLEMENTS = 422;
  public const int K_FINAL = 423;
  public const int K_VIRTUAL = 424;
  public const int K_HIDEBYSIG = 425;
  public const int K_NEWSLOT = 426;
  public const int K_UNMANAGEDEXP = 427;
  public const int K_PINVOKEIMPL = 428;
  public const int K_NOMANGLE = 429;
  public const int K_OLE = 430;
  public const int K_LASTERR = 431;
  public const int K_WINAPI = 432;
  public const int K_NATIVE = 433;
  public const int K_IL = 434;
  public const int K_CIL = 435;
  public const int K_OPTIL = 436;
  public const int K_MANAGED = 437;
  public const int K_FORWARDREF = 438;
  public const int K_RUNTIME = 439;
  public const int K_INTERNALCALL = 440;
  public const int K_SYNCHRONIZED = 441;
  public const int K_NOINLINING = 442;
  public const int K_NOOPTIMIZATION = 443;
  public const int K_CUSTOM = 444;
  public const int K_FIXED = 445;
  public const int K_SYSSTRING = 446;
  public const int K_ARRAY = 447;
  public const int K_VARIANT = 448;
  public const int K_CURRENCY = 449;
  public const int K_SYSCHAR = 450;
  public const int K_VOID = 451;
  public const int K_BOOL = 452;
  public const int K_INT8 = 453;
  public const int K_INT16 = 454;
  public const int K_INT32 = 455;
  public const int K_INT64 = 456;
  public const int K_FLOAT32 = 457;
  public const int K_FLOAT64 = 458;
  public const int K_ERROR = 459;
  public const int K_UNSIGNED = 460;
  public const int K_UINT = 461;
  public const int K_UINT8 = 462;
  public const int K_UINT16 = 463;
  public const int K_UINT32 = 464;
  public const int K_UINT64 = 465;
  public const int K_DECIMAL = 466;
  public const int K_DATE = 467;
  public const int K_BSTR = 468;
  public const int K_LPSTR = 469;
  public const int K_LPWSTR = 470;
  public const int K_LPTSTR = 471;
  public const int K_VBBYREFSTR = 472;
  public const int K_OBJECTREF = 473;
  public const int K_IUNKNOWN = 474;
  public const int K_IDISPATCH = 475;
  public const int K_STRUCT = 476;
  public const int K_SAFEARRAY = 477;
  public const int K_INT = 478;
  public const int K_BYVALSTR = 479;
  public const int K_TBSTR = 480;
  public const int K_LPVOID = 481;
  public const int K_ANY = 482;
  public const int K_FLOAT = 483;
  public const int K_LPSTRUCT = 484;
  public const int K_NULL = 485;
  public const int K_PTR = 486;
  public const int K_VECTOR = 487;
  public const int K_HRESULT = 488;
  public const int K_CARRAY = 489;
  public const int K_USERDEFINED = 490;
  public const int K_RECORD = 491;
  public const int K_FILETIME = 492;
  public const int K_BLOB = 493;
  public const int K_STREAM = 494;
  public const int K_STORAGE = 495;
  public const int K_STREAMED_OBJECT = 496;
  public const int K_STORED_OBJECT = 497;
  public const int K_BLOB_OBJECT = 498;
  public const int K_CF = 499;
  public const int K_CLSID = 500;
  public const int K_METHOD = 501;
  public const int K_CLASS = 502;
  public const int K_PINNED = 503;
  public const int K_MODREQ = 504;
  public const int K_MODOPT = 505;
  public const int K_TYPEDREF = 506;
  public const int K_TYPE = 507;
  public const int K_WCHAR = 508;
  public const int K_CHAR = 509;
  public const int K_FROMUNMANAGED = 510;
  public const int K_CALLMOSTDERIVED = 511;
  public const int K_BYTEARRAY = 512;
  public const int K_WITH = 513;
  public const int K_INIT = 514;
  public const int K_TO = 515;
  public const int K_CATCH = 516;
  public const int K_FILTER = 517;
  public const int K_FINALLY = 518;
  public const int K_FAULT = 519;
  public const int K_HANDLER = 520;
  public const int K_TLS = 521;
  public const int K_FIELD = 522;
  public const int K_PROPERTY = 523;
  public const int K_REQUEST = 524;
  public const int K_DEMAND = 525;
  public const int K_ASSERT = 526;
  public const int K_DENY = 527;
  public const int K_PERMITONLY = 528;
  public const int K_LINKCHECK = 529;
  public const int K_INHERITCHECK = 530;
  public const int K_REQMIN = 531;
  public const int K_REQOPT = 532;
  public const int K_REQREFUSE = 533;
  public const int K_PREJITGRANT = 534;
  public const int K_PREJITDENY = 535;
  public const int K_NONCASDEMAND = 536;
  public const int K_NONCASLINKDEMAND = 537;
  public const int K_NONCASINHERITANCE = 538;
  public const int K_READONLY = 539;
  public const int K_NOMETADATA = 540;
  public const int K_ALGORITHM = 541;
  public const int K_FULLORIGIN = 542;
  public const int K_ENABLEJITTRACKING = 543;
  public const int K_DISABLEJITOPTIMIZER = 544;
  public const int K_RETARGETABLE = 545;
  public const int K_PRESERVESIG = 546;
  public const int K_BEFOREFIELDINIT = 547;
  public const int K_ALIGNMENT = 548;
  public const int K_NULLREF = 549;
  public const int K_VALUETYPE = 550;
  public const int K_COMPILERCONTROLLED = 551;
  public const int K_REQSECOBJ = 552;
  public const int K_ENUM = 553;
  public const int K_OBJECT = 554;
  public const int K_STRING = 555;
  public const int K_TRUE = 556;
  public const int K_FALSE = 557;
  public const int K_IS = 558;
  public const int K_ON = 559;
  public const int K_OFF = 560;
  public const int K_FORWARDER = 561;
  public const int K_CHARMAPERROR = 562;
  public const int K_LEGACY = 563;
  public const int K_LIBRARY = 564;
  public const int yyErrorCode = 256;
	};

	namespace yydebug
	{
		using System;

		internal interface yyDebug
		{
			void shift(int from, int to, int errorFlag);
			void lex(int state, int token, string name, Object value);
			void push(int state, Object value);
			void pop(int state);
			void discard(int state, int token, string name, Object value);
			void reduce(int from, int to, int rule, string text, int len);
			void shift(int from, int to);
			void accept(Object value);
			void error(string message);
			void reject();
		};

		sealed class yyDebugSimple : yyDebug
		{
			void write(String msg) => System.Diagnostics.Trace.WriteLine(msg);

			public void push(int state, Object value) => write("push\tstate " + state + "\tvalue " + value);

			public void pop(int state) => write("pop\tstate " + state + "\ton error");

			public void lex(int state, int token, string name, Object value) => 
								write("lex\tstate " + state + "\treading " + name + "\tvalue " + value);

			public void discard(int state, int token, string name, Object value) => 
								write("discard\tstate " + state + "\ttoken " + name + "\tvalue " + value);

			public void reduce(int from, int to, int rule, string text, int len) => 
								write("reduce\tstate " + from + "\tuncover " + to + "\trule (" + rule + ") " + text);

			public void shift(int from, int to, int errorFlag)
			{
				switch (errorFlag)
				{
					default: // normally
						write("shift\tfrom state " + from + " to " + to);
						break;
					case 0:
					case 1:
					case 2: // in error recovery
						write("shift\tfrom state " + from + " to " + to + "\t" + errorFlag + " left to recover");
						break;
					case 3: // normally
						write("shift\tfrom state " + from + " to " + to + "\ton error");
						break;
				}
			}

			public void shift(int from, int to) => write("goto\tfrom state " + from + " to " + to);

			public void accept(Object value) => write("accept\tvalue " + value);

			public void error(string message) => write("error\t" + message);

			public void reject() => write("reject");
		};
	}

	namespace yyParser
	{
		using System;

		/// <summary> thrown for irrecoverable syntax errors and stack overflow. </summary>
		internal class yyException : System.Exception
		{
			public yyException(string message) :
				base(message)
			{
			}
		};

		internal class yyUnexpectedEof : yyException
		{
			public yyUnexpectedEof(string message = null)
				: base(message ?? "")
			{
			}
		};

		/// <summary> Implemented by a scanner object to supply input to the parser. </summary>
		internal interface yyInput
		{
			bool advance();
			int token();
			Object value();
		};
	}
}
